/*
 A variable type for ACPLT/KS that is coupled to an underlying cell in 
 VISUAL's spreadsheet.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2002.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#include "ks/simpleserver.h"

#include "cellList.h"
#include "cellVar.h"
#include "plclog.h"

#include "lib0.h"
#include "cellVar.h"

#define ThisModule "ACPLT/KS : "
extern int rfd;

    CellVariable::CellVariable(int sheet, coord c):KssSimpleVariable(
        cellN(c), KsTime::now(), symbolName(1,c,symOnly).c_str()){
	this->sheet=sheet;
	this->c=c;
	cell * ce=sheets[sheet]->find(c);
	if (ce->isFormula()) lock();
    }
    
    KsValueHandle CellVariable::getValue() const {
	    KsValue *pVal = 0;
	    cell * ce=sheets[sheet]->find(c);
	    if (ce->isNumber() || ce->isFormula()) {
		pVal = new KsDoubleValue(ce->getVal());
	    }
	    else	
		pVal = new KsStringValue(ce->stringContents(0,0).c_str());
	    return KsValueHandle(pVal, KsOsNew);
    }
    
    
    KS_RESULT CellVariable::setValue(const KsValueHandle &h) {
        LOG1("ACPLT/KS setValue(h)\n");
	    
        KsValue * tmp = h.getPtr();
        KsDoubleValue * pVal_x = 
	    PLT_DYNAMIC_PCAST(KsDoubleValue, tmp);
        if (pVal_x) {
	    double ddd=*pVal_x;
	    LOG2("%g\n",ddd);
	    setCell(sheet,c,ddd,soExternal);
	    return KS_ERR_OK;
	}
	return KS_ERR_OK;
    }
    
    KsTime CellVariable::getTime() const
    {
	cell * ce=sheets[sheet]->find(c);
	if (ce->typeID() == tnTimeCell) {
	    return ((nTimeCell *) ce)->time;
	} else {
	    if (ce->typeID() == tfTimeCell) {
		return ((fTimeCell *) ce)->time;
	    }	
	}    
	return 0;
    }


/*
    Make a SimpleVar from each cell in sheet. The prototype fits what cellList.forEach expects:
*/
void makeSimpleVar(void *p, void * d)
{
    simpleVarData * svd=(simpleVarData*)d;
    KssSimpleDomain * dom=(KssSimpleDomain*)(svd->d);
    cell *c = (cell *) p;
    CellVariable *var =  new CellVariable(svd->sheet,c->k);
    var->setState(KS_ST_GOOD);
    dom->addChild(var);
}
/*
    Changes:
    
    12/01/2003  put in a seperate file to speed up compilation
    12/01/2003  write support for numeric values
    03/31/2004  changed sequence of headers (for gcc-3.3.3)
    11/10/2004  replace "../drivers/plclog" by "plclog"
*/
