/*
 Implementation of a cyclic recalculatin thread for the spreadsheet.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <pthread.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include "cellList.h"
#include "globals.h"

#include "vi_string.h"

#include <stdio.h>
#include "plclog.h"

#define scheduleResolution = 20000  // resolution in microseconds

void *calcThread(void *arg)
{
    timeval t1, t2;
    double usec;
    int delayTime;
    int i;
    time_t t;
    tm * tmc;
    LOG1( "starting\n");
/*
    Find the minimum delay time. 
*/    
    int minDelay = 100000000;
    int maxDelay = 1;
    for (i = 1; i <= maxSheet; i++) {
	LOG2( "check interval of %d\n",i);
	if (sheets[i] != NULL) {
	    if (minDelay > sheets[i]->calcIntervall) {
		minDelay = sheets[i]->calcIntervall;
	    }
	    if (maxDelay < sheets[i]->calcIntervall) {
		maxDelay = sheets[i]->calcIntervall;
	    }
	} else break;
    }
    int factor=maxDelay / minDelay;
    LOG4( "got min:%d max:%d factor:%d\n", minDelay, maxDelay, factor);
    for (i = 1; i <= maxSheet; i++) {
	LOG2("calc divisor of %d\n",i);
	if (sheets[i] != NULL) {
	    int x=sheets[i]->calcIntervall;
	    int divisor=x/minDelay;
	    if (divisor<1) divisor=1;
	    LOG3("best divisor for %d is %d\n",i,divisor);
	    sheets[i]->calcDivisor=divisor;
	    sheets[i]->doRecalcCounter=1;
	} else break;
    }
    
/*
    Give some time to other initialization processes.
*/    
    usleep(100000);

    do {
//	gettimeofday(&t1, NULL);
//	time_t tt1;
	gettimeofday(&t1, NULL);
	tmc=localtime(&t1.tv_sec);
	glSeconds = tmc->tm_sec;
	glMinutes = tmc->tm_min;    
	glHours = tmc->tm_hour;
	glDay = tmc->tm_mday;
	glMonth = 1+tmc->tm_mon;
	glYear = tmc->tm_year+1900;
	glDayOfWeek = tmc->tm_wday;
	glUSeconds=t1.tv_usec;
/*
	Recalculate all sheets and measure elapsed time
*/
//	gettimeofday(&t1, NULL);
	for (i = 1; i <= maxSheet; i++) {
	    if (sheets[i] != NULL) {
		sheets[i]->doRecalcCounter--;
		if(sheets[i]->doRecalcCounter==0) {
		    LOG2("sheet %d is recalculated.\n",i);
		    sheets[i]->recalc(i);
		    sheets[i]->doRecalcCounter=sheets[i]->calcDivisor;
		}    
		FLUSH;
	    }
	}
	gettimeofday(&t2, NULL);
	
	usec = 1e6 * (t2.tv_sec - t1.tv_sec) + t2.tv_usec - t1.tv_usec;
	time(&t);
	LOG4( "recalc took %ld usecs. %ld %s", t2.tv_usec -
	       t1.tv_usec, t2.tv_usec, ctime(&t));
	FLUSH;
	delayTime = int (minDelay - usec);
	glCPULoad = usec;
	if (delayTime > 00)
	    usleep(delayTime);
    }
    while (1);
}

/*
    Changes:
    
    05/13/2002	replaced cout<<... by printf(...
    05/13/2002	Each sheet can have it's own calcIntervall. Here, the minimum is
		used for all. That's provisional.
    10/13/2002	replaced printf(... by logx(... where x is the number of parameters. Log output
		is thus printed to what is defined in log.h;
    02/20/2003	fixed date/time variables
    07/16/2002  added USeconds system var
    08/16/2005  tried to implement different recalculation times per sheet
    
*/
