/*
 Implementation of cells for the spreadsheet.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include "mytypes.h"
#include "cell.h"
#include "lib0.h"
#include "pString.h"
#include "parser.h"
#include <stdio.h>
#include "plclog.h"
#define debug 0
#define ThisModule "cell     : "

    coordW::~coordW(){
//	LOG1("~coordW");
    }

    coordW::coordW(coord ak){
	k=ak;
/*	
	waste=calloc(10000000,1);
	((char *)waste)[44444]='A';
	((char *)waste)[6444]='A';
	((char *)waste)[54444]='A';
	((char *)waste)[44]='A';
	((char *)waste)[444444]='A';
	((char *)waste)[4444444]='A';
	((char *)waste)[434444]='A';
	((char *)waste)[4344444]='Q';
	LOG2("waste: %p\n",waste);
*/	
    }


cell::cell(word r, word c, double v, double pv):flags(valChangedAll),
cflags(0), state(0)
{
#if debug > 0
    LOG1(ThisModule "this is cell::cell() \n");
    FLUSH;
#endif
    k.row = r;
    k.col = c;
    value = v;
    prevalue = pv;
}

int
 cell::compare(cell * c)
{
    int a, b, d;
    a = k.l;
    b = c->k.l;
    if (a > b)
	d = 1;
    else if (a < b)
	d = -1;
    else
	d = 0;
    return (d);
}

int cell::compare(coord c)
{
    int a, b, d;
    a = k.l;
    b = c.l;
    if (a > b)
	d = 1;
    else if (a < b)
	d = -1;
    else
	d = 0;
    return (d);
}


char * cell::cStringContents(int, int, char * buf) {
    strcpy(buf,"cell");
    return buf+4;
}

void cell::Write(Engine & e) const
{
#if debug > 4
    LOG1(ThisModule "cell::Write\n");
    LOG2(ThisModule "%s\n", cellN(k));
    FLUSH;
#endif
    e << cellName(k) << value << prevalue << Label;
}

void cell::Read(Engine & e)
{
    string s;
    e >> s;
#if debug > 0
    LOG2(ThisModule "cell::Read:%s\n", s.c_str());
    FLUSH;
#endif
    k = getPos(s);
    e >> value >> prevalue >> Label;
}


IMPLEMENT_PERSISTENCE(cell, "CELL")

    nCell::nCell()
{
#if debug > 0
    LOG1(ThisModule "this is nCell::nCell\n");
    FLUSH;
#endif
    sourceType = soDirect;
    nextNum = NULL;
}

string nCell::stringContents(int len, int decimals)
{
    return (formatDouble(value, 0, decimals));
}

char * nCell::cStringContents(int, int, char * buf) {
    buf+=sprintf(buf,"%.12g",value);
    return buf;
}

string nCell::getViewerText(int decimals, bool preferText) {
    return (formatDouble(value, 0, decimals));
}

IMPLEMENT_PERSISTENCE(nCell, "NCELL")

void nCell::Write(Engine & e) const
{
    cell::Write(e);
    int i = sourceType;
    e << i;
}

void nCell::Read(Engine & e)
{
    cell::Read(e);
    int i;
    e >> i;
    sourceType = i;
    //
    // If we load data that normally comes from an external source,
    // load the last value, hoping its ok for calculations.
    // But it is not up to date. We mark it to avoid recording this
    // value in trends, alarms, protocolls printing etc..
    //
    if (sourceType == soExternal)
	state |= stDirty;
}


nAvrCell::nAvrCell()
{
#if debug > 0
    LOG1(ThisModule "this is nAvrCell::nAvrCell()\n");
    FLUSH;
#endif
}

IMPLEMENT_PERSISTENCE(nAvrCell, "NAVRCELL")

    nTimeCell::nTimeCell()
{
    time =::time(NULL);
#if debug > 0
    LOG2(ThisModule "this is nTimeCell::nTimeCell() %ld\n", time);
    FLUSH;
#endif
}

IMPLEMENT_PERSISTENCE(nTimeCell, "NTIMECELL")

    tCell::tCell()
{
#if debug > 0
    LOG2(ThisModule "tCell::tCell %s\n", text.c_str());
    FLUSH;
#endif
}

char * tCell::cStringContents(int, int, char * buf) {
    strcpy(buf,text.c_str());
    return buf+text.length();
}

IMPLEMENT_PERSISTENCE(tCell, "TCELL")
void tCell::Write(Engine & e) const
{
    cell::Write(e);
    e << text;
}

void tCell::Read(Engine & e)
{
    cell::Read(e);
    e >> text;
}

string tCell::getViewerText(int decimals, bool preferText) {
    return text;
}

pString& ps() {
    pString * p= new pString();
    return *p;
}


fCell::fCell():TPN(ps())
{
    mathError = 0;
//    TPN = pString();
#if debug > 0
    LOG2(ThisModule "this is fCell::fCell tpn.len: %d\n", TPN.length());
    FLUSH;
#endif
}

//fCell::fCell(string &s):TPN(parse(s))
fCell::fCell(string &s):TPN(*(new pString()))
{
    text=s;
    mathError = 0;
    TPN = parse(s,TPN);
//    TPN = pString();
#if debug > 0
    LOG2(ThisModule "this is fCell::fCell tpn.len: %d\n", TPN.length());
    FLUSH;
#endif
}


string fCell::stringContentsNum(int len, int decimals)
{
    return (formatDouble(value, 0, decimals));
}

char * fCell::cStringContents(int, int, char * buf) {
    *buf='=';buf++;
    strcpy(buf,text.c_str());
    return buf+text.length()+1;
}

string fCell::getViewerText(int decimals, bool preferText) {
    if (mathError==ceNone) {
    if (preferText) {
	return text;
    } else	
	return (formatDouble(value, 0, decimals));
    } else
	return ("Error");	
}

IMPLEMENT_PERSISTENCE(fCell, "FCELL")
void fCell::Read(Engine & e)
{
    tCell::Read(e);
//    pString& old=TPN;
///    delete &TPN;
///    TPN = parse(text);
    TPN = parse(text,TPN);
//    delete &old;
}


fTimeCell::fTimeCell()
{
    time =::time(NULL);
#if debug > 0
    LOG2(ThisModule "this is fTimeCell::fTimeCell() %ld\n", time);
    FLUSH;
#endif
}

char * fTimeCell::cStringContents(int, int, char * buf) {
    *buf='*';buf++;
    *buf='=';buf++;
    strcpy(buf,text.c_str());
    return buf+text.length()+2;
}

IMPLEMENT_PERSISTENCE(fTimeCell, "FTIMECELL")
/*
    Changes:
    
    05/13/2002 Object Read & write functions return void now. cout<<... replaced
		by printf(..
	
    05/25/2002 reordered initialization of cell to avoid warnings from gcc.
    10/13/2002 replaced printf(... by LOGx(... where x is the number of parameters. Log output
		is thus printed to what is defined in log.h;
    02/26/2003 introduced the cStringContents functions. Hope they are faster.
    04/09/2003 going towards OO.
    05/20/2003 return "Error" in fCell::getViewerText.
*/
