/*
 Definitions of cells for the spreadsheet.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#ifndef __cell
#define __cell

#include "mytypes.h"
#include "pString.h"
#include "persist.h"

#include "vi_string.h"
#include "plclog.h"

/*
    type identifiers for cell types:
*/
//#define tCell 0
#define tnCell 6501
#define tnAvrCell 6502
#define tnTimeCell 6503
#define ttCell 6504
#define tfCell 6505
#define tfTimeCell 6506
#define tRefCell 6507
/*
    source types for numerical values:
*/
#define soDirect 0		// Cell value is direct input via spread sheet user
			// interface.
#define soExternal 1		// value from external device via serial or network i/o
#define soExternal2 2		// not used
#define soSetByHMI 3		// cell value stems from an element in HMI pictures.
			// Forms part of current machine settings and has to be
			// restored on program restart.
#define	soAcknowledge 4		// not used
#define soRecipe 5		// cell value comes from a recipe. Treat like soHMIsetting.
#define soBasicVar 6		// cell serves as a BASIC variable
#define soPushButton 7		// Cell value is set by a pushbutton in the HMI. Special
			// treatment: The driver sends it to the PLC and
			// then sets its value to 0. In the next cycle, it 
			// sends the 0 to the PLC, thus emulating a button that
			// is pressed and released again.
#define soPushBAck 8		// value is set to zero by a PLC driver after sending
			// the original value.
#define soIoWorkSpace 9		// cell serves as a variable for the general purpose
			// serial driver.
#define soIoState 10		// value represents a communication error state.
#define soUDFmodule 11	// value iscalculated by a user defined module.


/*
    state bits:
*/
#define stDirty 0x8		// numerical value is not up to date (loaded from disk etc.)
#define stCalcError 0x4		// numerical value results from erroneous calculation
#define stNetError 0x2		// numerical value is invalid because of network error 
#define stMissing 0x1		// formula used nonexisting cell 
/*
    flag bits:
*/
/*
#define  valChanged1 0x8000	// value changed to driver at ttyS0 / COM1
#define  valChanged2 0x4000	// value changed to driver at ttyS1 / COM2
#define  valChanged3 0x2000	// value changed to driver at ttyS2 / COM3
#define  valChanged4 0x1000	// value changed to driver at ttyS3 / COM4
#define  valChanged5 0x0800	// value changed to driver at ttyS4 / COM5
#define  valChanged6 0x0400	// value changed to driver at ttyS5 / COM6
#define  valChanged7 0x0200	// value changed to driver at ttyS6 / COM7
#define  valChanged8 0x0100	// value changed to driver at ttyS7 / COM8
#define  valChangedP 0x0080	// value changed to grahics display
#define  valChangedD 0x0040	// value changed to data recorder
#define  valChangedA 0x0020	// value changed for average calculation
#define  valChangedS 0x0010	// value changed for alarm/message recorder
#define  valChanged0 0x0008	// value changed for network communication
*/

#define  valChangedC 0x0004	// value changed for client modules
#define  flSelfRef   0x0002	// this cell contains a formula that refers to itself
#define  flMarked    0x0001	// this cell is marked (user interface)

#define valChangedAll 0xFFFFFFFC
/*
 valChanged1 | valChanged2 | valChanged3\
| valChanged4 = 0x1000\
| valChanged5 = 0x0800\
| valChanged6 = 0x0400 \
| valChanged7 = 0x0200\
| valChanged8 = 0x0100\
| valChangedP = 0x0080\
| valChangedD = 0x0040\
| valChangedA = 0x0020\
| valChangedS = 0x0010\
| valChangedC = 0x0008\
| valChanged0 = 0x0004
*/
/*
    possible calculation errors:
*/
#define ceNone 0		// no error
#define ceDiv0 1		// Division by 0
#define ceLogNeg 2		// logarithm from negative number
#define ceWrongType 3		// a cell used in the formula is not numerical or
			// does not track time, if required
#define ceMissing 4		// a cell used in the formula does'nt exist
#define ceRootNeg 5		// root from negative number
#define ceSucc 6		// a cell used in the formula is in a state of error
#define ceMissingSheet 7	// the sheet from which the formula imports a cell
			// does'nt exist

class coordW:public base {
//friend class cellList;	// no longer 04/09/03
  public:
    coord k;			// position in spreadsheet 
  coordW(){}; 
  coordW(coord ak);        
  ~coordW();        
};

class cell:public coordW {
//friend class cellList;	// no longer 04/09/03
  public:
//    coord k;			// position in spreadsheet 
protected:    
    double value;		// actual value 
    double prevalue;		// value from previous calculation
    unsigned int flags;		// bits mark "changed" condition for some purposes
    unsigned int cflags;			// bits mark subscription by client process 
    unsigned int state;			// error conditions
public:    
    string Label;
    string Source;			// to be filled by PLC driver.
  public:
     DECLARE_PERSISTENCE(cell) 
    virtual void Write(Engine & e) const;
    virtual void Read(Engine & e);
     cell(word r = 0, word c = 0, double v = 0, double pv = 0);
    int compare(cell * c);	// compare to coordinates of *c
    int compare(coord k);	// compare to coordinates k
    
    virtual string stringContents(int, int) {
	return ("cell");
    }
    virtual char * cStringContents(int, int, char * buf);
    virtual string stringContentsNum(int, int) {
	return ("cell");
    }
    virtual string typeString() {
	return ("cell");
    }
    virtual int getError() {
	return (0);
    }
    
    virtual void setError(int e) {}
    virtual int typeID() {
	return (0);
    }
    
    virtual double getVal() {
	return (value);
    }
    virtual double getPreVal() {
	return (prevalue);
    }

    virtual void setVal(double d) {
	value=d;
    }
    virtual void setPreVal(double d) {
	prevalue=d;
    }
    
    virtual int getSourceType() {
	return (-1);
    }
    virtual void setSourceType(int s) {}
    virtual bool isNumber() {return false;}
    virtual bool isFormula() {return false;}
    virtual string getViewerText(int decimals, bool preferText) {return "cell";}  // This is for extensibility
    virtual unsigned int getCflags() {
	return cflags;
    }
    virtual void setCflags(unsigned int c) {
	cflags=c;
    }
    virtual unsigned int getFlags() {
	return flags;
    }
    virtual void setFlags(unsigned int c) {
	flags=c;
    }
    
    virtual unsigned int getState() {
	return state;
    }
    virtual void setState(unsigned int c) {
	state=c;
    }
    virtual string getSource() {
	return (Source);
    }
    
    virtual void setSource(string s) {
	Source=s;
    }    
    ~cell()
    {
	LOG1("~cell");
    }

};

class nCell:public cell {	// This is a numerical cell. Simply
    // stores a real number.
  protected:
    char sourceType;		// type of origin of data    
  public:
    nCell();
    DECLARE_PERSISTENCE(nCell) 
    virtual void Write(Engine & e) const;
    virtual void Read(Engine & e);

    nCell *nextNum;		// a pointer to build a linked list
    virtual string typeST() {
	return ("NCELL");
    } 
    virtual int typeID() {
	return (tnCell);
    }
    virtual string stringContents(int, int);
    virtual string stringContentsNum(int w, int d) {
	return stringContents(w, d);
    };
    virtual char * cStringContents(int, int, char * buf);
    virtual string typeString() {
	return ("Number");
    }
    virtual int getSourceType() {
	return (sourceType);
    }
    virtual void setSourceType(int s) {
	sourceType=s;
    }
    virtual bool isNumber() {return true;}
    virtual string getViewerText(int decimals, bool preferText); 
};

class nAvrCell:public nCell {	// This is a numerical cell specialized in
    // calculating the average of another cell.
  public:
    DECLARE_PERSISTENCE(nAvrCell) nAvrCell();
    int avrTime;		// time to calculate average in ms.
    int count;			// number of values average is calculated from
    coord source;		// where the values come from
    virtual string typeST() {
	return ("NAVRCELL");
    } virtual int typeID() {
	return (tnAvrCell);
    }
};

class nTimeCell:public nCell {	// This is a numerical cell which tracks the
    // time of last change.
  public:
    DECLARE_PERSISTENCE(nTimeCell) // Makro from persist.h
    time_t time;	//linux timestamp of last change
    nTimeCell();
    virtual string typeST() {
	return ("NTIMECELL");
    } 
    virtual int typeID() {
	return (tnTimeCell);
    }
};

class tCell:public cell {	// This is a text cell. Simply stores a text.
  public:
    string text;
    DECLARE_PERSISTENCE(tCell) tCell();
    virtual void Write(Engine & e) const;
    virtual void Read(Engine & e);
    virtual string typeST() {
	return ("TCELL");
    } virtual int typeID() {
	return (ttCell);
    }
    virtual string stringContents(int, int) {
	return (text);
    }
    virtual char * cStringContents(int, int, char * buf);
    virtual string typeString() {
	return ("Text");
    }
    virtual string getViewerText(int decimals, bool preferText); 
};

class fCell:public tCell {	// This is a formula cell. It stores the original
    // input in text and the translated form in TPN.
  protected:    
    int mathError;		// error code from last calculation
    fCell();
  public:
    pString& TPN;		// the tokenized postfix notation
    ~fCell(){
	LOG1("~fCell()");
	delete &TPN;
    }
    fCell *nextFormula;		// a pointer to build a linked list
    
    fCell(string& s);
    
    DECLARE_PERSISTENCE(fCell) 
    
    virtual void Read(Engine & e);
    virtual string typeST() {
	return ("FCELL");
    }
    virtual int typeID() {
	return (tfCell);
    }
    virtual char * cStringContents(int, int, char * buf);
    virtual string stringContents(int, int) {
	return ("=" + text);
    }
    virtual int getError() {
	return (mathError);
    }
    virtual void setError(int e) {
	mathError=e;
    }
    
    virtual string typeString() {
	return ("Formula");
    }
    virtual string stringContentsNum(int w, int d);
    virtual bool isFormula() {return true;}
    virtual string getViewerText(int decimals, bool preferText); 
};

class fTimeCell:public fCell {	// This is a formula cell which tracks the
    // time of last change. 
  public:
    time_t time;		//linux timestamp of last change
    DECLARE_PERSISTENCE(fTimeCell)
    fTimeCell();
    fTimeCell(word r, word c);
    virtual string typeST() {
	return ("FTIMECELL");
    } 
    virtual int typeID() {
	return (tfTimeCell);
    }
    virtual string stringContents(int, int) {
	return ("*=" + text);
    }
    virtual char * cStringContents(int, int, char * buf);
};

#endif	/* __cell */

/*
    Changes:
    
    05/13/2002  Object Read & write functions return void now. cout<<... replaced
		by printf(..
    06/27/2002  Added method to retrieve sourceType.
    02/26/2003  introduced the cStringContents functions. Hope they are faster.
    04/09/2003  introduced the refCell type.
    07/05/2003  added string Source. should take information about data source provided 
		by PLC driver
*/
