/*
 cellList.cc: implements the basic structure of the spreadsheet.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include "cellList.h"
#include "lib0.h"
#include <limits.h>
#include "evalTPN.h"
#include "list.h"
#include "reflist.h"
#define debug 0
#define maxCollectionSize INT_MAX / sizeof(void *)

#include <stdio.h>
#include "plclog.h"

#define ThisModule "cellList : "

string cellList::symbolName(coord k, int mode)
{
    string s = "";
    if (mode != symOnly) {
	s = cellName(k);
    }
    cell * c = find(k);
    if (c != NULL) {
	if (!c->Label.empty()) {
	    s = c->Label;
	}
    }
    return (s);
}
int cellList::compare(void *key1, void *key2)
{
#if debug >3
    LOG2("compare:\n");
    FLUSH;
#endif
    return ((cell *) key1)->compare((cell *) key2);
}

cell *cellList::find(coord k)
{
#if debug >3
    LOG2("enter find: %s\n", cellN(k));
    FLUSH;
#endif
    int l = 0;
    int h = list::count - 1;
    while (l <= h) {
	int mid = (l + h) >> 1;
	int c = ((cell *) (items[mid]))->compare(k);
	if (c < 0)
	    l = mid + 1;
	else {
	    h = mid - 1;
	    if (c == 0) {
		return ((cell *) (items[mid]));
	    }
	}
    }
    return (NULL);
}

void cellList::appendFormula(fCell * newFormula)
{
//
// inserts newFormula after all cells which are referred to by newFormula's 
// formula, so they will be calculated before newFormula.
//
    fCell * minPos;		// least possible position in calcList
    fCell *aux;
    bool first = true;		// can be 1st in list, if no dependencies
    list *rl;			// list of references
    int i;
    minPos = (fCell *) calcList;
    rl = new list();
    reflist(newFormula, rl);		// fill the list
#if debugAppend > 1
    LOG3("references list for %s has %d items.\n", cellN(newFormula),rl->count);
#endif
    if (rl->count > 0) {
	for (i = 0; i < rl->count; i++) {	// for all references
	    aux = minPos;	// start at known least possible position
	    while (aux) {	// follow the linked list
		base * b= rl->items[i];
#if debugAppend > 1
//		LOG4("aux: %s %d==%d?\n", cellN(aux),aux->k.l,*((int *) rl->items[i]));
		LOG4("aux: %s %d==%d?\n",
		     cellN(aux),
		     aux->k.l,
		     ((coordW*)b)->k2.l
		    );
#endif
//		if (aux->k.l == *((int *) rl->items[i])) {	// found the coord in calcList
		if (aux->k.l == ((coordW*)b)->k.l) {	// found the coord in calcList
		    first = false;	// Zelle kann nicht erste sein
		    minPos = aux;	// this is new minimum
#if debugAppend > 1
		    LOG3("new min for %s after: %s\n",
			   cellN(newFormula), cellN(minPos));

#endif
		}
		aux = aux->nextFormula;
	    }
	}
    }
    delete rl;
    if (first) {
#if debug >1
	LOG2("prepend %s\n", cellN(newFormula));
#endif    
	newFormula->nextFormula = (fCell *) calcList;
	calcList = newFormula;
    } else {
#if debug >1
	LOG3("append %s after: %s\n",
	       cellN(newFormula), cellN(minPos));
#endif
	newFormula->nextFormula = minPos->nextFormula;	// append after minPos
	minPos->nextFormula = newFormula;
    }
    if (calcList == NULL)
	calcList = newFormula;
//  signal(calcSema); 
#ifdef SELFREF
    if (TestSelfRef(newFormula)) newFormula->setFlags(newFormula->getFlags() | flSelfRef);
#endif				/*  */
}

//int cellList::insert(void *item)
int cellList::insert(base *item)
{
    cell * it = (cell *) item;
#if debug > 3
    LOG3("insert: %s %d\n",cellN(it),it->typeID());
    FLUSH;
#endif
    int i;
    if (it->isFormula()) {
//    if ((((cell *) item)->typeID() == tfCell)
//	|| (((cell *) item)->typeID() == tfTimeCell)) {
#if debug >3
	LOG2("cellList::insert needs append to calcList.\n");
#endif
	appendFormula(((fCell *) item));
#if debug >3
//      LOG2(<< ((fCell *) item)->
//          text << " " << ((fCell *) item)->TPN.length() << endl;
//      FLUSH;
#endif
//      ((fCell *) item)->nextFormula = (fCell *) calcList;
//      calcList = item;
    }
    if (it->isNumber()) {
//    if ((((cell *) item)->typeID() == tnCell)
//	|| (((cell *) item)->typeID() == tnTimeCell)) {
#if debug > 1
	LOG1("insert needs append to numList.\n");
	FLUSH;
#endif
	((nCell *) item)->nextNum = (nCell *) numList;
	numList = item;
    }
#if debug > 1
    LOG1("before search.\n");
    FLUSH;
#endif    
    if (search(item, i) == 0)
	atInsert(i, item);
#if debug > 1
    LOG1("insert done.\n");
    FLUSH;
#endif
    return i;
}

bool cellList::search(void *key, int &index)
{
    int l = 0;
    int h = count - 1;
    bool res = false;
    while (l <= h) {
	int i = (l + h) >> 1;
	int c = compare(items[i], key);
	if (c < 0)
	    l = i + 1;
	else {
	    h = i - 1;
	    if (c == 0) {
		res = true;
	    }
	}
    }
    index = l;
    return res;
}

cellList::cellList():calcIntervall(1000000),
changed(0), calcList(NULL), numList(NULL),
avrList(NULL)
{
#if debug > 5
    LOG1("cellList::cellList()\n");
    FLUSH;
#endif
}

module_registration module_registrations[maxModules];

void doCallClients(void *p, void *arg)
{
    ClientInfo ci;
    ci.c = (cell *) p;
    ci.sheet = (int) arg;
#if debug > 1
    LOG2("test flags %s\n", cellN(ci.c));
    FLUSH;
#endif
    if ((ci.c->getFlags() & valChangedC) == 0)
	return;
#if debug > 1
    LOG2("test cflags global %s\n", cellN(ci.c));
    FLUSH;
#endif
    if (ci.c->getCflags() == 0)
	return;
#if debug > 1
    LOG2("test single cflags %s\n", cellN(ci.c));
    FLUSH;
#endif
    for (int i = 0; i < 32; i++) {
//      if (module_registrations[i].mask != 0) {
#if debug > 1
	LOG5("cells flags: %x module %d: %x condition: %x \n",
	       ci.c->getCflags(), i, module_registrations[i].mask,
	       module_registrations[i].mask & ci.c->getCflags());
	FLUSH;
#endif
	if ((module_registrations[i].mask & ci.c->getCflags()) != 0) {
#if debug > 1
	    LOG1("calling callback\n");
	    FLUSH;
#endif
	    module_registrations[i].Client->recalcCallBack(&ci);
	}
    }
    ci.c->setFlags(ci.c->getFlags() & (~valChangedC));
}

//
// recalculate all formulas:
//
#define setPrevValuesLately

void cellList::recalc(int sh)
{
    fCell *f;
    nCell *n;
    unsigned int tempState;
    int tempError;
    f = (fCell *) calcList;
    while (f != NULL) {
#if debug > 1
	LOG3("recalc %s: %s\n", cellN(f), f->text.c_str());
	FLUSH;
#endif
#if debug > 1
	LOG2("TPN.len %d\n", f->TPN.length());
	FLUSH;
#endif
	tempState=f->getState();
	tempError=f->getError();
	f->setVal(evalTPN(f->TPN, *this, tempError, tempState));
	f->setState(tempState);
	f->setError(tempError);
	f = f->nextFormula;
    }
//
// set flags and time on formula cells:
//    
    f = (fCell *) calcList;
    while (f != NULL) {
#if debug > 3
	LOG2("setting flags and time %s\n", cellN(f));
	FLUSH;
#endif
	if (f->getVal() != f->getPreVal()) {
	    f->setFlags(f->getFlags() | valChangedAll);
#ifndef setPrevValuesLately
	    f->setPreVal(f->getVal());
#endif	    
	    if (f->typeID() == tfTimeCell) {
		((fTimeCell *) f)->time = time(NULL);
	    }
	}
	f = f->nextFormula;
    }
//
// set flags and time on numeric cells:
//   
#if debug > 3
    LOG1("setting flags and time on numeric cells\n");
    FLUSH;
#endif

    n = (nCell *) numList;
#if debug > 3
    LOG1("n set\n");
    FLUSH;
#endif

    while (n != NULL) {
#if debug > 3
	LOG2("setting flags and time %s\n", cellN(n));
	FLUSH;
#endif
	if (n->getVal() != n->getPreVal()) {
	    n->setFlags(n->getFlags() | valChangedAll);
	    if (n->typeID() == tnTimeCell) {
		((nTimeCell *) n)->time = time(NULL);
	    }
#ifndef setPrevValuesLately	    
	    n->setPreVal(n->getVal());
#endif
	}
	n = n->nextNum;
    }
#if debug > 1
    LOG2("will call module callbacks for sheet %d\n", sh);
#endif
    forEach(doCallClients, (void *) sh);
#ifdef setPrevValuesLately    
    n = (nCell *) numList;
    while (n != NULL) {
#if debug > 3
	LOG2("setting previous values %s\n", cellN(n));
	FLUSH;
#endif
        n->setPreVal(n->getVal());
	n = n->nextNum;
    }
    
    f = (fCell *) calcList;
    while (f != NULL) {
#if debug > 3
	LOG2("setting previous values %s\n", cellN(f));
	FLUSH;
#endif
	if (f->getVal() != f->getPreVal()) {
	    f->setPreVal(f->getVal());
	}
	f = f->nextFormula;
    }
#endif //setPrevValuesLately    
#if debug > 1
    FLUSH;
#endif

}

cellList *sheets[maxSheet];

string stringContents(int shn, coord c, int len, int decimals)
{
    if (sheets[shn] != NULL) {
	cell *cl = sheets[shn]->find(c);
	if (cl != NULL) {
	    return (cl->stringContents(len, decimals));
	}
    }
    return ("");
}


char * cStringContents(int shn, coord c, int len, int decimals, char * buf)
{
    if (sheets[shn] != NULL) {
	cell *cl = sheets[shn]->find(c);
	if (cl != NULL) {
	    return (cl->cStringContents(len, decimals, buf));
	}
    }
    return (buf);
}


void doCallQueueClients(void *p, void *arg)
{
    ClientInfo ci;
    ci.c = (cell *) p;
    ci.sheet = (int) arg;
/*    
#if debug > 1
    LOG2("doCallQueueClients test flags %s\n", cellN(ci.c));
    FLUSH;
#endif
    if ((ci.c->flags & valChangedC) == 0)
	return;
	
#if debug > 1
    LOG2("doCallQueueClients test cflags global %s\n", cellN(ci.c));
    FLUSH;
#endif
    if (ci.c->cflags == 0)
	return;
*/	
#if debug > 1
    LOG2("doCallQueueClients test single cflags %s\n", cellN(ci.c));
    FLUSH;
#endif
    for (int i = 0; i < 32; i++) {
//      if (module_registrations[i].mask != 0) {
#if debug > 1
	LOG5("doCallQueueClients cells flags: %x module %d: %x condition: %x \n",
	       ci.c->getCflags(), i, module_registrations[i].mask,
	       module_registrations[i].mask & ci.c->getCflags());
	FLUSH;
#endif
	if ((module_registrations[i].mask & ci.c->getCflags()) != 0) {
#if debug > 1
	    LOG1("calling callback\n");
	    FLUSH;
#endif
//	    if (module_registrations[i].Client->valueQueueCallBack!=NULL)
		module_registrations[i].Client->enterCallBack(&ci);
	    
	}
    }
    ci.c->setFlags(ci.c->getFlags() & (~valChangedC));
}


void cellList::setCell(coord c, double v, int sourceTy)
{
    cell *cl = find(c);
    if (cl != NULL) {
//	if ((cl->typeID() == tnCell) || (cl->typeID() == tnTimeCell)) {
	if (cl->isNumber()) {
	    cl->setVal(v);
	    cl->setState(cl->getState() & ~(stDirty|0xFFFFFFF0));
	    cl->setSourceType(sourceTy);
	    if (cl->getCflags()!=0) {
#if debug > 0
		LOG2("will call module value queue callbacks for cell %s\n", cellN(c));
		FLUSH;
#endif
		doCallQueueClients(cl, (void *) shNumber);
	    } 
	}
    }
}


void setCell(int shn, coord c, double v, int sourceTy)
{
    if (sheets[shn] != NULL) {
	sheets[shn]->setCell(c, v, sourceTy);
    }
}

void cellList::setCell(coord c, double v, int sourceTy, unsigned int &state)
{
    cell *cl = find(c);
    if (cl != NULL) {
//	if ((cl->typeID() == tnCell) || (cl->typeID() == tnTimeCell)) {
	if (cl->isNumber()) {
	    cl->setVal(v);
	    cl->setState(state);
//           cl->state & ~stDirty;
	    cl->setSourceType(sourceTy);
	    if (cl->getCflags()!=0) {
#if debug > 0
		LOG2("will call module value queue callbacks for cell %s\n", cellN(c));
		FLUSH;
#endif
		doCallQueueClients(cl, (void *) shNumber);
	    } 
	}
    }
}


void setCell(int shn, coord c, double v, int sourceTy, unsigned int &state)
{
    if (sheets[shn] != NULL) {
	sheets[shn]->setCell(c, v, sourceTy, state);
    }
}

void cellList::setState(coord c, int eMask, bool sORr)
{
    cell *cl = find(c);
    if (cl != NULL) {
	if (sORr)
	    cl->setState(cl->getState() | eMask);
	else
	    cl->setState(cl->getState() & ~eMask);
    }
}

void setState(int shn, coord c, int eMask, bool sORr)
{
    if (sheets[shn] != NULL) {
	sheets[shn]->setState(c, eMask, sORr);
    }
}

int cellList::getFlags(coord c, bool & ok)
{
    cell *cl = find(c);
    if (cl != NULL) {
	ok = true;
	return (cl->getFlags());
    } else
	ok = false;
    return 0;
}

int getFlags(int shn, coord c, bool & ok)
{
    if (sheets[shn] != NULL) {
	return (sheets[shn]->getFlags(c, ok));
    } else
	ok = false;
    return 0;
}

int cellList::getCFlags(coord c, bool & ok)
{
    cell *cl = find(c);
    if (cl != NULL) {
	ok = true;
	return (cl->getCflags());
    } else
	ok = false;
    return 0;
}

int getCFlags(int shn, coord c, bool & ok)
{
    if (sheets[shn] != NULL) {
	return (sheets[shn]->getCFlags(c, ok));
    } else
	ok = false;
    return 0;
}

unsigned int cellList::getState(coord c, bool & ok)
{
    cell *cl = find(c);
    if (cl != NULL) {
	ok = true;
	return (cl->getState());
    } else
	ok = false;
    return 0;
}

unsigned int getState(int shn, coord c, bool & ok)
{
    if (sheets[shn] != NULL) {
	return (sheets[shn]->getState(c, ok));
    } else
	ok = false;
    return 0;
}



void clrFlags(int shn, coord c, int mask)
{
    if (sheets[shn] != NULL) {
    }
}

double numContents(int shn, coord c, bool & ok)
{
    cell *cl;
    ok = false;
    if (sheets[shn] == NULL)
	return (0);
    cl = sheets[shn]->find(c);
    if (cl == NULL)
	return (0);
    ok = true;
    return (cl->getVal());
}

double numContents(int shn, coord c, bool & ok, unsigned int &state)
{
    cell *cl;
    ok = false;
    if (sheets[shn] == NULL)
	return (0);
    cl = sheets[shn]->find(c);
    if (cl == NULL)
	return (0);
    ok = true;
    state |= cl->getState();
    return (cl->getVal());
}


string stringContentsNum(int shn, coord c, int width, int decimals)
{
    cell *cl;
    if (sheets[shn] == NULL)
	return ("no sheet");
    cl = sheets[shn]->find(c);
    if (cl == NULL)
	return ("no cell");
    return (cl->stringContentsNum(width, decimals));
}

string typeString(int shn, coord c)
{
    cell *cl;
    if (sheets[shn] == NULL)
	return ("");
    cl = sheets[shn]->find(c);
    if (cl == NULL)
	return ("");
#if debug >3
    LOG2("about to call typeString(%s)\n", cellN(c));
    FLUSH;
#endif
    return (cl->typeString());
}

string symbolName(int shn, coord c, int mode)
{
    if (sheets[shn] == NULL)
	return ("");
    return (sheets[shn]->symbolName(c, mode));
}

unsigned int usedFlags;		// bit mask, ones mark "changed" flags used by a driver
/*
struct registration {
    int mask;
    string name;
};
*/


registration registrations[maxDrivers];

int registerDriver(string Name, void *instance)
{
    int iMask = 16;
    for (int i = 0; i < 28; i++) {
//	LOG3("registrations: %s  %x\n",
//	       registrations[i].name.c_str(), registrations[i].mask);
	if (registrations[i].mask == 0) {
	    registrations[i].mask = iMask;
	    registrations[i].name = Name;
//          registrations[i].info = info;
	    registrations[i].pointerToInstance = instance;
	    return (iMask);
	}
	iMask <<= 1;
    }
    return (0);
}


int unRegisterDriver(int mask)
{
    for (int i = 0; i < 28; i++) {
	if (registrations[i].mask == mask) {
	    registrations[i].mask = 0;
//          registrations[i].info = info;
	    registrations[i].pointerToInstance = NULL;
	    return (0);
	}
    }
    return (-1);
}

#define badhack
int registerModule(string Name, client * aClient)
{
#ifndef badhack
    int iMask = 1, mask = 0, i = 0;
#else    
    int iMask = 2, mask = 0, i = 1;
#endif    
    while ((i < 32) && (mask == 0)) {
	if (module_registrations[i].mask == 0) {
	    module_registrations[i].mask = iMask;
	    module_registrations[i].name = Name;
	    module_registrations[i].Client = aClient;
	    mask = iMask;
	}
	iMask <<= 1;
	i++;
    }
//#if debug > 1
    for (i = 0; i < 32; i++) {
	LOG3("module_registrations: %s  %x\n",
	       module_registrations[i].name.c_str(),
	       module_registrations[i].mask);
    }
//#endif
    return (mask);
}

/*
    11/07/04: There is still no implementation of unRegisterModule.
    Hope this will do it:
*/
int unRegisterModule(int mask)
{
    for (int i = 0; i < 32; i++) {
	if (module_registrations[i].mask == mask) {
	    module_registrations[i].mask = 0;
//	    module_registrations[i].name = NULL;
	    module_registrations[i].name = "Reuseable";
	    module_registrations[i].Client = NULL;
	    return (0);
	}
    }
    return (-1);
}


/*
    Changes:
    
    02/11/2002 fixed a typo in setState
*/

void putit_22(void *a, void *x)
{
    *((Engine *) x) << ((base *) a);
}


IMPLEMENT_PERSISTENCE(cellList, "SHEET")
void cellList::Write(Engine & e) const
{
    Engine *ee = &e;
    e << calcIntervall;
    forEach(putit_22, ee);
}

void cellList::Read(Engine & e)
{
    cell *c;
    Engine *ee = &e;
    e >> calcIntervall;
    do {
	*ee >> c;
	if (c != 0)
//	    insert(((void *) c));
	    insert(c);
    }
    while (c != NULL);
}

void cellList::setCell(coord c, string s, int sourceTy)
{
    cell *cl = find(c);
    if (cl != NULL) {
//	if ((cl->typeID() == tnCell) || (cl->typeID() == tnTimeCell)) {
	if (cl->isNumber()) {
	    cl->setVal(atof(s.c_str()));
	    cl->setState( cl->getState() & ~stDirty);
	    cl->setSourceType(sourceTy);
	} else if ((cl->typeID() == ttCell)) {
	    ((tCell *) cl)->text = s;
	    cl->setState( cl->getState() & ~stDirty);
	}
    }
}

void setCell(int shn, coord c, string s, int sourceTy)
{
    if (sheets[shn] != NULL) {
	sheets[shn]->setCell(c, s, sourceTy);
    }
}

int getSourceType(int shn, coord c)
{
    if (sheets[shn] != NULL) {
	cell *cl = sheets[shn]->find(c);
	if (cl)
	    return (cl->getSourceType());
    }
    return (-2);
}

void setSource(int shn, coord c, string s)
{
    if (sheets[shn] != NULL) {
	sheets[shn]->setSource(c, s);
    }
}

void cellList::setSource(coord c, string s)
{
    cell *cl = find(c);
    if (cl != NULL) {
	cl->setSource(s);
    }
}

string getSource(int shn, coord c)
{
    if (sheets[shn] != NULL) {
	return sheets[shn]->getSource(c);
    } else
	return "";
}

string cellList::getSource(coord c)
{
    cell *cl = find(c);
    if (cl != NULL) {
	return(cl->getSource());
    } else
	return "";
}

double cellList::getTimestamp(coord c)
{
    cell *cl = find(c);
    if (cl != NULL) {
	if (cl->typeID() == tnTimeCell) {
	    return ((nTimeCell *) cl)->time;
	} else if (cl->typeID() == tfTimeCell) {
	    return ((fTimeCell *) cl)->time;
	}    
    } 
    return 0;
}

double getTimestamp(int shn, coord c)
{
    if (sheets[shn] != NULL) {
	return sheets[shn]->getTimestamp(c);
    }
    return (0);
}

double cellList::getPreVal(coord c)
{
    cell *cl = find(c);
    if (cl != NULL) {
	return (cl->getPreVal());
    } 
    return 0;
}

double getPreVal(int shn, coord c)
{
    if (sheets[shn] != NULL) {
	return sheets[shn]->getPreVal(c);
    }
    return (0);
}

int cellList::getError(coord c)
{
    cell *cl = find(c);
    if (cl != NULL) {
	return (cl->getError());
    } 
    return 0;
}

int getError(int shn, coord c)
{
    if (sheets[shn] != NULL) {
	return sheets[shn]->getError(c);
    }
    return (0);
}

/*
    Changes:
    
    02/11/2002 fixed a typo in setState
    
    04/20/2002  Sheet is now loaded using persistence mechanism.
		Object Read & write functions return void now. 
		cout<<... replaced by printf(..
		
    06/27/2002  Added method to retrieve sourceType.		
    02/26/2003  introduced the cStringContents functions. Hope they are faster.
    04/09/2003 	adaptions to changes in cell.h
    06/19/2003  setSource, getSource
    09/26/2003  reset driver error states 0xFFFFFFF and dirty state in setCell.
    02/14/2004  added getTimestamp
    02/14/2004  added getPreVal
    02/14/2004  added extra loops to update prevVal after module calls. Does it
		work?
    02/15/2004  added getError		
    11/10/2004  new logging, removed ThisModule
*/
