/*
 cellList.cc: Definitions of the basic structure of the spreadsheet.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#ifndef __cellList
#define __cellList
#include "cell.h"
#include "list.h"

#include "vi_string.h"

#define maxSheet 1000
#define maxModules 32
#define maxDrivers 30
/*
    modes for symbol Name:
*/
#define symOrCoord 0		//may return cellName if no lable defined
#define symOnly 1

struct ClientInfo{
    int sheet;
    cell * c;
};

class client {
public:
    virtual void recalcCallBack(ClientInfo * ci){}	// This is called when recalculation detects a change in value.	
    virtual void enterCallBack(ClientInfo * ci){}	// This is called when a value is set (which may be more often).	
};

class cellList:public list {
  public:
    int calcIntervall,		// recalculation intervall in microseconds
	calcDivisor,
	doRecalcCounter,
	shNumber; 		// number for this sheet	
    char changed;		// flag, if changed since construction                                 
     cellList();
    void *calcList;		// linked list of formulas for calculation
    void *numList;		// linked list of numbers to setup prevalue
    void *avrList;		// linked list average cells for average calculation
//    virtual int insert(void *item);
    virtual int insert(base *item);
    virtual void appendFormula(fCell * n);
    virtual bool search(void *key, int &index);
    void error(int code, int) {
	exit(212 - code);
    }
    void recalc(int sh);

    cell *find(coord k);

//protected:
    
    virtual int compare(void *key1, void *key2);
    string symbolName(coord c, int mode);
    void setCell(coord c, double v, int sourceTy);
    void setCell(coord c, double v, int sourceTy, unsigned int &state);
    void setCell(coord c, string s, int sourceTy);
    void setState(coord c, int eMask, bool sORr);
    int getFlags(coord c, bool & ok);
    int getCFlags(coord c, bool & ok);
    unsigned int getState(coord c, bool & ok);
//    virtual string typeST() {
//	return ("SHEET");
//    }	
    DECLARE_PERSISTENCE(cellList) 
    virtual void Read(Engine & e);
    virtual void Write(Engine & e) const;
    void setSource(coord c, string s);
    string getSource(coord c);
    double getTimestamp(coord c);  // this is double to leave room for
				    // subsecond precision
    double getPreVal(coord c);
    int getError(coord c);
};

extern cellList *sheets[maxSheet];

string stringContents(int shn, coord c, int len, int decimals);
string stringContentsNum(int shn, coord c, int width, int decimals);
char * cStringContents(int shn, coord c, int width, int decimals, char * buf);
void setCell(int shn, coord c, double v, int sourceTy);
void setCell(int shn, coord c, double v, int sourceTy,unsigned int &state);
void setCell(int shn, coord c, string s, int sourceTy);
void clrFlags(int shn, coord c, int mask);
void setState(int shn, coord c, int eMask, bool sORr);
int getFlags(int shn, coord c, bool & ok);
int getCFlags(int shn, coord c, bool & ok);
unsigned int getState(int shn, coord c, bool & ok);
double numContents(int shn, coord c, bool & ok);
double numContents(int shn, coord c, bool & ok, unsigned int &state);
string typeString(int shn, coord c);
string symbolName(int shn, coord c, int mode);
int registerDriver(string name,void *);
int unRegisterDriver(int mask);
//typedef void (*modCallBack) (void *);
int registerModule(string name, client * aClient);
int unRegisterModule(int mask);
int getSourceType(int shn, coord c);
double getTimestamp(int shn, coord c);  // this is double to leave room for
					// subsecond precision
double getPreVal(int shn, coord c); 
void setSource(int shn, coord c, string s);
string getSource(int shn, coord c);
int getError(int shn, coord c); 

struct registration {
    int mask;
    void * pointerToInstance;
    string name;
};

extern registration registrations[maxDrivers];

struct module_registration {
    int mask;
    string name;
    client * Client;
};

extern module_registration module_registrations[maxModules];

#endif				/* _cellList_ */
/*
    Changes:
    
    05/13/2002  Sheet is now loaded using persistence mechanism.
		Object Read & write functions return void now. 
		cout<<... replaced by printf(..
    27/11/2002  The sheet should know it's number.
    02/26/2003  introduced the cStringContents functions. Hope they are faster.
    06/19/2003  setSource, getSource
    02/14/2004  added getTimestamp
    02/14/2004  added getPreVal
    06/29/2004  added unRegisterModule (for redundancy)
*/
