/*
 test to load a configuration file.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <iostream.h>
#include <fstream.h>
#include "lib0.h"
#include "cellList.h"
#include <dlfcn.h>
#include "globals.h"
#include "plcdrv.h"
#include <pthread.h>
#include <unistd.h>

#include "vi_string.h"
#ifndef use_string_2
#else
#define getline readline
#define find index
#define erase del
#endif

#define debug 1



int main()
{
    string s, tfiName;
    string driverPath, driverLibName, driverInfo;
    int i = 1, p;
    fstream f("visual.cfg", ios::in);
#if debug > 3
    cout << "after open" << endl;
#endif
  nextline:
    while (getline(f, s)) {
#if debug > 3
	cout << "line " << i << ": " << s << endl;
#endif
	i++;
	if (s.find("#") == 0)
	    goto nextline;
	if (s.find("project:") == 0) {
#ifndef use_string_2
	    projectPath = noSpace(s.substr(8));
#else
	    projectPath = noSpace(s.from(8));
#endif
	    cout << "found project directory: " << projectPath << endl;
	    cout << "trying to load the spread sheets: " << endl;
	    for (int sh = 1; sh <= maxSheet; sh++) {
		sheets[sh] = NULL;
//          cout<<"trying to load: "<<projectPath+digits(sh)+".sht"<<endl;
		fstream f((projectPath + digits(sh) + ".sht").c_str(),
			  ios::in);
		bool b = f.ios::good();
		if (b) {
		    cout << "loading: " << projectPath + digits(sh) +
			".sht" << endl;
		    Engine *e = new Engine(f, Engine::modeRead);
		    sheets[sh] = new cellList();
		    cell *c;
		    do {
			*e >> c;
			if (c != 0)
			    sheets[1]->insert(c);
		    } while (c != NULL);
		    delete e;
		}
		f.close();
		if (!b)
		    goto nextline;
	    }			// for
	    goto nextline;
	}

	if (s.find("driverpath:") == 0) {
#ifndef use_string_2
	    driverPath = noSpace(s.substr(11));
#else
	    driverPath = noSpace(s.from(11));
#endif
	    cout << "found driver directory: " << driverPath << endl;
	    goto nextline;
	}
	if (s.find("driver:") == 0) {
#ifndef use_string_2
	    driverLibName = s.substr(7);
#else
	    driverLibName = s.from(7);
#endif
	    do {
		p = driverLibName.find(" ");
		if (p == 0)
		    driverLibName.erase(0, 1);
	    } while (p == 0);
#ifndef use_string_2
	    driverInfo = driverLibName.substr(p + 1);
	    driverLibName = driverLibName.substr(0, p);
#else
	    driverInfo = driverLibName.after(p);
	    driverLibName = driverLibName.before(p);
#endif
	    cout << "found driver: " << driverLibName << endl;
	    cout << "found driver info: " << driverInfo << endl;
	    p = driverInfo.find(".tfi");
	    if (p >= 0) {
		tfiName = driverInfo.substr(0, p + 4);
		driverInfo.erase(0, p + 4);
		cout << "Driver Instruction List in: " << projectPath +
		    tfiName << endl;
	    }
	    cout << "I will dlopen driver: " << driverPath +
		driverLibName << endl;
	    void *handle =
		dlopen((driverPath + driverLibName).c_str(), RTLD_LAZY);
	    if (!handle)
		cout << "dlopen: " << dlerror() << endl;
	    void *(*driver) (void *);
	    pthread_t taskHandle;
	    (void *) driver = dlsym(handle, "DriverThread__FPv");
	    cout << "dlsym: " << dlerror() << endl;
	    DriverInfo *di = new DriverInfo;
	    di->s1 = tfiName;
	    di->s = driverInfo;
	    int res = pthread_create(&taskHandle, NULL, driver, di);
	    cout << "created driver thread: " << res << endl;
	    cout.flush();
	    goto nextline;
	}
	cout << "*** Syntax error in line " << i - 1 << endl;
    }
    cout << "file ended" << endl;
    f.close();
    usleep(6000000);
    if (projectPath.empty()) {
	cout << "***No project directory given. Change visual.cfg!" <<
	    endl;
	abort();
    }
    return 0;
}
