/*
 Common functions for a serial line driver for Allen Bradley PLCs

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2002.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include "AB_lib.h"
/*
    I found AB's manual (ww.ab.com/manuals(cn/17706516.pdf) not very 
    precise about what goes into the sum. Experimenting with their 
    example showed me, that it's the data plus the final ETX, but not
    the STX. 
    This assumes we have the raw block before prepending it with
    DLE STX and adding DLE ETX. The ETX would go into the sum, but 
    not the DLE. To achieve that, we calc a final ETX that is not in 
    the block.
*/

unsigned short calc_CRC(unsigned char *b1, int size)
{
    unsigned short sum = 0;
    int i, j;
    for (j = 0; j < size; j++) {
	sum = sum ^ (b1[j]);
	for (i = 0; i <= 7; i++) {
	    if (sum & 0x1) {
		sum = sum >> 1;
		sum = sum ^ 0xA001;
	    } else
		sum = sum >> 1;
	}
    }

    sum = sum ^ 0x03;
    for (i = 0; i <= 7; i++) {
	if (sum & 0x1) {
	    sum = sum >> 1;
	    sum = sum ^ 0xA001;
	} else {
	    sum = sum >> 1;
	}
    }
    return sum;
}

/*
void removeDLEDLE(unsigned char *ab, int &size)
{
    unsigned char *b1 = ab;
    int j, j2 = 0;
    for (j = 0; j < size; j++, j2++) {
	ab[j2] = ab[j];
	if ((ab[j] == DLE) && (ab[j + 1] == DLE)) {
	    j++;
	    size--;
	}
    }
}
*/
