/*
 Part of multithreaded network servers. The servers MAIN thread is started by
 the main program. It starts in turn the ACCEPTER thread, which accepts a
 connection and passes the file descriptor to the MAIN thread. The server MAIN
 thread starts a PORT SERVER thread to handle the connection whenever it receives
 a connection fd. Thus, the PORT SERVER thread is NOT a child of ACCEPTER, but a 
 "brother" and SIGCHILD goes to the server MAIN thread. If it would go to ACCEPTER,
 which is blocked in the accept() system call, we would get 
 "process interrupted in system call" and that's it. 
 Maybe its different with pthread library..?

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

/* This holds the information passed from main thread to accepter thread */
struct accepter_info {
    int fd;		// The pipe end from main thread
    int port;		// The port number ehere to listen
};

struct portserver_info {
    int fd;		// file descriptor of accepted connection
    int IP;		// Client IP
};


void *accepter(void *arg)
{
    int s;
    int pipeToMain_fd;
    portserver_info pi;
    socklen_t addrlen;
    sockaddr_in addr;
    int i;
    int opt;
    usleep(10000);
    pipeToMain_fd = ((accepter_info *) arg)->fd;
    LOG2(ThisModule "My pipe is:%d\n", pipeToMain_fd);
    FLUSH;
    s = socket(AF_INET, SOCK_STREAM, 0);
    if (errno != 0) {
	LOG2("socket %s\n", strerror(errno));
	FLUSH;
    }
    LOG2("port %d\n", ((accepter_info *) arg)->port);
    addr.sin_family = AF_INET;
    addr.sin_port =htons(((accepter_info *) arg)->port);    
//    addr.sin_port =
//	(((((accepter_info *) arg)->port) & 0xff) << 8) |
//	(((((accepter_info *) arg)->port) & 0xff00) >> 8);
//    LOG2(ThisModule"Accepter : port %d\n",addr.sin_port);
    inet_aton("0.0.0.0", &addr.sin_addr);
    opt = 1;
    i = setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &opt, 4);
    LOG2("setsockopt %s\n", strerror(errno));
    addrlen = sizeof(addr);
    bind(s, (sockaddr *) & addr, addrlen);
    LOG2("bind %s\n", strerror(errno));
    listen(s, 1);
    LOG2("listen: %s\n", strerror(errno));
    while (1) {
	addrlen = sizeof(addr);
	LOG1("before accept\n");
	pi.fd = (accept(s, (sockaddr *) & addr, &addrlen));
	pi.IP=0;
	LOG2("after accept. New fd:%d\n", pi.fd);
	FLUSH;
	LOG3("PID(%d) client:%s\n", getpid(),
	       inet_ntoa(addr.sin_addr));
	FLUSH;
	write(pipeToMain_fd, &pi, sizeof(pi));
    }
    shutdown(s, 2);
//    LOG3(ThisModule "Accepter : %d shutdown done. Socket is: %d\n",
//	   getpid(), s);
    LOG2("shutdown done. Socket is: %d\n",s);
}

struct abstr {
    int fd;
    int mask;
};
