/*
 Implementation of a PLC driver for binary data exchange.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <sys/types.h>
#include <unistd.h>
#include "mytypes.h"
#include "plcdrv.h"
#include "cellList.h"
#include "cell.h"
#include <stdio.h>
#include "plclog.h"
#include "bin_drv.h"

#define uc unsigned char
#define ThisModule "bin_drv : "

//class binDriver:public PLCdrv {
//  public:
    binDriver::binDriver() {};
    
    void binDriver::setShortValues(short * w, int count, int sheet, coord cell) {
	for (int i = 0; i < count; i ++) {
	    setCell(sheet, cell, *w, soExternal);
#if debug >1
	    LOG3("I set Cell %s=%d\n", cellN(cell), *w);
#endif
	    w++;
	    cell.row++;
	}
    };
    
    void binDriver::setFloatValues(float * w,int count, int sheet, coord cell) {
	for (int i = 0; i < count; i ++) {
	    setCell(sheet, cell, *w, soExternal);
#if debug >1
	    LOG3("I set Cell %s=%g\n", cellN(cell), *w);
#endif
	    w++;
	    cell.row++;
	}
    }
    
    int binDriver::readChars(uc *b, int tmo) {
	fd_set FDS;
	bool charsAvail;
	int res;
	timeval t;
	t.tv_sec = 0;
	t.tv_usec = tmo;
	FD_ZERO(&FDS);
	FD_SET(rfd, &FDS);
	charsAvail = (select(rfd + 1, &FDS, NULL, NULL, &t) > 0);
	t.tv_sec = 0;
	t.tv_usec = tmo;
#if debug >2
	LOG1("readChars (here we are) \n");
#endif
	if (!charsAvail) {
	    LOG1("timeout !\n");
	    return (0);
	} else {
#if debug >2
	    LOG1("readChars reading up to 1024 chars.\n");
#endif
	    res = read(rfd, b, 1024);
//	    * real_size=res;
#if debug >2
	    int i;
#include "dumpchars3.cc"
	    LOG2("readChars %d chars read.\n", res);
	    FLUSH;
#endif
	    return (res);
	} 
    }

    int binDriver::readSingle(uc *b, int tmo) {
	fd_set FDS;
	bool charsAvail;
	int res;
	timeval t;
	t.tv_sec = 0;
	t.tv_usec = tmo;
	FD_ZERO(&FDS);
	FD_SET(rfd, &FDS);
	charsAvail = (select(rfd + 1, &FDS, NULL, NULL, &t) > 0);
	t.tv_sec = 0;
	t.tv_usec = tmo;
#if debug >2
	LOG1("readChars (here we are) \n");
#endif
	if (!charsAvail) {
	    LOG1("timeout !\n");
	    return (0);
	} else {
	    res = read(rfd, b, 1);
//	    * real_size=res;
#if debug >2
	    int i;
#include "dumpchars3.cc"
	    LOG2("readChars %d chars read.\n", res);
	    FLUSH;
#endif
	    return (res);
	} 
    }


//};

/*
    Changes:
    
    23/12/2002 	created.
*/
