/*
 Implementation of a client driver to exchange data with another instance
 of VISUAL. While client copied a whole sheet, this copies only the cells
 given in the tfi file.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include "mytypes.h"
#include "plcdrv.h"
#include "cellList.h"
#include "cell.h"
#include <termios.h>
#include "setport.h"
#include "lib0.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <errno.h>
#include "nethelp.h"
#ifndef DONT_USE_GETHOSTBYNAME
#include <netdb.h>		// for gethostbyname
#endif

#include "vi_string.h"

#include "plclog.h"
#define debug 5

#define ThisModule "client2  : "

class clientTI:public transferInstruction {
  public:
    string peer;		// IP (or Name)
    int port;
    int fd;
    coord first;
    coord last;
    coord target;
    coord offset;
    DECLARE_PERSISTENCE(clientTI)
    void Read(Engine & e);
};

IMPLEMENT_PERSISTENCE(clientTI, "CLIENT")

void clientTI::Read(Engine & e)
{
    string stc, ctc, nc;
    e >> comment >> peer >> port >> sheet >> stc >> ctc >> nc;
    first=getPos(stc);
    last=getPos(ctc);
    target=getPos(nc);
    offset.row=target.row-first.row;
    offset.col=target.col-first.col;
#if debug >3
    LOG2(ThisModule "comment    :%s\n", comment.c_str());
    LOG2(ThisModule "peer	:%s\n", peer.c_str());
    LOG2(ThisModule "port       :%d\n", port);
    LOG2(ThisModule "sheet      :%d\n", sheet);
    LOG2(ThisModule "first      :%s\n", cellN(first));
    LOG2(ThisModule "last       :%s\n", cellN(last));
    LOG2(ThisModule "target     :%s\n", cellN(target));
#endif
}

class clientDriver:public PLCdrv {
  public:
    word DriverNr;
    clientTI *pa;
    word DriverState;
    clientDriver() {};
    virtual void execTI(transferInstruction * p);
    virtual void intSetup(DriverInfo * i);
    void execute() {
	while (1) {
	    execList();
	    execCount++;
	}
    };
    void openSocket(clientTI * ti);
    void markErrorStates(clientTI * ti);
};

void openSocket1(void *ti, void *ad)
{
    ((clientDriver *) ad)->openSocket(((clientTI *) ti));
}

void clientDriver::intSetup(DriverInfo * i)
{
    LOG1(ThisModule "intSetup \n");
    TypeManager::Add("CLIENT2", CreateNewclientTI);
    PLCdrv::load(i);
    changedMask = registerDriver(ThisModule, this);
    errorMask = changedMask;
    il->forEach(openSocket1, this);
};

void setStateOfCell(void *c, void *m)
{
    LOG2(ThisModule "setStateOfCell %s \n", cellN(((cell *) c)->k));
    ((cell *) c)->setState( ((cell *) c)->getState() | *((int *) m));
};

void clientDriver::markErrorStates(clientTI * ti)
{
    sheets[ti->sheet]->forEach(setStateOfCell, &errorMask);
};

void clientDriver::openSocket(clientTI * ti)
{
    sockaddr_in addr;
    socklen_t addrlen;
    LOG1(ThisModule "enter OpenSocket");
    FLUSH;
    addr.sin_family = AF_INET;
    addr.sin_port =
	(((ti->port) & 0xff) << 8) | (((ti->port) & 0xff00) >> 8);
#ifndef DONT_USE_GETHOSTBYNAME
    hostent *he = gethostbyname(ti->peer.c_str());
    memcpy(&addr.sin_addr, he->h_addr_list[0], sizeof(addr.sin_addr));
#else
    inet_aton(ti->peer.c_str(), &addr.sin_addr);
#endif
    ti->fd = socket(AF_INET, SOCK_STREAM, 0);
    LOG2(ThisModule "OpenSocket: socket is %d\n", ti->fd);
    addrlen = sizeof(addr);
    if (connect(ti->fd, (sockaddr *) & addr, addrlen)) {
	LOG2(ThisModule "Socket error: %s \n", strerror(errno));
	close(ti->fd);
	ti->fd = 0;
	markErrorStates(ti);
    } else
	LOG2(ThisModule "Connected to host: %s \n", ti->peer.c_str());
    FLUSH;
}

void clientDriver::execTI(transferInstruction * t)
{
    char buffer[bSize];
    int wpos = 0, rpos = 0, po;
    bool b;
    coord cell;
    coord source;
    string aCell;
    string answ;
#if debug >2
    LOG1(ThisModule "execTI\n");
    FLUSH;
#endif
    execError = false;
    clientTI * ti = (clientTI *) t;
    if (ti->fd <= 0)
	openSocket(ti);
    if (ti->fd > 0) {
	string s = "getlist ";
	for (source.row = ti->first.row; source.row<=ti->last.row; source.row++) {
	    for (source.col = ti->first.col; source.col<=ti->last.col; source.col++) {
		aCell =cellName(source);
#if debug >4
		LOG2(ThisModule "cell %s\n",cellN(source));
		FLUSH;
#endif		
		if (s.length()+aCell.length()+1<253) {
		    s=s+aCell+',';
		} else {
		    s=s.substr(0,s.length()-1)+"\n";
		    write(ti->fd, s.c_str(), s.length()); 
		    string s = "getlist "+aCell;
		}
	    }
	}    	    
	if (s!="getlist ") {
	    s=s.substr(0,s.length()-1)+"\n";
	    write(ti->fd, s.c_str(), s.length()); 
	}    
#if debug >2
	int res =
#endif
	    write(ti->fd, s.c_str(), s.length());
#if debug >2
	LOG2(ThisModule "execTI write: %d\n", res);
	FLUSH;
#endif
	usleep(10000 * 5);
	do {
	    b = readline(ti->fd, &buffer[0], answ, wpos, rpos);
#if debug >3
	    LOG3(ThisModule "execTI answ: %s length:%d\n", answ.c_str(), b);
	    FLUSH;
#endif
	    if (answ.empty()) {
		LOG1(ThisModule "no answer from server!");
		FLUSH;
		break;
	    }
	    if (answ == "end of list")
		break;
	    po = answ.Find("=");
	    string cellStr = answ.before(po);
	    answ = answ.after(po);
	    po = answ.Find(":");
	    string valueStr = answ.before(po);
	    answ = answ.after(po);
	    cell = getPos(cellStr);
	    cell.row+=ti->offset.row;
	    cell.col+=ti->offset.col;
	    double v = atof(valueStr.c_str());
	    int st = atol(answ.c_str());
#if debug >3
	    LOG3(ThisModule "I will set Cell %s to %0.7f\n",
		   cellN(cell), v);
	    FLUSH;
#endif
	    setCell(ti->sheet, cell, v, soExternal);
	    setState(ti->sheet, cell, st, true);
	} while (b);
	usleep(500000);		// try to give time to other threads??
    } else
	usleep(2000000);	// try to give time to other threads??
}

#include "thread_makro.cc"
#if debug>0
DTDebug(clientDriver)
#else
DT(clientDriver)
#endif
/*
    Changes:
    
    03/29/2003 	written
*/
