/*
 Helper functions for messages in the spreadsheet viewer. Maybe replaced
 by something which allows to change language without recompilation.

 Part of VISUAL, a human machine interface and data acquisition program

 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001. 

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#include "config.h"
#define Uses_TRect
#define Uses_TStatusLine
#define Uses_TStatusDef
#define Uses_TStatusItem
#define Uses_TKeys
#define Uses_TMenuBar
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuItem
#define Uses_TDialog
#define Uses_TSItem
#define Uses_TCheckBoxes
#define Uses_TInputLine
#define Uses_TLabel
#define Uses_TButton
#define Uses_TView

#include <tvision/tv.h>


#include "cell.h"
#include "lib0.h"
#include "helpers.h"
#include "shcommands.h"
#include "viewcommands.h"
#include "hcontext.h"
#include "help.h"
#include "lang.h"


#include "vi_string.h"

string scL(int sc)
{
    switch (sc) {
    case scShLeer:
	return ("leer");
    case scShGoto:
	return ("gehe zu");
    case scShText:
	return ("Text");
    case scShZahl:
	return ("Zahl");
    case scShForm:
	return ("Formel");
    case scShLastChange:
	return ("letzte nderung:");
    case scShError:
	return ("FEHLER");
    case scShFOR:
	return ("FOR");
    case scShRES:
	return ("ERG");
    case scShSave:
	return ("Speichern!");
    case scShTitle:
	return ("Rechenblatt");
    case scShPageNumber:
	return ("Bl.");
    case scShAskCreate:
	return ("Rechenblatt neu erzeugen ?");
    case scShNoArea:
	return ("Whlen Sie zuerst einen Bereich");
    case scShNoSheet:
	return ("Kein Rechenblatt Nr. ");
    case scShDelArea:
	return ("Bereich wirklich lschen");
    case scShClose:
	return ("Rechenblatt wurde gendert. Wirklich schlieen?");

    case scOfSourcet:
	return ("direkte Eingabe");
    case scOfSourcet + soExternal:
	return ("externe Eingabe");
    case scOfSourcet + soExternal2:
	return ("externe Eingabe 2");
    case scOfSourcet + soAcknowledge:
	return ("Acknowledge");
    case scOfSourcet + soRecipe:
	return ("Wert aus Rezept");
    case scOfSourcet + soBasicVar:
	return ("BASIC-Variable");
    case scOfSourcet + soSetByHMI:
	return ("Wert von Bedienelement");
    case scOfSourcet + soPushButton:
	return ("flag Taster gedrckt");
    case scOfSourcet + soPushBAck:
	return ("Taster verarbeitet");
    case scOfSourcet + soIoWorkSpace:
	return ("E/A Variable");
    case scOfSourcet + soIoState:
	return ("E/A Fehlerstatus");
    case scOfSourcet + soUDFmodule:
	return ("von benutzerdefiniertem Modul");


    case scOfMath + ceDiv0:
	return ("Division durch Null");
    case scOfMath + ceLogNeg:
	return ("Logarithmus aus negativer Zahl");
    case scOfMath + ceWrongType:
	return ("Falscher Zelltyp");
    case scOfMath + ceMissing:
	return ("Zelle existiert nicht");
    case scOfMath + ceRootNeg:
	return ("Wurzel aus negativer Zahl");
    case scOfMath + ceSucc:
	return ("Folgefehler");
    case scOfMath + ceMissingSheet:
	return ("Blatt existiert nicht");

    case scOfMenus:
	return ("\003Visual Version"VERSION" alpha\n\n" "\003Copyright (c) 2001 Thomas Hergenhahn\n"	// The \003 centers the line.
		"\003Thomas.Hergenhahn@web.de\n\n"
		"\003Erstellt mit TurboVision\n"
		"\003Copyright (c) 1994 Borland International\n"
		"\003Angepasst von Sergio Sigala <ssigala@globalnet.it>\n");
    case scOfMenus+1:
	return ("");	
    case scOfIoErrors:	return ("O.K.");
    case scOfIoErrors+1:	return ("Gertespezifischer Fehler");		
    case scOfIoErrors+2:	return ("was war noch 256?");		
    case scOfIoErrors+3:	return ("timeout");			
    default:
	return ("fixme sc" + digits(sc));
    }
}

TStatusLine *langStatus(TRect r)
{
    return (new TStatusLine(r,
			    *new TStatusDef(0, 50) +
			    *new TStatusItem("~F1~ Hilfe", kbF1, cmHelp) +
			    *new TStatusItem("~Alt-X~ Beenden", kbAltX,
					     cmQuit) + *new TStatusItem(0,
									kbAltF3,
									cmClose)
			    + *new TStatusItem(0, kbF10,
					       cmMenu) +
			    *new TStatusItem(0, kbF5,
					     cmZoom) + *new TStatusItem(0,
									kbCtrlF5,
									cmResize)
			    + *new TStatusDef(hcShViewer,
					      hcShViewer) +
			    *new TStatusItem("~F1~ Hilfe", kbF1,
					     cmHelp) +
			    *new TStatusItem("~F2~ Speichern", kbF2,
					     cmshSave) +
			    *new TStatusItem("~Alt-E~ Editieren", kbAltE,
					     cmshEditCell) +
			    *new TStatusItem("~Alt-L~ Label", kbAltL,
					     cmshEditLabel) +
			    *new TStatusItem("~Alt-G~ Gehe zu", kbAltG,
					     cmshGoto) +
			    *new TStatusItem("~Alt-C~ Lschen", kbAltC,
					     cmshDelete) +
			    *new TStatusItem("~Alt-B~ Bereich", kbAltB,
					     cmshArea) +
			    *new TStatusItem(0, kbAltF3,
					     cmClose) + *new TStatusItem(0,
									 kbF10,
									 cmMenu)));
}

TMenuBar *langMenu(TRect r)
{
    TSubMenu & sub1 =
	*new TSubMenu("~\360~", 0, hcSystem) +
	newLine() +
	*new TMenuItem("~I~nfo...", cmAboutCmd, kbNoKey, hcSAbout);

    TSubMenu & sub2 = *new TSubMenu("~D~atei", 0, hcFile) +
//        *new TMenuItem( "~O~pen...", cmOpenCmd, kbF3, hcFOpen, "F3" ) +
	newLine() + *new TMenuItem("~B~eenden", cmQuit, kbAltX, hcFExit,
				   "Alt-X");


    TSubMenu & sub3 =
	*new TSubMenu("~F~enster", 0, hcWindows) +
	*new TMenuItem("~G~re/Position", cmResize, kbCtrlF5, hcWSizeMove,
		       "Ctrl-F5") + *new TMenuItem("~Z~oom", cmZoom, kbF5,
						   hcWZoom,
						   "F5") +
	*new TMenuItem("~N~chstes", cmNext, kbF6, hcWNext,
		       "F6") + *new TMenuItem("~S~chlieen", cmClose,
					      kbAltF3, hcWClose,
					      "Alt-F3") +
	*new TMenuItem("~A~ufteilen", cmTile, kbNoKey,
		       hcWTile) + *new TMenuItem("~b~ereinander",
						 cmCascade, kbNoKey,
						 hcWCascade);


    TSubMenu & sub4 =
	*new TSubMenu("~R~echenblatt", 0, hcOptions) +
	*new TMenuItem("~~ffnen", cmshOpen, kbF3, hcNoContext) +
	*new TMenuItem("~S~peichern", cmshSave, kbF2, hcNoContext) +
	*new TMenuItem("~N~chstes", cmshNext, '+', hcNoContext) +
	*new TMenuItem("~V~origes", cmshPrev, '-', hcNoContext) +
	*new TMenuItem("Op~t~ionen", cmshOptions, kbNoKey, hcNoContext) +
	*new TMenuItem("~F~inden", 0,
		       new TMenu(*new
				 TMenuItem("~N~chste Zelle", cmshFindAny,
					   kbNoKey, hcNoContext, NULL,
					   new
					   TMenuItem("~N~chste Formel",
						     cmshFindFor, kbNoKey,
						     hcNoContext, NULL,
						     new
						     TMenuItem
						     ("~N~chsten Text",
						      cmshFindFor, kbNoKey,
						      hcNoContext, NULL,
						      new
						      TMenuItem
						      ("~N~chste Zahl mit Zeitstempel",
						       cmshFindTnum,
						       kbNoKey,
						       hcNoContext, NULL,
						       new
						       TMenuItem
						       ("~N~chste Formel mit Zeitstempel",
							cmshFindTfor,
							kbNoKey,
							hcNoContext, NULL,
							new
							TMenuItem
							("~V~erbundene Zelle zu HMI-Element",
							 cmshFindHMICell,
							 kbNoKey,
							 hcNoContext)
						       )
						      )
						     )
					   )
				 )
		       )
		       , hcNoContext) +
//    )+
	*new TMenuItem("~M~arkieren", 0,
		       new TMenu(*new
				 TMenuItem("~Z~elle", cmshMark, kbAltM,
					   hcNoContext, NULL,
					   new TMenuItem("Re~f~erenzen",
							 cmshMarkRef,
							 kbNoKey,
							 hcNoContext, NULL,
							 new
							 TMenuItem
							 ("~B~ereich",
							  cmshMarkArea,
							  kbNoKey,
							  hcNoContext)
					   )
				 )
		       )
		       , hcNoContext) +
	*new TMenuItem("~B~ereich", 0,
		       new TMenu(*new
				 TMenuItem("~L~schen", cmshDelArea,
					   kbNoKey, hcNoContext, NULL,
					   new TMenuItem("~K~opieren",
							 cmshCopyArea,
							 kbAltK,
							 hcNoContext, NULL,
							 new
							 TMenuItem
							 ("~M~arkieren",
							  cmshMarkArea,
							  kbNoKey,
							  hcNoContext)
					   )
				 )
		       )
		       , hcNoContext);


    r.b.y = r.a.y + 1;
    TSubMenu & sub5 =
	*new TSubMenu("~A~nzeigen", 0, hcWindows) +
	*new TMenuItem("~T~reiber-Module", cmViewDrivers, kbNoKey,
		       hcNoContext,
		       "") + *new TMenuItem("~R~ecorder-Module",
					    cmViewRecorders, kbNoKey,
					    hcNoContext, "");

    return (new TMenuBar(r, sub1 + sub2 + sub3 + sub4 + sub5));
}

TDialog *lOptionsDialog()
{
    TRect R;
#define normScreenWidth 80        
#define optionsDialogLeft 15
#define optionsDialogRight normScreenWidth-optionsDialogLeft
#define optionsBoxWidth optionsDialogRight-optionsDialogLeft-1
#define optionsBoxHeight 10    
    R = TRect(optionsDialogLeft, 1, optionsDialogRight, optionsBoxHeight+9);
    TDialog *D = new TDialog(R, "Optionen Rechenblatt");
    R = TRect(1, 2, optionsBoxWidth, optionsBoxHeight);
    TView *aux = new TCheckBoxes(R,
				 new TSItem("~I~nterne Formeldarstellung",
					    new
					    TSItem
					    ("~F~ormeln statt Ergebnisse anzeigen",
					     new TSItem("F~l~ags anzeigen",
					    new TSItem	("~S~tatus anzeigen",
							 new
							 TSItem
							 ("~H~exadezimale Zahlen",
							  new
							  TSItem
							  ("~Q~uellen der Eingaben anzeigen",
							  new
							  TSItem
							  ("~K~oordinaten beim Kopieren nicht nicht anpassen",
							  new TSItem("~C~lient flags anzeigen",
							   NULL))))))))
	);

    D->insert(aux);
    aux = new TInputLine(TRect(19, optionsBoxHeight, 26, 1+optionsBoxHeight), 5, NULL);
    D->insert(aux);
    D->insert(new TLabel(TRect(1, optionsBoxHeight, 16, 1+optionsBoxHeight), "Spaltenbreite", aux));

    D->insert(aux);
    aux = new TInputLine(TRect(19, 1+optionsBoxHeight, 26, 2+optionsBoxHeight), 5, NULL);
    D->insert(aux);
    D->insert(new TLabel(TRect(1, 1+optionsBoxHeight, 16, 2+optionsBoxHeight), "Dezimalstellen", aux));

    R = TRect(9, 5+optionsBoxHeight, 19, 7+optionsBoxHeight);
    D->insert(new TButton(R, "O~K~", cmOK, bfDefault));
    R = TRect(21, 5+optionsBoxHeight, 33, 7+optionsBoxHeight);
    D->insert(new TButton(R, "Abbrechen", cmCancel, bfNormal));
    return (D);
}

void initLanguage()
{
    LangMenu = &langMenu;
    LangStatus = &langStatus;
    scLang = &scL;
    LangOptionsDialog = &lOptionsDialog;
}

/*
    Changes:
    
    06/20/2002  fixed a typo
    21/09/2002  added topic "coordinate shift" to options dialog
*/
