/*
 Helper functions for messages in the spreadsheet viewer. Maybe replaced
 by something which allows to change language without recompilation.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001. 

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include "config.h"
#define Uses_TRect
#define Uses_TStatusLine
#define Uses_TStatusDef
#define Uses_TStatusItem
#define Uses_TKeys
#define Uses_TMenuBar
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuItem
#define Uses_TDialog
#define Uses_TSItem
#define Uses_TCheckBoxes
#define Uses_TInputLine
#define Uses_TLabel
#define Uses_TButton
#define Uses_TView

#include "tvision/tv.h"


#include "cell.h"
#include "helpers.h"
#include "lib0.h"

#include "shcommands.h"
#include "viewcommands.h"
#include "hcontext.h"
#include "help.h"
#include "lang.h"

#include "vi_string.h"

string scL(int sc)
{
    switch (sc) {
    case scShLeer:
	return ("empty");
    case scShGoto:
	return ("go to");
    case scShText:
	return ("text");
    case scShZahl:
	return ("number");
    case scShForm:
	return ("formula");
    case scShLastChange:
	return ("last change:");
    case scShError:
	return ("ERROR");
    case scShFOR:
	return ("FOR");
    case scShRES:
	return ("RES");
    case scShSave:
	return ("Save!");
    case scShTitle:
	return ("Spreadsheet");
    case scShPageNumber:
	return ("Sh.");
    case scShAskCreate:
	return ("Create new Spreadsheet ?");
    case scShNoArea:
	return ("Choose an Area first");
    case scShNoSheet:
	return ("No Spreadsheet No. ");
    case scShDelArea:
	return ("Really delete this area");
    case scShClose:
	return ("Sheet contains unsaved changes. really close");

    case scOfSourcet:
	return ("direct input");
    case scOfSourcet + soExternal:
	return ("external input");
    case scOfSourcet + soExternal2:
	return ("external input 2");
    case scOfSourcet + soAcknowledge:
	return ("Acknowledge");
    case scOfSourcet + soRecipe:
	return ("value from recipe");
    case scOfSourcet + soBasicVar:
	return ("value is a BASIC variable");
    case scOfSourcet + soSetByHMI:
	return ("value set by HMI");
    case scOfSourcet + soPushButton:
	return ("flag Pushbutton pressed");
    case scOfSourcet + soPushBAck:
	return ("Pushbutton processed");
    case scOfSourcet + soIoWorkSpace:
	return ("I/O Variable");
    case scOfSourcet + soIoState:
	return ("I/O Error State");
    case scOfSourcet + soUDFmodule:
	return ("from user defined module");


    case scOfMath + ceDiv0:
	return ("division by zero");
    case scOfMath + ceLogNeg:
	return ("logaritm number<=0");
    case scOfMath + ceWrongType:
	return ("wrong cell type");
    case scOfMath + ceMissing:
	return ("cell doesn't exist");
    case scOfMath + ceRootNeg:
	return ("sqare root from negative number");
    case scOfMath + ceSucc:
	return ("consecutive error");
    case scOfMath + ceMissingSheet:
	return ("sheet doesn't exist");

    case scOfMenus:
	return ("\003Visual Version" VERSION" alpha\n\n" "\003Copyright (c) 2001 Thomas Hergenhahn\n"	// The \003 centers the line.
		"\003Thomas.Hergenhahn@web.de\n\n"
		"\003Build with TurboVision\n"
		"\003Copyright (c) 1994 Borland International\n"
		"\003Modified by Sergio Sigala <ssigala@globalnet.it>\n");
    case scOfMenus+1:
	return ("");
    case scOfIoErrors:	 return ("O.K.");
    case scOfIoErrors+1: return ("Device specific error");		
    case scOfIoErrors+2: return ("what was 256?");		
    case scOfIoErrors+3: return ("timeout");			
	
    default:
	return ("fixme sc" + digits(sc));
    }
}

TDialog *lOptionsDialog()
{
    TRect R;
#define normScreenWidth 80        
#define optionsDialogLeft 15
#define optionsDialogRight normScreenWidth-optionsDialogLeft
#define optionsBoxWidth optionsDialogRight-optionsDialogLeft-1
#define optionsBoxHeight 10    
    R = TRect(optionsDialogLeft, 1, optionsDialogRight, optionsBoxHeight+9);    
    TDialog *D = new TDialog(R, "Spreadsheet Options");
    R = TRect(1, 2, optionsBoxWidth, optionsBoxHeight);    
    
    TView *aux
    
     = new TCheckBoxes(R,
     
        new TSItem ("Display ~i~nternal Formula Representation",
	new TSItem ("Display ~F~ormulas (instead of results)",
	new TSItem ("Display F~l~ags",
	new TSItem ("Display ~S~tate",
	new TSItem ("~H~ex Numbers",
	new TSItem ("Display Input ~S~ources",
	new TSItem ("Do not shift coordinates in copy function",
	new TSItem ("Display ~C~lient flags",
		NULL
		))))))))
	);

    D->insert(aux);
    
    aux = new TInputLine(TRect(19, optionsBoxHeight, 26, 1+optionsBoxHeight), 5, NULL);
    D->insert(aux);
    D->insert(new TLabel(TRect(1, optionsBoxHeight, 16, 1+optionsBoxHeight), "Column Width", aux));

    D->insert(aux);
    aux = new TInputLine(TRect(19, 1+optionsBoxHeight, 26, 2+optionsBoxHeight), 5, NULL);
    D->insert(aux);
    D->insert(new TLabel(TRect(1, 1+optionsBoxHeight, 14, 2+optionsBoxHeight), "Decimals", aux));

    R = TRect(9, 5+optionsBoxHeight, 19, 7+optionsBoxHeight);
    D->insert(new TButton(R, "O~K~", cmOK, bfDefault));
    R = TRect(21, 5+optionsBoxHeight, 33, 7+optionsBoxHeight);
    D->insert(new TButton(R, "Cancel", cmCancel, bfNormal));
    return (D);
}
void initLanguage()
{
//    LangMenu=&langMenu;
//    LangStatus=&langStatus;
    scLang = &scL;
    LangOptionsDialog = &lOptionsDialog;
}
