/*
 Helper functions for messages in the spreadsheet viewer. Maybe replaced
 by something which allows to change language without recompilation.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001. 

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#include "config.h"
#define Uses_TRect
#define Uses_TStatusLine
#define Uses_TStatusDef
#define Uses_TStatusItem
#define Uses_TKeys
#define Uses_TMenuBar
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuItem
#define Uses_TDialog
#define Uses_TSItem
#define Uses_TCheckBoxes
#define Uses_TInputLine
#define Uses_TLabel
#define Uses_TButton
#define Uses_TView

#include <tvision/tv.h>

#include "cell.h"
#include "helpers.h"
#include "lib0.h"
#include "shcommands.h"
#include "viewcommands.h"
#include "hcontext.h"
#include "help.h"
#include "lang.h"

#include "vi_string.h"

string scL(int sc)
{
    switch (sc) {
    case scShLeer:
	return ("vite");
    case scShGoto:
	return ("allez a");
    case scShText:
	return ("text");
    case scShZahl:
	return ("number");
    case scShForm:
	return ("formule");
    case scShLastChange:
	return ("dernier change:");
    case scShError:
	return ("ERREUR");
    case scShFOR:
	return ("FOR");
    case scShRES:
	return ("RES");
    case scShSave:
	return ("Sauver!");
    case scShTitle:
	return ("Tableau de calculation");
    case scShPageNumber:
	return ("Tb.");
    case scShAskCreate:
	return ("Creer noveau tablaeu ?");
    case scShNoArea:
	return ("Selectioner un areal s.v.p.");
    case scShNoSheet:
	return ("Pas de tableau numero. ");
    case scShDelArea:
	return ("Really delete this area");
    case scShClose:
	return ("Sheet contains unsaved changes. really close");

    case scOfSourcet:
	return ("Entree direct");
    case scOfSourcet + soExternal:
	return ("Entree externel");
    case scOfSourcet + soExternal2:
	return ("external input 2");
    case scOfSourcet + soAcknowledge:
	return ("Acknowledge");
    case scOfSourcet + soRecipe:
	return ("valeur de recipe");
    case scOfSourcet + soBasicVar:
	return ("valeur est une variable BASIC");
    case scOfSourcet + soSetByHMI:
	return ("valeur du element HMI");
    case scOfSourcet + soPushButton:
	return ("flag Pushbutton premee");
    case scOfSourcet + soPushBAck:
	return ("Buton processee");
    case scOfSourcet + soIoWorkSpace:
	return ("Variable E/S");
    case scOfSourcet + soIoState:
	return ("Etat d'Erreur E/S");
    case scOfSourcet + soUDFmodule:
	return ("de module definee par utilisant");


    case scOfMath + ceDiv0:
	return ("division par zero");
    case scOfMath + ceLogNeg:
	return ("logarithm du numero<=0");
    case scOfMath + ceWrongType:
	return ("mauvais type du cellule");
    case scOfMath + ceMissing:
	return ("cellule n'exist pas");
    case scOfMath + ceRootNeg:
	return ("radi... carree du numero negative");
    case scOfMath + ceSucc:
	return ("Erreur consecutive");
    case scOfMath + ceMissingSheet:
	return ("Tableau n'exist pas");

    case scOfMenus:
	return ("\003Visual Version" VERSION " alpha\n\n" "\003Copyright (c) 2001 Thomas Hergenhahn\n"	// The \003 centers the line.
		"\003Thomas.Hergenhahn@web.de\n\n"
		"\003fait avec TurboVision\n"
		"\003Copyright (c) 1994 Borland International\n"
		"\003Modifiee par Sergio Sigala <ssigala@globalnet.it>\n");
	case scOfMenus+1:
	return ("");	

    default:
	return ("Il faut ... message sc" + digits(sc));
    }
}

TStatusLine *langStatus(TRect r)
{
    return (new TStatusLine(r,
			    *new TStatusDef(0, 50) +
			    *new TStatusItem("~F1~ Help", kbF1, cmHelp) +
			    *new TStatusItem("~Alt-Q~ Quitter", kbAltQ,
					     cmQuit) + *new TStatusItem(0,
									kbAltF3,
									cmClose)
			    + *new TStatusItem(0, kbF10,
					       cmMenu) +
			    *new TStatusItem(0, kbF5,
					     cmZoom) + *new TStatusItem(0,
									kbCtrlF5,
									cmResize)
			    + *new TStatusDef(hcShViewer,
					      hcShViewer) +
			    *new TStatusItem("~F1~ Help", kbF1,
					     cmHelp) +
			    *new TStatusItem("~F2~ Save", kbF2,
					     cmshSave) +
			    *new TStatusItem("~Alt-E~ Edit", kbAltE,
					     cmshEditCell) +
			    *new TStatusItem("~Alt-L~ Label", kbAltL,
					     cmshEditLabel) +
			    *new TStatusItem("~Alt-A~ Allez a", kbAltA,
					     cmshGoto) +
			    *new TStatusItem("~Alt-C~ Delete", kbAltC,
					     cmshDelete) +
			    *new TStatusItem("~Alt-B~ Area", kbAltB,
					     cmshArea) +
			    *new TStatusItem(0, kbAltF3,
					     cmClose) + *new TStatusItem(0,
									 kbF10,
									 cmMenu)));
}

TMenuBar *langMenu(TRect r)
{
    TSubMenu & sub1 =
	*new TSubMenu("~\360~", 0, hcSystem) +
	newLine() +
	*new TMenuItem("~A~bout...", cmAboutCmd, kbNoKey, hcSAbout);

    TSubMenu & sub2 = *new TSubMenu("~F~ichier", 0, hcFile) +
//        *new TMenuItem( "~O~pen...", cmOpenCmd, kbF3, hcFOpen, "F3" ) +
	newLine() + *new TMenuItem("~Q~itter", cmQuit, kbAltQ, hcFExit,
				   "Alt-Q");


    TSubMenu & sub3 =
	*new TSubMenu("~F~enetre", 0, hcWindows) +
	*new TMenuItem("~R~esize/move", cmResize, kbCtrlF5, hcWSizeMove,
		       "Ctrl-F5") + *new TMenuItem("~Z~oom", cmZoom, kbF5,
						   hcWZoom,
						   "F5") +
	*new TMenuItem("~P~rochaine", cmNext, kbF6, hcWNext,
		       "F6") + *new TMenuItem("~F~ermer", cmClose, kbAltF3,
					      hcWClose,
					      "Alt-F3") +
	*new TMenuItem("~T~ile", cmTile, kbNoKey,
		       hcWTile) + *new TMenuItem("C~a~scade", cmCascade,
						 kbNoKey, hcWCascade);


    TSubMenu & sub4 =
	*new TSubMenu("~T~ableau", 0, hcOptions) +
	*new TMenuItem("~O~uvrir", cmshOpen, kbF3, hcNoContext) +
	*new TMenuItem("~S~ave", cmshSave, kbF2, hcNoContext) +
	*new TMenuItem("~P~rochaine", cmshNext, '+', hcNoContext) +
	*new TMenuItem("~P~rev", cmshPrev, '-', hcNoContext) +
	*new TMenuItem("Op~t~ions", cmshOptions, kbNoKey, hcNoContext) +
	*new TMenuItem("~T~rouver", 0,
		       new TMenu(*new
				 TMenuItem("~N~ext Cell", cmshFindAny,
					   kbNoKey, hcNoContext, NULL,
					   new TMenuItem("~N~ext Formula",
							 cmshFindFor,
							 kbNoKey,
							 hcNoContext, NULL,
							 new
							 TMenuItem
							 ("~N~ext Text",
							  cmshFindFor,
							  kbNoKey,
							  hcNoContext,
							  NULL,
							  new
							  TMenuItem
							  ("~N~ext Number with time",
							   cmshFindTnum,
							   kbNoKey,
							   hcNoContext,
							   NULL,
							   new
							   TMenuItem
							   ("~N~ext Formula with time",
							    cmshFindTfor,
							    kbNoKey,
							    hcNoContext,
							    NULL,
							    new
							    TMenuItem
							    ("~C~oupled cell for HMI element",
							     cmshFindHMICell,
							     kbNoKey,
							     hcNoContext)
							   )
							  )
							 )
					   )
				 )
		       )
		       , hcNoContext) +
//    )+
	*new TMenuItem("~M~ark", 0,
		       new TMenu(*new
				 TMenuItem("~C~ell", cmshMark, kbAltM,
					   hcNoContext, NULL,
					   new TMenuItem("Re~f~erences",
							 cmshMarkRef,
							 kbNoKey,
							 hcNoContext, NULL,
							 new
							 TMenuItem
							 ("~R~ange",
							  cmshMarkArea,
							  kbNoKey,
							  hcNoContext)
					   )
				 )
		       )
		       , hcNoContext) +
	*new TMenuItem("~A~rea", 0,
		       new TMenu(*new
				 TMenuItem("~D~elete", cmshDelArea,
					   kbNoKey, hcNoContext, NULL,
					   new TMenuItem("~C~opy",
							 cmshCopyArea,
							 kbAltK,
							 hcNoContext, NULL,
							 new
							 TMenuItem
							 ("~M~ark",
							  cmshMarkArea,
							  kbNoKey,
							  hcNoContext)
					   )
				 )
		       )
		       , hcNoContext);


    r.b.y = r.a.y + 1;
    TSubMenu & sub5 =
	*new TSubMenu("~V~oir", 0, hcWindows) +
	*new TMenuItem("~D~rivers", cmViewDrivers, kbNoKey, hcNoContext,
		       "") + *new TMenuItem("~R~ecorders", cmViewRecorders,
					    kbNoKey, hcNoContext, "");

    return (new TMenuBar(r, sub1 + sub2 + sub3 + sub4 + sub5));
}

TDialog *lOptionsDialog()
{
    TRect R;
#define normScreenWidth 80        
#define optionsDialogLeft 15
#define optionsDialogRight normScreenWidth-optionsDialogLeft
#define optionsBoxWidth optionsDialogRight-optionsDialogLeft-1
#define optionsBoxHeight 10    
    R = TRect(optionsDialogLeft, 1, optionsDialogRight, optionsBoxHeight+9);    
    TDialog *D = new TDialog(R, "Options tableau de calculacion");
    R = TRect(1, 2, optionsBoxWidth, optionsBoxHeight);
    TView *aux = new TCheckBoxes(R,
		new TSItem ("Voir rappresentation interne du formule",
		new TSItem ("Voir ~F~ormule en lieu de results",
		new TSItem ("Voir F~l~ags",
		new TSItem ("Voir ~E~tats",
		new TSItem ("~H~ex Numbers",
		new TSItem ("Display Input ~S~ources",
		new TSItem ("Copie n'adapte pas les cordonnees",
		new TSItem ("Voir ~C~lient Flags",
		NULL))))))))
	);

    D->insert(aux);
    aux = new TInputLine(TRect(19, optionsBoxHeight, 26, 1+optionsBoxHeight), 5, NULL);
    D->insert(aux);
    D->insert(new TLabel(TRect(1, optionsBoxHeight, 16, 1+optionsBoxHeight), "Column Width", aux));

    D->insert(aux);
    aux = new TInputLine(TRect(19, 1+optionsBoxHeight, 26, 2+optionsBoxHeight), 5, NULL);
    D->insert(aux);
    D->insert(new TLabel(TRect(1, 1+optionsBoxHeight, 14, 2+optionsBoxHeight), "Decimals", aux));

    R = TRect(9, 5+optionsBoxHeight, 19, 7+optionsBoxHeight);
    D->insert(new TButton(R, "~A~ccepter", cmOK, bfDefault));
    R = TRect(21, 5+optionsBoxHeight, 33, 7+optionsBoxHeight);
    D->insert(new TButton(R, "~C~ancel", cmCancel, bfNormal));
    return (D);
}


void initLanguage()
{
    LangMenu = &langMenu;
    LangStatus = &langStatus;
    scLang = &scL;
    LangOptionsDialog = &lOptionsDialog;
}
