/*
 Qualcuno che parla l'Italiano perfettamente deve fare una corretura
 di questo.
 
 Helper functions for messages in the spreadsheet viewer. Maybe replaced
 by something which allows to change language without recompilation.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001. 

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include "config.h"
#define Uses_TRect
#define Uses_TStatusLine
#define Uses_TStatusDef
#define Uses_TStatusItem
#define Uses_TKeys
#define Uses_TMenuBar
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuItem
#define Uses_TDialog
#define Uses_TSItem
#define Uses_TCheckBoxes
#define Uses_TInputLine
#define Uses_TLabel
#define Uses_TButton
#define Uses_TView

#include <tvision/tv.h>

#include "cell.h"
#include "helpers.h"
#include "lib0.h"
#include "shcommands.h"
#include "viewcommands.h"
#include "hcontext.h"
#include "help.h"
#include "lang.h"

#include "vi_string.h"

string scL(int sc)
{
    switch (sc) {
    case scShLeer:
	return ("vuoto ");
    case scShGoto:
	return ("ire a");
    case scShText:
	return ("testo ");
    case scShZahl:
	return ("numero");
    case scShForm:
	return ("formula");
    case scShLastChange:
	return ("ultimo cambio:");
    case scShError:
	return ("ERRORE");
    case scShFOR:
	return ("FOR");
    case scShRES:
	return ("RES");
    case scShSave:
	return ("Salvare!");
    case scShTitle:
	return ("Tavolo di calcolazione");
    case scShPageNumber:
	return ("Tb.");
    case scShAskCreate:
	return ("Creare nuovo tavolo ?");
    case scShNoArea:
	return ("Per favore selezzionare una regione.");
    case scShNoSheet:
	return ("Non esiste uno tavolo numero. ");
    case scShDelArea:
	return ("Davvero cancellare il contenuto di questa regione");
    case scShClose:
	return ("Tavolo contiene cambi non salvati. Finire davvero");

    case scOfSourcet:
	return ("Dati entrados a mano");
    case scOfSourcet + soExternal:
	return ("Dati da sorgente esterna");
    case scOfSourcet + soExternal2:
	return ("Dati da sorgente esterna 2");
    case scOfSourcet + soAcknowledge:
	return ("Confirmazione");
    case scOfSourcet + soRecipe:
	return ("Dati da ricetta");
    case scOfSourcet + soBasicVar:
	return ("Valore di variabile BASIC");
    case scOfSourcet + soSetByHMI:
	return ("Valore di elemento HMI");
    case scOfSourcet + soPushButton:
	return ("flag Pushbutton premuto");
    case scOfSourcet + soPushBAck:
	return ("Butone processato");
    case scOfSourcet + soIoWorkSpace:
	return ("Variabile E/S");
    case scOfSourcet + soIoState:
	return ("Stato di errore E/S");
    case scOfSourcet + soUDFmodule:
	return ("da uno modulo definito per il utente");


    case scOfMath + ceDiv0:
	return ("divisione per zero");
    case scOfMath + ceLogNeg:
	return ("logaritmo di numero<=0");
    case scOfMath + ceWrongType:
	return ("Tipo di cellula falso");
    case scOfMath + ceMissing:
	return ("Cellula non esistente");
    case scOfMath + ceRootNeg:
	return ("Radice quadrato di numero negativo");
    case scOfMath + ceSucc:
	return ("Errore consecutivo");
    case scOfMath + ceMissingSheet:
	return ("Tavolo non esistente");

    case scOfMenus:
	return ("\003Visual Version" VERSION" alpha\n\n" "\003Copyright (c) 2001 Thomas Hergenhahn\n"	// The \003 centers the line.
		"\003Thomas.Hergenhahn@web.de\n\n"
		"\003fatto con TurboVision\n"
		"\003Copyright (c) 1994 Borland International\n"
		"\003Modificato per Sergio Sigala <ssigala@globalnet.it>\n");
	case scOfMenus+1:
	    return ("");	

    default:
	return ("Bisogna aggiungere messagio sc" + digits(sc));
    }
}

TStatusLine *langStatus(TRect r)
{
    return (new TStatusLine(r,
			    *new TStatusDef(0, 50) +
			    *new TStatusItem("~F1~ Aiuto", kbF1, cmHelp) +
			    *new TStatusItem("~Alt-X~ Uscire", kbAltX,
					     cmQuit) + *new TStatusItem(0,
									kbAltF3,
									cmClose)
			    + *new TStatusItem(0, kbF10,
					       cmMenu) +
			    *new TStatusItem(0, kbF5,
					     cmZoom) + *new TStatusItem(0,
									kbCtrlF5,
									cmResize)
			    + *new TStatusDef(hcShViewer,
					      hcShViewer) +
			    *new TStatusItem("~F1~ Aiuto", kbF1,
					     cmHelp) +
			    *new TStatusItem("~F2~ Salvare", kbF2,
					     cmshSave) +
			    *new TStatusItem("~Alt-E~ Editar", kbAltE,
					     cmshEditCell) +
			    *new TStatusItem("~Alt-L~ Label", kbAltL,
					     cmshEditLabel) +
			    *new TStatusItem("~Alt-G~ Andar a", kbAltG,
					     cmshGoto) +
			    *new TStatusItem("~Alt-C~ Cancellare", kbAltC,
					     cmshDelete) +
			    *new TStatusItem("~Alt-R~ Regione", kbAltR,
					     cmshArea) +
			    *new TStatusItem(0, kbAltF3,
					     cmClose) + *new TStatusItem(0,
									 kbF10,
									 cmMenu)));
}

TMenuBar *langMenu(TRect r)
{
    TSubMenu & sub1 =
	*new TSubMenu("~\360~", 0, hcSystem) +
	newLine() +
	*new TMenuItem("~S~obre eso...", cmAboutCmd, kbNoKey, hcSAbout);

    TSubMenu & sub2 = *new TSubMenu("~F~ile", 0, hcFile) +
//        *new TMenuItem( "~O~pen...", cmOpenCmd, kbF3, hcFOpen, "F3" ) +
	newLine() + *new TMenuItem("~U~scire", cmQuit, kbAltX, hcFExit,
				   "Alt-U");


    TSubMenu & sub3 =
	*new TSubMenu("~F~inestra", 0, hcWindows) +
	*new TMenuItem("~R~esize/mover", cmResize, kbCtrlF5, hcWSizeMove,
		       "Ctrl-F5") + *new TMenuItem("~Z~oom", cmZoom, kbF5,
						   hcWZoom,
						   "F5") +
	*new TMenuItem("~P~rossimo", cmNext, kbF6, hcWNext,
		       "F6") + *new TMenuItem("~C~hiudere", cmClose, kbAltF3,
					      hcWClose,
					      "Alt-F3") +
	*new TMenuItem("~T~ile", cmTile, kbNoKey,
		       hcWTile) + *new TMenuItem("C~a~scadare", cmCascade,
						 kbNoKey, hcWCascade);


    TSubMenu & sub4 =
	*new TSubMenu("~T~avolo", 0, hcOptions) +
	*new TMenuItem("~A~prire", cmshOpen, kbF3, hcNoContext) +
	*new TMenuItem("~S~alvare", cmshSave, kbF2, hcNoContext) +
	*new TMenuItem("~P~rossimo", cmshNext, '+', hcNoContext) +
	*new TMenuItem("~P~recedente", cmshPrev, '-', hcNoContext) +
	*new TMenuItem("Op~z~ioni", cmshOptions, kbNoKey, hcNoContext) +
	*new TMenuItem("~C~ercare", 0,
		       new TMenu(*new
				 TMenuItem("~P~rossima cellula", cmshFindAny,
					   kbNoKey, hcNoContext, NULL,
					   new TMenuItem("Prossima ~F~ormula",
							 cmshFindFor,
							 kbNoKey,
							 hcNoContext, NULL,
							 new
							 TMenuItem
							 ("Prossimo ~T~exto",
							  cmshFindFor,
							  kbNoKey,
							  hcNoContext,
							  NULL,
							  new
							  TMenuItem
							  ("Prossimo ~N~umero temporizato",
							   cmshFindTnum,
							   kbNoKey,
							   hcNoContext,
							   NULL,
							   new
							   TMenuItem
							   ("Prossima ~F~ormula temporizata",
							    cmshFindTfor,
							    kbNoKey,
							    hcNoContext,
							    NULL,
							    new
							    TMenuItem
							    ("~C~ellula ligata ad elemento HMI",
							     cmshFindHMICell,
							     kbNoKey,
							     hcNoContext)
							   )
							  )
							 )
					   )
				 )
		       )
		       , hcNoContext) +
//    )+
	*new TMenuItem("~M~arcare", 0,
		       new TMenu(*new
				 TMenuItem("~C~ellula", cmshMark, kbAltM,
					   hcNoContext, NULL,
					   new TMenuItem("Re~f~erenza",
							 cmshMarkRef,
							 kbNoKey,
							 hcNoContext, NULL,
							 new
							 TMenuItem
							 ("~R~egione",
							  cmshMarkArea,
							  kbNoKey,
							  hcNoContext)
					   )
				 )
		       )
		       , hcNoContext) +
	*new TMenuItem("~R~egione", 0,
		       new TMenu(*new
				 TMenuItem("~C~ancellare", cmshDelArea,
					   kbNoKey, hcNoContext, NULL,
					   new TMenuItem("~C~opiare",
							 cmshCopyArea,
							 kbAltK,
							 hcNoContext, NULL,
							 new
							 TMenuItem
							 ("~M~arcare",
							  cmshMarkArea,
							  kbNoKey,
							  hcNoContext)
					   )
				 )
		       )
		       , hcNoContext);


    r.b.y = r.a.y + 1;
    TSubMenu & sub5 =
	*new TSubMenu("~M~ostrar", 0, hcWindows) +
	*new TMenuItem("~D~rivers", cmViewDrivers, kbNoKey, hcNoContext,
		       "") + *new TMenuItem("~R~ecorders", cmViewRecorders,
					    kbNoKey, hcNoContext, "");

    return (new TMenuBar(r, sub1 + sub2 + sub3 + sub4 + sub5));
}

TDialog *lOptionsDialog()
{
    TRect R;
#define normScreenWidth 80        
#define optionsDialogLeft 15
#define optionsDialogRight normScreenWidth-optionsDialogLeft
#define optionsBoxWidth optionsDialogRight-optionsDialogLeft-1
#define optionsBoxHeight 10    
    R = TRect(optionsDialogLeft, 1, optionsDialogRight, optionsBoxHeight+9);    
    TDialog *D = new TDialog(R, "Opzioni tavolo di calcolazione");
    R = TRect(1, 2, optionsBoxWidth, optionsBoxHeight);    
    TView *aux = new TCheckBoxes(R,
	new TSItem ("Visualizzare rappresentazione interna di formule",
	new TSItem ("Visualizzare ~F~ormule in luogo di risultati",
	new TSItem("Mostrar F~l~ags",
	new TSItem("Visualizzare ~S~tati",
	new TSItem("Numeri ~h~ex",
							  new
							  TSItem
							  ("Visualizzare ~S~orgenti",
	new TSItem ("Non addattare coordinati in funzione di copiare",
	new TSItem ("Mostrar ~C~lient Flags",
		 NULL))))))))
	);

    D->insert(aux);
    aux = new TInputLine(TRect(19, optionsBoxHeight, 26, 1+optionsBoxHeight), 5, NULL);
    D->insert(aux);
    D->insert(new TLabel(TRect(1, optionsBoxHeight, 16, 1+optionsBoxHeight), "Spessore colonne", aux));

    D->insert(aux);
    aux = new TInputLine(TRect(19, 1+optionsBoxHeight, 26, 2+optionsBoxHeight), 5, NULL);
    D->insert(aux);
    D->insert(new TLabel(TRect(1, 1+optionsBoxHeight, 14, 2+optionsBoxHeight), "Decimali", aux));

    R = TRect(9, 5+optionsBoxHeight, 19, 7+optionsBoxHeight);
    D->insert(new TButton(R, "~A~cettare", cmOK, bfDefault));
    R = TRect(21, 5+optionsBoxHeight, 33, 7+optionsBoxHeight);
    D->insert(new TButton(R, "~C~ancelare", cmCancel, bfNormal));
    return (D);
}


void initLanguage()
{
    LangMenu = &langMenu;
    LangStatus = &langStatus;
    scLang = &scL;
    LangOptionsDialog = &lOptionsDialog;
}
