/*
 Implementation of a data logger. This one uses an older file format I
 created in a former project. It is here only for backward compatibility. 

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <sys/time.h>
//#include "base.h"
#include "persist.h"
#include "lib0.h"
#include "cellList.h"
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#define debug 0
#define ThisModule "log_md7  : "

#include "driverinfo.h"
#include "plclog.h"
#include <math.h>

#include "module.h"

extern "C" void setup(void *);
/*
    This may be redefined when there is a solution where to store old time/value
    for each coord from a range. Note that recordinstruction::read reads endCell
    and discards it then to keep file compatible.
*/
//#define handleCoordRange


class recordInstruction:public moduleInstruction {
    DECLARE_PERSISTENCE(recordInstruction)
  public:
    float signiTime;		// time difference that will be considered significant
    float signiValue;		// value difference that will be considered significant
    double lastValue;		// last recorded value
    int lastTime;		// time when last value was recorded
    bool firstCall;		// true on start
     recordInstruction();
    virtual void Read(Engine & e);
    virtual bool subscribe(Module * mod);
    virtual void process(ClientInfo * ci, Module * mod, int flags);
};

class aModule:public Module {
  public:
    aModule(DriverInfo * di) {
	Module::init(di);
    }
    virtual void addInstructionType(){
#if debug >4    
	LOG1(ThisModule "add Instruction Type\n");
#endif	
        TypeManager::Add("RECORD", CreateNewrecordInstruction);
    }	
};

void setup(void *);

#pragma pack(1)
struct datarecord {
    coord c;
    float v;
    int m;
    word s;
    word res;
};

struct dataheader {
    int Version;
    word color;
    word mode;
    int AvrSec;		// Seconds for average
    int ExactMin;		// Minutes to keep. Not used any more
    int MaxAgeDays;		// Delete data older than this
    int Min, Max;
    				// Limits for valid values
     word Signi_value;		// Wert in 1/1000 = Promille/10 = Prozent/100 }
    word Signi_time;		// seconds
    // Threshold of significance, distance from previous
    // value
    int Reserve[19];
    char nl, Name[255];
};

bool recordInstruction::subscribe(Module * mod) {
    if (moduleInstruction::subscribe(mod)) {
#if debug >4
	LOG4(ThisModule "I mark: %d:%s my bit mask: %x\n", sheet, cellN(Cell), mod->changedMask);
	FLUSH;
#endif		
	string pp = ((aModule*)mod)->path + digits(sheet) + "/" + cellN(Cell) + ".md7";
#if debug >4
	LOG2(ThisModule "I try to open: %s\n", pp.c_str());
	FLUSH;
#endif	
	int fd = open(pp.c_str(), O_RDWR | O_CREAT, 0644);
#if debug >4	
	LOG3(ThisModule "fd: %d error: %s\n", fd, strerror(errno));
#endif	
	int pos = lseek(fd, 0, SEEK_END);
#if debug >4	
	LOG2(ThisModule "seek_end returned: %d\n", pos);
#endif	
	if (pos == 0) {
	    cell *c = sheets[sheet]->find(Cell);
	    if (c != NULL) {
		dataheader h;
		memset(&h, 0, sizeof(h));
		h.nl = c->Label.length();
		strncpy(&h.Name[0], c->Label.c_str(), c->Label.length());
		h.Signi_time = (int)signiTime;
		h.Signi_value = (int) (10000.0 * signiValue + 0.5);
		write(fd, &h, sizeof(h));
	    }	
	}
	close(fd);
	return true;
    } else return false;
}

void recordInstruction::process(ClientInfo * ci, Module * mod, int flags) {
#if debug >4
	LOG3(ThisModule"process %s %p",cellN(ci->c),mod);
        FLUSH;
#endif	
        bool doRec;
	if (firstCall) {
	    firstCall = false;
	} else {
	    double tdiff =
		(time(NULL) - lastTime) / signiTime;
/*
 *	We want to know relative change of value. We refer to old value. If
 *	this is 0.0 we cannot divide and use new value instead. If this is 
 *	0.0 too, we cannot divide and use divisor 1.0.
 */
	    double ref = lastValue;
	    if (ref == 0.0)
		ref = (ci->c->getVal() != 0.0);
	    if (ref == 0.0)
		ref = 1.0;
	    double vdiff =
		(ci->c->getVal() - lastValue) / ref / signiValue;
	    double distance = sqrt(tdiff * tdiff + vdiff * vdiff);
#if debug >4
	    LOG5(ThisModule "old %g %d , new %g %ld\n", lastValue,
		   lastTime, ci->c->getVal(), time(NULL)
		);
	    LOG4(ThisModule
		   "norm'ed tdiff %g norm'ed vdiff %g dist. %g\n", tdiff,
		   vdiff, distance);
	    FLUSH;
#endif
	    doRec = (distance >= 1.0);	// significant change in data/time
	}
    if (doRec) {
	datarecord r;
	r.c = ci->c->k;
	r.v = ci->c->getVal();
	r.res = ci->c->getState();
	r.m = time(NULL) + 3600 + 507427200;

	r.s = r.m % 60;
	r.m = r.m / 60;
	string pp = mod->path + digits(ci->sheet) + "/" + cellName(ci->c->k) + ".md7";
	int fd = open(pp.c_str(), O_WRONLY | O_APPEND);
#if debug >3
	LOG2(ThisModule "open %s\n", pp.c_str());
#endif	
	write(fd, &r, sizeof(r));
	lastTime = time(NULL);
	lastValue = ci->c->getVal();
#if debug >3
	LOG3(ThisModule "fd: %d %s\n", fd, strerror(errno));
#endif
	close(fd);
	FLUSH;
    }
}

void setup(void *arg)
{
    DriverInfo *di = (DriverInfo *) arg;
#if debug >4    
    LOG3(ThisModule "enter setup. list: %s path: %s\n",
	   di->s1.c_str(), di->s.c_str());
#endif	   
    Module *drv = new Module(di);
    if (!drv)
	LOG1(ThisModule "no instance generated!\n");
    FLUSH;
}



recordInstruction::recordInstruction()
{
};

IMPLEMENT_PERSISTENCE(recordInstruction, "RECORD")

void recordInstruction::Read(Engine & e)
{
    string stc, etc;
    e >> comment >> sheet >> stc >> etc >> signiTime >> signiValue;
    Cell = getPos(stc);
#ifdef handleCoordRange
    endCell = getPos(etc);
#endif
    lastValue = 0;
    lastTime = 0;
    firstCall = true;
}

/*
    Changes:
    
    06/20/2002  compares amount of change since last recording and records
		only when combined distance of time and data is outside
		a circle given by signiTime/signiValue. Cannot do this
		with whole ranges of coordinates.
    06/20/2002 	* got rid of warning
    04/14/2003 	Moved common module code to module.cc
    05/20/2003 	Bug fixes, removed members variables which are already defined in module.h
*/
