/*
 Implementation of a data logger that stores data into a MySQL database.
 
 Host,user,password and database are common to all instructions and must be 
 specified in a comma separated list in visual.cfg.
 The table name is specified per instruction. It is assumed that the table is
 layed out like this example:
 
#
# Table structure for table 'testtable1'
#

CREATE TABLE testtable1 (
  id int(11) NOT NULL auto_increment,
  date datetime NOT NULL default '0000-00-00 00:00:00',
  sheet int(11) NOT NULL default '0',
  cell varchar(10) NOT NULL default '',
  value double NOT NULL default '0',
  PRIMARY KEY  (id)
) TYPE=MyISAM;

#
# Dumping data for table 'testtable1'
#

INSERT INTO testtable1 VALUES (1,'2003-03-26 22:15:19',5,'I7',11);
INSERT INTO testtable1 VALUES (2,'2003-03-26 22:15:22',5,'I7',15);
 

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <sys/time.h>
//#include "base.h"
#include "persist.h"
#include "lib0.h"
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include "cellList.h"
#include "mytypes.h"
#include "list.h"

#include "vi_string.h"

#include "driverinfo.h"
#include "config.h"

extern "C" {
#include <mysql/mysql.h>
}

#include "plclog.h"
#include "module.h"

#define debug 10

extern "C" void setup(void *);

#define ThisModule "log_mysql: "
#pragma pack(1)

class recordInstruction: public moduleInstruction {
    DECLARE_PERSISTENCE(recordInstruction)
  public:
    string table;
    recordInstruction();
    virtual void Read(Engine & e);
    virtual void process(ClientInfo * ci, Module * mod, int flags);
};

class aModule:public Module {
  public:
    string host;
    string user;
    string passwd;
    string database;
    virtual void init(DriverInfo * di);
    virtual ~aModule() {};				//pro forma destrutor
    MYSQL connection;
    aModule (DriverInfo* di) {
	LOG1(ThisModule "aModule(DriverInfo)\n");
	init(di);
    }
    virtual void addInstructionType(){
	LOG1(ThisModule "add Instruction Type\n");
        TypeManager::Add("RECORD", CreateNewrecordInstruction);
    }
};

void aModule::init(DriverInfo * di)
{
    string s = di->s;
    int p = s.Find(",");
    host = noSpace(s.before(p));
    LOG2(ThisModule "host: %s\n", host.c_str());
    s = s.after(p);
    p = s.Find(",");
    user = noSpace(s.before(p));
    LOG2(ThisModule "user: %s\n", user.c_str());
    s = s.after(p);
    p = s.Find(",");
    passwd = noSpace(s.before(p));
    LOG2(ThisModule "password: %s\n", passwd.c_str());
    database = noSpace(s.after(p));
    LOG2(ThisModule "database: %s\n", database.c_str());

    Module::init(di);
    
    if (!mysql_init(&connection)) {
	LOG1(ThisModule "Error initializing client.\n");
    } else {
	LOG1(ThisModule "Client initialized.\n");
    }
    FLUSH;
    if (!mysql_real_connect(&connection, host.c_str(),	// host
			    user.c_str(),	// user
			    passwd.c_str(),	// password
			    database.c_str(),	//"testdata", // database
			    0,	// port
			    NULL,	// unix socket
			    0	// client flags         
	)) {
	LOG2(ThisModule "Connection error.%s\n",
	       mysql_error(&connection));
    } else {
	LOG1(ThisModule "Connection established.\n");
    }
}

void recordInstruction::process(ClientInfo * ci, Module * mod, int flags)
{
    aModule * m = (aModule *) mod;
#if debug >3
    LOG4(ThisModule " %d:%s changed to: %g\n",
	   ci->sheet, cellName(ci->c->k).c_str(), ci->c->getVal());
#endif
    char SQL[200];
    sprintf(SQL, "INSERT into %s values(0,NOW(),%d,'%s',%g)",
	    table.c_str(), 
	    ci->sheet, cellN(ci->c), ci->c->getVal());
#if debug >3
    LOG2(ThisModule "I will do %s\n", SQL);
#endif
    int qres = mysql_real_query(&(m->connection), SQL, strlen(SQL));
    if (qres)
	LOG3(ThisModule "Query returned %d %s\n", qres,
	       mysql_error(&(m->connection)) );
#if debug >3
    LOG2(ThisModule "Query result %d\n", qres);
#endif
    FLUSH;
}

void setup(void *arg)
{
    DriverInfo *di = (DriverInfo *) arg;
    LOG3(ThisModule "enter setup. list: %s path: %s\n",
	   di->s1.c_str(), di->s.c_str());
    Module *drv = new aModule(di);
    if (!drv)
	LOG1(ThisModule "no instance generated!\n");
    FLUSH;
}

recordInstruction::recordInstruction()
{
    sheet = 1;
};

IMPLEMENT_PERSISTENCE(recordInstruction, "RECORD")

void recordInstruction::Read(Engine & e)
{
    string stc;
    e >> comment >> sheet >> stc >> table;
    Cell = getPos(stc);
}

/*
    Changes:
    
    04/14/2003 	Moved common module code to module.cc
    05/20/2003 	Bug fixes, removed members variables which are already defined in module.h
*/
