/*
 Implementation of a data logger that stores data into a MySQL database.
 
 Host,user,password and database are common to all instructions and must be 
 specified in a comma separated list in visual.cfg.
 
 Every instruction has an SQL statement. (Usually INSERT)
 The table name is specified within the SQL statement.
 The statement can have up to 20 variables. Variables are specified like with printf().
 The table layout must fit the SQL statement.

 
 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <sys/time.h>
//#include "base.h"
#include "persist.h"
#include "lib0.h"
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include "cellList.h"
#include "mytypes.h"
#include "list.h"

#include "vi_string.h"

#include "driverinfo.h"
#include "config.h"
extern "C" {
#include <mysql/mysql.h>
}

#include "plclog.h"
#include "module.h"

#define debug 5
#define ThisModule "log_mysql2:"

extern "C" void setup(void *);

class recordInstruction: public moduleInstruction {
    DECLARE_PERSISTENCE(recordInstruction)
  public:
    string SQL;			// an SQL statement to use
    recordInstruction();
    virtual void Read(Engine & e);
    virtual void process(ClientInfo * ci, Module * mod, int flags);
};

class aModule: public Module {
  public:
    string host;
    string user;
    string passwd;
    string database;
    virtual ~aModule() {};				//pro forma destrutor     
    MYSQL connection;
    virtual void init(DriverInfo * di);
    aModule (DriverInfo* di) {
	LOG1(ThisModule "aModule(DriverInfo)\n");
	init(di);
    }
    virtual void addInstructionType(){
	LOG1(ThisModule "add Instruction Type\n");
        TypeManager::Add("RECORD", CreateNewrecordInstruction);
    }	
};

void aModule::init(DriverInfo * di)
{
    string s = di->s;
    int p = s.Find(",");
    host = noSpace(s.before(p));
    LOG2(ThisModule "host: %s\n", host.c_str());
    s = s.after(p);
    p = s.Find(",");
    user = noSpace(s.before(p));
    LOG2(ThisModule "user: %s\n", user.c_str());
    s = s.after(p);
    p = s.Find(",");
    passwd = noSpace(s.before(p));
    LOG2(ThisModule "password: %s\n", passwd.c_str());
    database = noSpace(s.after(p));
    LOG2(ThisModule "database: %s\n", database.c_str());
    Module::init(di);
    
    if (!mysql_init(&connection)) {
	LOG1(ThisModule "Error initializing client.\n");
    } else {
	LOG1(ThisModule "Client initialized.\n");
    }
    FLUSH;
    if (!mysql_real_connect(&connection, host.c_str(),	// host
			    user.c_str(),	// user
			    passwd.c_str(),	// password
			    database.c_str(),	//"testdata", // database
			    0,	// port
			    NULL,	// unix socket
			    0	// client flags         
	)) {
	LOG2(ThisModule "Connection error.%s\n",
	       mysql_error(&connection));
    } else {
	LOG1(ThisModule "Connection established.\n");
    }
}

//void recordInstruction::process(ClientInfo * ci, Module * mod)
string insertValues(string s, int sheet)
{
    string result = "";
    string co = "";
    string format = "";
    double parms[20];
    coord c;
    bool ok;
    int pcount = 0, i;
    char SQL[500];

    LOG2(ThisModule "insertValues: %s\n", s.c_str());
    int fbeg = s.Find('"');
    s = s.after(fbeg);
    int fend = s.Find('"');
    format = s.substr(0,fend);
    s = s.from(fend + 2);
    LOG2(ThisModule "format: %s\n", format.c_str());
    LOG2(ThisModule "rest: %s\n", s.c_str());
    i = s.Find(',');
    while ((i > -1) && (!s.empty())) {
	co = s.substr(0,i);
	s = s.after(i);
	i = s.Find(',');
	LOG2(ThisModule "Variable: %s\n", co.c_str());
	LOG2(ThisModule "rest: %s\n", s.c_str());
	c = getPos(co);
	parms[pcount] = numContents(sheet, c, ok);
	pcount++;
    }
    co = s;
    LOG2(ThisModule "Variable: %s\n", co.c_str());
    c = getPos(co);
    parms[pcount] = numContents(sheet, c, ok);
    LOG4(ThisModule "Variables: %12.2f %12.2f %12.2f... \n", parms[0],
	   parms[1], parms[2]);
    sprintf(SQL, format.c_str(), parms[0], parms[1], parms[2], parms[3],
	    parms[4], parms[5], parms[6], parms[7], parms[8], parms[9],
	    parms[10], parms[11], parms[12], parms[13], parms[14],
	    parms[15], parms[16], parms[17], parms[18], parms[19]
	);
    result = SQL;
    return result;
}

struct sheet_cell_and_SQL {
    int sh;
    coord c;
    string *SQL;
};


void recordInstruction::process(ClientInfo * ci, Module * mod, int flags)
{
    aModule * m = (aModule *) mod;
    string sx = SQL;
    sx = insertValues(sx, sheet);
#if debug >3
    LOG2(ThisModule "I will do %s\n", sx.c_str());
#endif
    int qres = mysql_real_query(&(m->connection), sx.c_str(), sx.length());
    if (qres)
	LOG3(ThisModule "Query returned %d %s\n", qres, mysql_error(&(m->connection)));
#if debug >3
    LOG2(ThisModule "Query result %d\n", qres);
#endif
    FLUSH;
}

void setup(void *arg)
{
    DriverInfo *di = (DriverInfo *) arg;
    LOG3(ThisModule "enter setup. list: %s path: %s\n",
	   di->s1.c_str(), di->s.c_str());
    Module *drv = new aModule(di);
    if (!drv)
	LOG1(ThisModule "no instance generated!\n");
    FLUSH;
}

IMPLEMENT_PERSISTENCE(recordInstruction, "RECORD")

void recordInstruction::Read(Engine & e)
{
    string stc, etc;
    e >> comment >> SQL >> sheet >> stc;
    Cell = getPos(stc);
    LOG2(ThisModule "SQL %s\n", SQL.c_str());
}

/*
    Changes:

    04/14/2003 	Moved common module code to module.cc
    05/20/2003 	Bug fixes, removed members variables which are already defined in module.h
*/
