/*
 Implementation of a data logger, which stores data into a text file.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <sys/time.h>
//#include "base.h"
#include "persist.h"
#include "lib0.h"
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include "cellList.h"
#include "mytypes.h"
#include "list.h"

#include "vi_string.h"

#include "driverinfo.h"
#include "plclog.h"

#include "module.h"

extern "C" void setup(void *);

#define debug 14
#define ThisModule "log_text : "

#define queryStringBufferSize 2000

#define stINSERT 0
#define stSELECT 1

class recordInstruction: public moduleInstruction {
    DECLARE_PERSISTENCE(recordInstruction)
  public:
    int type;			// statement type
    string comment;		// a comment
    string SQL;			// an SQL statement to use
    int varCount;
    coord * Variables;		// memory space for coordinates		
    double * actValues;		// memory space for actual values    
    recordInstruction();
    void prepareVars();
    string insertValues();
    virtual void Read(Engine & e);
    virtual void process(ClientInfo * ci, Module * mod, int flags);
};

class aModule:public Module {
  public:
    aModule (DriverInfo* di) {
	LOG1(ThisModule "aModule()\n");
	Module::init(di);
    }
    virtual void addInstructionType(){
	LOG1(ThisModule "add Instruction Type\n");
        TypeManager::Add("RECORD", CreateNewrecordInstruction);
    }	
};

string recordInstruction::insertValues()
{
    bool ok;
    char buf[queryStringBufferSize];
    for (int i = 0; i < varCount; i++) {
	actValues[i] = numContents(sheet, Variables[i], ok);
    }
    vsnprintf(buf, queryStringBufferSize, SQL.c_str(), (char*)actValues);
    return string(buf);
}

void recordInstruction::process(ClientInfo * ci, Module * mod, int flags)
{
    string sx = insertValues();
    sx += "\n";
#if debug >3
    LOG2(ThisModule "I will process: %s\n", sx.c_str());
    LOG2(ThisModule "File:%s\n", mod->path.c_str());
#endif
    int fd = open(mod->path.c_str(), O_APPEND | O_RDWR, 0644);
    if (fd<0)
	fd = open(mod->path.c_str(), O_CREAT | O_RDWR,0644);
#if debug >3    
    LOG2(ThisModule "fd is: %d\n", fd);
#endif    
    int res = write(fd, sx.c_str(), sx.length());
#if debug >3    
    LOG3(ThisModule "write returned: %d %s\n", res, strerror(errno));
    FLUSH;
#endif    
    close(fd);
}

void setup(void *arg)
{
    DriverInfo *di = (DriverInfo *) arg;
    LOG3(ThisModule "enter setup. list: %s path: %s\n",
	   di->s1.c_str(), di->s.c_str());
    Module *drv = new Module(di);
    if (!drv)
	LOG1(ThisModule "no instance generated!\n");
    FLUSH;
}

void recordInstruction::prepareVars()
{
    string varList,varList2;
    string var = "";
    const char * p1;
    int i;
    int fbeg = SQL.Find('"');
    SQL = SQL.after(fbeg);
    int fend = SQL.Find('"');
    varList = SQL.from(fend + 2);
    SQL = SQL.before(fend);
    LOG2(ThisModule "prepareVars format : %s\n", SQL.c_str());
    LOG2(ThisModule "prepareVars varList: %s\n", varList.c_str());
    p1=varList.c_str();
    varCount=1;
    while (*p1) {
	if (*p1==',') varCount++;
	p1++;
    }
    LOG2(ThisModule "prepareVars varCount: %d\n", varCount);
    Variables=(coord*)malloc(varCount*sizeof(coord));
    actValues=(double *)malloc(varCount*sizeof(double));
//    varList=varList2;
    int vc=0;
    i = varList.Find(',');
    while ((i > -1) && (!varList.empty())) {
	var = varList.substr(0,i);
	varList = varList.after(i);
	i = varList.Find(',');
#if debug > 5	
	LOG2(ThisModule "prepareVars Variable: %s\n", var.c_str());
	LOG2(ThisModule "prepareVars varList: %s\n", varList.c_str());
#endif
	Variables[vc] = getPos(var);
	vc++;
    }
    LOG2(ThisModule "prepareVars last Variable: %s\n", varList.c_str());
    Variables[vc] = getPos(varList);
}


recordInstruction::recordInstruction()
{
};

IMPLEMENT_PERSISTENCE(recordInstruction, "RECORD")

void recordInstruction::Read(Engine & e)
{
    string stc;
    e >> comment >> SQL >> sheet >> stc;
    Cell = getPos(stc);
    LOG3(ThisModule "type: %d SQL %s\n", type, SQL.c_str());
    prepareVars();
}

/*
    Changes:
    
    04/14/2003 	Moved common module code to module.cc
    05/20/2003 	Bug fixes, removed members variables which are already defined in module.h
    02/13/2004 	More bug fixes.
*/
