/* Code by Paolo Baroni

 Implementation of a module to handle redundancy, interacting with another
 (and more in the future??) instance of VISUAL.


 Part of VISUAL, a human machine interface and data acquisition program

*/

#ifndef __redundancy2
#define __redundancy2

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <fstream.h>
#include "mytypes.h"
#include "cellList.h"
#include "cell.h"
#include <termios.h>
#include "setport.h"
#include "lib0.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <errno.h>
#include "nethelp.h"
#include "driverinfo.h"
#ifndef DONT_USE_GETHOSTBYNAME
#include <netdb.h>		// for gethostbyname
#endif

#include "vi_string.h"


// Each Visual's instance can be temporarily (or permanently) the master, depending on priority.

class Master: public client {
  public:
    bool IAmCurrentMaster;
    bool TRexist;  			// TRexist is needed to avoid multiple creations of Transfer Records 
    int priority;
    int port_trg;
    int Mask;
    int permfd;				// permfd is the file descriptor common to all the transfer
    string peer;			// operations to the spare. peer and port_trg refer to the other instance.
    int flagSheet;
    int priorSheet;			// an external sheet will host a "CurrentMaster" flag for other modules/drivers
    coord flagCell;
    coord priorCell;			// to check. Another one holds the priority value.
    string fakestart, fakestop;		// Strings used in the system commands to launch "fake" and spoof an IP address
    timeval LastTransf;
    timeval LastRec;			// maybe one timeval could be enough, but this way is clearer
    pthread_mutex_t * connectMutex;
    float signiValue;			// To reduce traffic, we'll send to the spare only values that differ
					// from the last value sent more than a certain percentual.
					// Only the value is checked. The spare can keep track of its own times?
    Master();
    virtual ~ Master() {};
    void setStatus();
    void recalcCallBack(ClientInfo * ci);
    void enterCallBack(ClientInfo * ci);
    void process(ClientInfo * ci);
    void markAllCells();
    void sendAllValues();
    int OpenSocket();
    void SignalToSpare();
    void MonitorActivity();           // Used when in spare status to monitor the other instance
};

class TransfRecord:public base {
  public:
    cell* c;
    float LastValSent;
    TransfRecord *nextTR;
    TransfRecord();
};


class TransfList:public list {       // one list for each sheet
  public:
    TransfRecord *recordList;
    TransfList();
    int insert(void *base);
    TransfRecord *find(coord k);
};

#endif

