/*
 This module implemets part of a state machine. If triggered by a change in
 some cell, it will change the state of another cell to a predefined value.
 If combined with spreadsheet formulas, this can be used to implement
 a primitive way of sequential programs.
 
 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <sys/time.h>
//#include "base.h"
#include "persist.h"
#include "lib0.h"
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include "cellList.h"
#include "mytypes.h"
#include "list.h"
#include "globals.h"

#include "vi_string.h"

#include "driverinfo.h"

#define debug 1
#define ThisModule "mod_stm   : "

#include "plclog.h"
#include "module.h"

extern "C" void setup(void *);
class stateChangeInst:public moduleInstruction {
    DECLARE_PERSISTENCE(stateChangeInst)
  public:
    string comment;
    int sheet;			// Sheet number to use
    coord Cell;			// Trigger cell to change
    double newValue;		// puts a limit on rows that will be copied to sheet
    coord  targetCell;		// Cell to change
    stateChangeInst(){};
    virtual void Read(Engine & e);
    virtual void process(ClientInfo * ci, Module * mod, int flags);
};

class aModule:public Module {
  public:
    aModule(DriverInfo * di);
    virtual ~aModule() {};				//pro forma destructor
};


void stateChangeInst::process(ClientInfo * ci, Module * mod, int flags) {
    if (ci->c->getVal()>0)
    setCell(sheet, targetCell, newValue, soUDFmodule);
}

void setup(void *arg)
{
    DriverInfo *di = (DriverInfo *) arg;
    LOG3(ThisModule "enter setup. list: %s path: %s\n",
	   di->s1.c_str(), di->s.c_str());
    Module *drv = new aModule(di);
    if (!drv)
	LOG1(ThisModule "no instance generated!\n");
    FLUSH;
}

IMPLEMENT_PERSISTENCE(stateChangeInst, "STM")

void stateChangeInst::Read(Engine & e)
{
    string stc,ttc;
    e >> comment >> sheet >> stc >> ttc >> newValue;
    Cell = getPos(stc);
    targetCell = getPos(ttc);
}

/*
    Changes:
    
    04/14/2003 	Moved common module code to module.cc
    05/20/2003 	Bug fixes, removed members variables which are already defined in module.h
*/
