/*
 This module stores short time trend data in memory.
 
 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <sys/time.h>
//#include "base.h"
#include "persist.h"
#include "plclog.h"
#include "lib0.h"
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include "cellList.h"
#include "mytypes.h"
#include "list.h"
//#include "persist.h"
#include "globals.h"

#include "vi_string.h"

#include "driverinfo.h"

#define debug 10
#define ThisModule "mod_trend : "

#include "module.h"
#include "mod_trend.h"

extern "C" void setup(void *);

class aModule:public Module {
  public:
    virtual ~aModule() {};	//pro forma destructor
    aModule (DriverInfo* di) {
	LOG1(ThisModule "aModule(DriverInfo)\n");
	init(di);
    }
    virtual void addInstructionType(){
	LOG1(ThisModule "add Instruction Type\n");
        TypeManager::Add("TREND", CreateNewtrendRecordInst);
    }
    virtual void setupAfterListLoad(){
	trendMemList=il;
    }
};

void trendRecordInst::process(ClientInfo * ci, Module * mod, int flags) {
#if debug >5
    LOG1(ThisModule "enter process\n");
    FLUSH;
#endif
/* 
    Test if we are called in valueChange. Note if so. Then we ignore calls from recalc from now.
*/					
    if (flags & 1) {
	calledInEnter=true; 
#if debug >5
	LOG1(ThisModule "called in setCell\n");
	FLUSH;
#endif	
    } else {
#if debug >5
	LOG1(ThisModule "called in recalc.\n");
	FLUSH;
#endif	    
/* 
    if not, test if we were ever called in enter (setCell()). If so, we ignore this call.
*/					
	if (calledInEnter) return;    
    }	
    trendMem[nextEntry].v=ci->c->getVal();
    struct timeval tv;
    gettimeofday(&tv, NULL);
    trendMem[nextEntry].time = tv.tv_sec + 1e-6 * tv.tv_usec;
    trendMem[nextEntry].number=number;	
    int ne=nextEntry+1;
    ne%=numEntries;
    nextEntry=ne;
    number++;
    if (usedEntries < numEntries) usedEntries++;
#if debug >5
    LOG2(ThisModule "%d entries\n", numEntries);
    FLUSH;
#endif
}

void setup(void *arg) {
    DriverInfo *di = (DriverInfo *) arg;
    LOG3(ThisModule "enter setup. list: %s path: %s\n",
	   di->s1.c_str(), di->s.c_str());
    Module *drv = new aModule(di);
    if (!drv)
	LOG1(ThisModule "no instance generated!\n");
    FLUSH;
}

IMPLEMENT_PERSISTENCE(trendRecordInst, "TREND")

void trendRecordInst::Read(Engine & e)
{
    string stc;
    e >> comment >> sheet >> stc>> numEntries;
    Cell = getPos(stc);
    trendMem=NULL;
    nextEntry=0;
    usedEntries=0;
    number=0;
}

bool trendRecordInst::subscribe(Module * mod)	// subscribe to a cell
{
//    if (moduleInstruction::subscribe(mod)) {
	trendMem=(trendData*)malloc(numEntries*sizeof(trendData));
#if debug >3
	LOG3(ThisModule "got %d bytes at %p.\n", numEntries*sizeof(float), trendMem);
	FLUSH;
#endif	
//	return true;
//    } else return false;
    return moduleInstruction::subscribe(mod);
}

/*
    Changes:

    04/14/2003 	Moved common module code to module.cc
    05/20/2003 	Bug fixes, removed members variables which are already defined in module.h
    08/03/2003 	Bug fixes
*/
