/*
 Definitions of some handy types for the PLC drivers.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#define debug 0

#include <sys/time.h>
#include "plcdrv.h"
#include "lib0.h"
#include "cellList.h"
#include "persist.h"
#include <fstream>
#include <stdio.h>
#include <unistd.h>
#include "plclog.h"

#include "lilo.h"

using std::fstream;
#define ThisModule "plcdrv   : "
/*
    Definition of a transport instruction for all PLC drivers.
    One instruction describes a block of data in PLC which is copied
    to or from a column of the spreadsheet beginning at coordinate
    startCell. Transport instructions are held in a list, which is 
    executed round and round by the driver. Conditional execution may be 
    achieved using controlCell. PLC drivers may extend this class.
*/

transferInstruction::transferInstruction()
{
    sheet = 1;
    startMW = 0;
    endMW = 1;
    direction = 0;		// 0=read from PLC, 1=write to PLC
    DBNumber = 0;		// PLC data block number or data area, 
    // interpretation depends on driver.
    startCell.row = 1;		// first cell, corresponds to startMW is stored here 
    // consecutive rows are used until endMW is reached. 
    startCell.col = 2;
    ctrlCell.row = 1;		// instruction is only executed is this cell exists
    // and has value != 0
    ctrlCell.col = 1;
};

//IMPLEMENT_PERSISTENCE(transferInstruction, "TRANSFERINST")

void transferInstruction::Read(Engine & e)
{
    string stc, ctc;
    e >> comment >> sheet >> startMW >> endMW >> direction >> DBNumber >>
	stc >> ctc;
    startCell = getPos(stc);
    ctrlCell = getPos(ctc);
}

PLCdrv::PLCdrv()
{
    DI=NULL;
    _PID = getpid();
}

void PLCdrv::load(DriverInfo * i)
{
    LOG2("load from %s\n", i->s1.c_str());
    FLUSH;
    params = i->s;
    handle = i->th;
/*
    fstream f((i->s1).c_str(), ios::in);
    Engine *e = new Engine(f);
    il = new list();
    base *c;
    do {
	*e >> c;
	if (c != 0)
	    il->insert(c);
    } while (c != NULL);
    delete e;
    f.close();
*/
    il =loadlist((i->s1).c_str());
}

void execTI1(void *ti, void *drv)
{
#if debug >0
//    int rfd=((PLCdrv *) drv)->rfd;
//    int _PID=((PLCdrv *) drv)->_PID;	// 11/04/04
    LOG2("enter exec TI1 %s\n",((transferInstruction *) ti)->comment.c_str());
    FLUSH;
#endif
    ((PLCdrv *) drv)->execTI(((transferInstruction *) ti));
}

void PLCdrv::execTI(transferInstruction * ti)
{
    LOG1("Hello from skeleton of execTI(). Overwrite me!\n");
}

void PLCdrv::execList()
{
#if debug >0
    LOG1("enter exec list\n");
    FLUSH;
#endif
    if(DI){
	LOG2("execute: looking for commands %d.\n",DI->commands);
	if(DI->commands) {
	    LOG1("Ooops, beginning to reload.\n");
	    unRegisterDriver(changedMask);
	    if (rfd>0){	
	        if ((wfd!=rfd) && (wfd>0)) close(wfd);
	        close(rfd);
	    }	
	    DI->commands=0;
	    intSetup(DI);
	    LOG2("Reload done. Commands: %d\n",DI->commands=0);
	}
    } else {
        LOG1("Ooops, thought I kept my DI!\n");
    }
			
    il->forEach(&execTI1, (void *) this);
}

void PLCdrv::clrPushButton(int sheet, coord & acell)
{
#if debug >10
    LOG2("clrPushButton sheet %d\n",sheet);
    FLUSH;
#endif    
    if (sheets[sheet]!=NULL) {
#if debug >0
	LOG3("clrPushButton cell %d:%s\n",sheet,cellN(acell));
	FLUSH;
#endif    
	cell * c=sheets[sheet]->find(acell);
	if (c!=NULL) {
	    c->setFlags(c->getFlags() & ~changedMask);
/*	    
	    if ((c->typeID() == tnCell)|| (c->typeID() == tnTimeCell)) {
		if (((nCell*)c)->sourceType==soPushButton) {
//		    ((nCell*)c)->value=0;
//		    ((nCell*)c)->sourceType=soPushBAck;
		    setCell(sheet, acell, 0, soPushBAck);
		}
	    }
*/	  
	    if (c->getSourceType()==soPushButton) {
		setCell(sheet, acell, 0, soPushBAck);
	    }  
	}
    }
//    clrFlags(sheet, cell, changedMask);
}

void PLCdrv::intSetup(DriverInfo * i) {
    LOG1("Hello from skeleton of intSetup(). Overwrite me!\n");
}
    
    void PLCdrv::hexDump(char * text,uc * buffer, int len) {
	int i;
	LOG2("%s",text);
	for (i = 0; i < len; i++) {
	    LOG_2("%x,", buffer[i]);
	    if ((i % 16) == 15)
	    LOG_1("\n");
	}
    }					
/*
    Changes:
    
    02/12/2002 	change output statements to printf, this gives less code size.
    05/13/2002 	Object Read & write functions return void now. 
    11/26/2002 	Implemented the clrPushButton method. Thought it hed already been ready...
    04/09/2003 	adaptions to changes in cell.h
    08/28/2004 	added hexDump()
*/
