/*
 Implementation of the network server. The servers MAIN thread is started by
 the main program. It starts in turn the ACCEPTER thread, which accepts a
 connection and passes the file descriptor to the MAIN thread. The server MAIN
 thread starts a PORT SERVER thread to handle the connection. Thus, the 
 PORT SERVER thread is NOT a child of ACCEPTER, but a brother and SIGCHILD goes
 to the server MAIN. If it would go to ACCEPTER, which is blocked in the 
 accept() system call, we would get "process interrupted in system call" and 
 that's it. Maybe its different with pthread library..?

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <pthread.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>

#include "cellList.h"
#include "lib0.h"
#include "mytypes.h"
#include "shsaver.h"
#include "nethelp.h"
#include "plcdrv.h"

#include "vi_string.h"

#include "plclog.h"

#define ThisModule "server   : "

#define debug 5

#include "server_extGetPos.cc"
#include "accepter.cc"
#include "server_toClient.cc"

void parseLine(int fd, string & line, int mask, int &sheet)
{
    int p, p2;
    string s;
    string s1;
    coord c;
    int otherSh = 1, decimals=12;
    double v;
    bool ok;

    p = line.Find("celltype");
    if (p >= 0) {
	s = line.from(p + 8);
	c = getPos(s);
#if debug > 0
	LOG2("rest %s\n", s.c_str());
	FLUSH;
#endif
	s = typeString(sheet, c) + "\n";
#if debug > 0
	LOG2("answer %s\n", s.c_str());
	FLUSH;
#endif
	write(fd, s.c_str(), s.length());
	return;
    }

    p = line.Find("getallvalues");
    if (p >= 0) {
	sendValues(fd, sheet);
	return;
    }

    p = line.Find("getchanges");
    if (p >= 0) {
	sendChanges(fd, sheet, mask);
	return;
    }

#include "server_getlist.cc"
#include "server_getAlarms.cc"
#include "server_get.cc"
#include "server_sheet.cc"
#include "server_set.cc"

    s = "cannot handle: " + line + "\n";
    goto returnS;
    
returnOK:
    s = "ok\n";    
returnS:
#if debug > 0
    LOG2(ThisModule "answer %s", s.c_str());
    FLUSH;
#endif
    write(fd, s.c_str(), s.length());
    return;
}

void *portServer(void *arg)
{
    portserver_info pi;
    PLCdrv *dummy = new PLCdrv;
    int changeMask = registerDriver("server", dummy);
    dummy->errorMask = changeMask;
//    dummy->PID=getpid();
    int rp = 0, wp = 0;		// buffer positions
    int sheet = 1;
    char buffer[bSize];
    string line;
    pi = *(portserver_info *) arg;
    LOG2(ThisModule "portServer: My fd is:%d\n", pi.fd);
    FLUSH;
    int waitCount = 0;
    while (waitCount < 300) {
	while (readline(pi.fd, &buffer[0], line, wp, rp) && (pi.fd > 0)) {
	    LOG2(ThisModule "port server: request:%s\n", line.c_str());
	    FLUSH;
	    parseLine(pi.fd, line, changeMask, sheet);
	    usleep(10000);
	}
	waitCount++;
    }
    LOG1(ThisModule "portServer: I closed my fd.\n");
    FLUSH;
    unRegisterDriver(changeMask);
//    destroy(dummy);
    return NULL;
}

#include "driverThread.cc"

/*
    Changes:
    
    02/12/2002  server uses port given in visual.cfg
    05/13/2002  Cosmetical changes to align debug output.
    06/27/2002  added missing returns in savesetting
    11/16/2002  clenup
*/
