/*
 Implementation of the network server. The servers MAIN thread is started by
 the main program. It starts in turn the ACCEPTER thread, which accepts a
 connection and passes the file descriptor to the MAIN thread. The server MAIN
 thread starts a PORT SERVER thread to handle the connection. Thus, the 
 PORT SERVER thread is NOT a child of ACCEPTER, but a brother and SIGCHILD goes
 to the server MAIN. If it would go to ACCEPTER, which is blocked in the 
 accept() system call, we would get "process interrupted in system call" and 
 that's it. Maybe its different with pthread library..?

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <pthread.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include "cellList.h"
#include "nethelp.h"
#include "mytypes.h"
#include "lib0.h"

#include "shsaver.h"

#include "plcdrv.h"
#include "ediCellList.h"
#include "plclog.h"

#include "vi_string.h"

#include "mod_trend.h"

#define ThisModule "server4  : "

#define debug 0
#define debugGetAlarms 5

#include "fastGetPos.cc"

#include "server_extGetPos.cc"
#include "accepter.cc"

#include "server_toClient.cc"

#include "server_sendTrendData.cc"    

void parseLine(int fd, string & line, int mask, int &sheet) //, int _PID)
{
#include "server_pl_vars.cc"
    
#include "server_trendData.cc"    
#include "server_remoteSheet.cc"

    p = line.Find("getallvalues");
    if (p >= 0) {
	sendValues(fd, sheet);
	return;
    }

    p = line.Find("getchanges");
    if (p >= 0) {
	sendChanges(fd, sheet, mask);
	return;
    }

#include "server_fast_getlist.cc"
#include "server_getAlarms.cc"
#include "server_textForRange.cc"

    p = line.Find("needSave");
    if (p >= 0) {
	s = " \n";
	if (sheets[sheet]) {
	    if (sheets[sheet]->changed) {
		s = "Save! \n";
	    }
	}
	goto returnS;
    }

    p = line.Find("INsrAnDTCeLl");

    if (p >= 0) {
#if debug > 0
	LOG2( "found Insert Random type cell: %d\n", p);
	FLUSH;
#endif
//	string s = line.from(p + 12);
	s = line.from(p + 12);

	p = s.Find("!");
	if (p >= 0) {
#if debug > 0
	    LOG2( "rest contains separator at: %d\n", p);
	    FLUSH;
#endif
	    s1 = s.before(p);
#if debug > 0
	    LOG2( "before separator: %s\n", s1.c_str());
	    FLUSH;
#endif
	    s = s.after(p);
#if debug > 0
	    LOG2( "after separator: %s\n", s.c_str());
	    FLUSH;
#endif
	    c = getPos(s1);

	    tempLabel = sheets[sheet]->symbolName(c, symOnly);
	    tempFlags = sheets[sheet]->getFlags(c, ok);
	    tempCFlags = sheets[sheet]->getCFlags(c, ok);
	    ((ediCellList *) sheets[sheet])->deleteCell(c);
	    if (!s.empty()) {
		((ediCellList *) sheets[sheet])->createCell(c, s,
							    soDirect);
	    }
//
// restore label and cFlags:
//    
	    ((ediCellList *) sheets[sheet])->
		insertLabelAndCFlags(c, tempLabel, tempCFlags);

	    s1 = "insert done.\n";
	    write(fd, s1.c_str(), s1.length());
	}			// sep. (!) found   
	return;
    }

#include "server_setSymbol.cc"
#include "server_get.cc"
#include "server_sheet.cc"
#include "server_createSheet.cc"
#include "server_format.cc"
#include "server_set.cc"
#include "server_savesheet.cc"

    s = "cannot handle: " + line + "\n";
    goto returnS;
    
returnOK:
    s = "ok\n";    
returnS:
#if debug > 0
    LOG2( "answer %s", s.c_str());
    FLUSH;
#endif
    write(fd, s.c_str(), s.length());
    return;
}

void *portServer(void *arg)
{
    portserver_info pi;
#if debug > 0	        
    int _PID=getpid();
#endif    
    PLCdrv *dummy = new PLCdrv;
    int changeMask = registerDriver("server", dummy);
    dummy->errorMask = changeMask;
//    dummy->PID=getpid();
    int rp = 0, wp = 0;		// buffer positions
    int sheet = 1;
    char buffer[bSize];
    string line;
    pi = *(portserver_info  *) arg;
#if debug > 0	  
    int request=1;  
    timeval t1, t2;
#endif    
    LOG2( "portServer: My fd is:%d\n", pi.fd);
    FLUSH;
    int waitCount = 0;
    while (waitCount < 300) {
	while (readline(pi.fd, &buffer[0], line, wp, rp) && (pi.fd > 0)) {
#if debug > 0	
	    gettimeofday(&t1, NULL);
	    LOG3( "port server: request %d: %s\n", request,line.c_str());
	    FLUSH;
#endif	    
	    parseLine(pi.fd, line, changeMask, sheet);//, _PID);
#if debug > 0		    
	    gettimeofday(&t2, NULL);
	    LOG3( "request %d took %ld usecs.\n", request, t2.tv_usec -
	       t1.tv_usec);
	    request++;   
#endif	       
	    usleep(10000);
	}
//	usleep(10000);
	waitCount++;
    }
    LOG1( "portServer: I closed my fd.\n");
    FLUSH;
    unRegisterDriver(changeMask);
//    destroy(dummy);
    return NULL;
}

#include "driverThread.cc"

/*
    Changes:
    
    02/12/2003  copied this from server3.cc. Replace C++ string functions by c code. See
		server_fast_getlist.cc.
    02/23/2003  set standard precision for numbers to 12 decimals.
    
*/
