/*
 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <termios.h>
#include <fcntl.h>
/* 
    include this to use sockets
*/    
#include <sys/socket.h>	
#include <sys/un.h>	
#include <netinet/in.h> 
#include <arpa/inet.h>	

#include <unistd.h>
#include "setport.h"
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include "vi_string.h"

#define debug 10
#define ThisModule "setPort  : "
#include "plclog.h"

#define DONT_USE_GETHOSTBYNAME

string getToken(string & params)
{
    int p = 0;
    do {
	p = params.Find(' ');
	if (p == 0)
	    params.erase(0, 1);
    } while (p == 0);
    string s = params.before(p);
    params.erase(0, p + 1);
    LOG3(ThisModule "%s rest %s\n", s.c_str(), params.c_str());
    return s;
}

struct portPair {
    int rfd;
    int wfd;
};

int setPort(string params, int * wfd)
{
    termios oldtio, newtio;
    string s;

    s = getToken(params);
    if (s=="socket") {
	string s1,s2;
	s1 = getToken(params);
	s2 = getToken(params);
	sockaddr_un addr;
	socklen_t addrlen;
	addr.sun_family = AF_UNIX;
	char * path=addr.sun_path;
	memset(path,0,108);
	memcpy(path,s1.c_str(),s1.length());
	LOG3(ThisModule "using socket %s as %s \n", path,s2.c_str());
	int fd=socket(AF_UNIX, SOCK_DGRAM, 0);
	addrlen = sizeof(addr);
	if (s2=="server") {
	    if (bind(fd, (sockaddr *) & addr, addrlen)) {
		LOG2(ThisModule "Socket error: %s \n", strerror(errno));
		close(fd);
		fd = failed_fd;
	    } else {
		LOG1(ThisModule "bound. \n");
	    }	
	}
	if (s2=="client") {
	    if (connect(fd, (sockaddr *) & addr, addrlen)) {
		LOG2(ThisModule "Socket error: %s \n", strerror(errno));
		usleep(1000000);
		if (connect(fd, (sockaddr *) & addr, addrlen)) {
		    LOG2(ThisModule "Socket error: %s \n", strerror(errno));
		    close(fd);
		    fd = failed_fd;
		}
	    } else {
		LOG1(ThisModule "Connected. \n");
//		int res=fcntl(fd,O_NONBLOCK);		// doesn't help. writing to a socket
//							// socket that is no longer connected
//							// throws SIGPIPE.	
//		LOG2(ThisModule "fcntl: %d \n", res);
	    }	
	}    
	FLUSH;
	*wfd=fd;
	return fd;
    }
    
    if (s=="tcp") {
	string hostS,portS,modeS;
	hostS = getToken(params);
	portS = getToken(params);
	modeS = getToken(params);
	sockaddr_in addr;
	socklen_t addrlen;
	addr.sin_family = AF_INET;
	int port=atol(portS.c_str());
	addr.sin_port =
	    (((port) & 0xff) << 8) | (((port) & 0xff00) >> 8);
#ifndef DONT_USE_GETHOSTBYNAME
	hostent *he = gethostbyname(hostS.c_str());
	memcpy(&addr.sin_addr, he->h_addr_list[0], sizeof(addr.sin_addr));
#else
	inet_aton(hostS.c_str(), &addr.sin_addr);
#endif
	LOG4(ThisModule "using tcp connection to %s port %s as %s \n", 
	    hostS.c_str(), portS.c_str(), modeS.c_str());
	int fd=socket(AF_INET, SOCK_STREAM, 0);
	LOG2(ThisModule "file descriptor: %d \n", fd);
	addrlen = sizeof(addr);
	if (modeS=="client") {
	    errno=0;
	    LOG1(ThisModule "trying to connect \n");
	    if (connect(fd, (sockaddr *) & addr, addrlen)) {
		LOG2(ThisModule "Socket error: %s \n", strerror(errno));
/*
		usleep(1000000);
		errno=0;
		
		if (connect(fd, (sockaddr *) & addr, addrlen)) {
		    LOG2(ThisModule "Socket error: %s \n", strerror(errno));
		    close(fd);
		    fd = failed_fd;
		}
*/		
		fd = failed_fd;		
	    } else {
		LOG1(ThisModule "Connected. \n");
//		int res=fcntl(fd,O_NONBLOCK);		// doesn't help. writing to a socket
//							// socket that is no longer connected
//							// throws SIGPIPE.	
//		LOG2(ThisModule "fcntl: %d \n", res);
	    }	
	}    
	FLUSH;
	*wfd=fd;
	return fd;
    }
    
    if (s=="fifos") {
	string s1,s2;
	s1 = getToken(params);
	s2 = getToken(params);
	LOG3(ThisModule "using fifos: read %s, write %s \n", s1.c_str(),s2.c_str());
	errno=0;	// 28.7.
	int fd=open(s1.c_str(), O_RDONLY| O_NOCTTY | O_NONBLOCK);
	LOG3(ThisModule "read fd %d %s\n", fd ,strerror(errno));
	FLUSH;
try_again:	
    	errno=0;	// 28.7.
	int fd2=open(s2.c_str(), O_WRONLY| O_NOCTTY | O_NONBLOCK);
	LOG4(ThisModule "read fd %d, write fd %d %s \n", fd, fd2, strerror(errno));
	FLUSH;
	if (fd2<0) {
	    LOG1(ThisModule "will retry to open write pipe later.\n");// 28.7.
	    usleep(1000000);
	    goto try_again;
	}    
	*wfd=fd2;
	return fd;
    }
    
    int fd = open(s.c_str(),
		  O_RDWR | O_NOCTTY | O_NONBLOCK);
#if debug >3
    LOG3(ThisModule "%s = %d\n", s.c_str(), fd);
//    int res =
#endif
	fcntl(fd, F_SETFL, 0x2000 /* FASYNC */ );
#if debug >3
    LOG2(ThisModule "fcntl: %s\n", strerror(errno));
#endif
    tcgetattr(fd, &oldtio);	// save current settings
#if debug >3
    LOG2(ThisModule "tcget...:%s\n", strerror(errno));
#endif
// set new port settings for noncanonical input processing:
    s = getToken(params);
    if (s == "115200")
	newtio.c_cflag = B115200;
    else if (s == "57600")
	newtio.c_cflag = B57600;
    else if (s == "38400")
	newtio.c_cflag = B38400;
    else if (s == "19200")
	newtio.c_cflag = B19200;
    else if (s == "9600")
	newtio.c_cflag = B9600;
    else if (s == "4800")
	newtio.c_cflag = B4800;
    else if (s == "2400")
	newtio.c_cflag = B2400;
    else if (s == "1200")
	newtio.c_cflag = B1200;
    else if (s == "600")
	newtio.c_cflag = B600;
    else if (s == "300")
	newtio.c_cflag = B300;
    else
	LOG2(ThisModule "illegal Baudrate: %s\n", s.c_str());

    s = getToken(params);
    if (s == "e")
	newtio.c_cflag = newtio.c_cflag & ~PARODD | PARENB;
    else if (s == "o")
	newtio.c_cflag = newtio.c_cflag | PARODD | PARENB;
    else if (s == "n")
	newtio.c_cflag = newtio.c_cflag & ~(PARODD | PARENB);
    else
	LOG2(ThisModule "illegal parity mode:%s\n", s.c_str());

    s = getToken(params);
    if (s == "8")
	newtio.c_cflag |= CS8;
    else if (s == "7")
	newtio.c_cflag |= CS7;
    else if (s == "6")
	newtio.c_cflag |= CS6;
    else if (s == "5")
	newtio.c_cflag |= CS5;
    else
	LOG2(ThisModule "illegal bit count:%s\n", s.c_str());

    newtio.c_cflag = newtio.c_cflag | CLOCAL | CREAD;
    newtio.c_iflag = 0;		// {IGNPAR |} ICRNL;
    newtio.c_oflag = ONOCR;
    newtio.c_lflag = 0;		//ICANON
    newtio.c_cc[VMIN] = 1;
    newtio.c_cc[VTIME] = 0;
    tcflush(fd, TCIFLUSH);
#if debug >3
    LOG2(ThisModule "tcflush:%s\n", strerror(errno));
#endif
    tcsetattr(fd, TCSANOW, &newtio);
#if debug >3
    LOG2(ThisModule "tcset...:%s\n", strerror(errno));
#endif
    *wfd=fd;
    return (fd);
}

/*
    Changes:
    
    02/12/2002 change output statements to printf, this gives less code size.
    06/21/2002 	* got rid of some warnings
*/
