/*
 Implements a loader to instatiate objects from a file.

 Part of VISUAL, a human machine interface and data acquisition program

 Much of this code is stolen from the sources of the CommonC++ library.
 
 CommonC++ is Copyright (C) 1999-2001 Open Source Telecom Corporation.
 Daniel Silverstone designed original Common C++ classes (persistence
	engine, math libs, etc...)
 
 The rest:  (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include "persist.h"
#include "mytypes.h"
#include "lib0.h"
#include <stdio.h>
#include "plclog.h"
#define debug 0

#define ThisModule "engine   : "

Engine::Engine(iostream & stream):myUnderlyingStream(stream)
{
    // Nothing else to initialise for now
}


Engine::~Engine()
{
}
void Engine::Write(const base * object)
{
#if debug > 1
    LOG2(ThisModule "Write object->GetPersistenceID %s\n", object->
	   GetPersistenceID());
#endif
    string a = "<";
    a += object->GetPersistenceID();
    a += "> ";

    myUnderlyingStream << a;
    object->Write(*this);
#ifndef use_string_2
    a.insert(1, "/");
#else
    a = a.before(1) + "/" + a.after(0);
#endif
    myUnderlyingStream << a << endl;

//    Write(a);

}

void Engine::Read(base * &object)
{
    string className;
  nextLine:
    myUnderlyingStream >> className;
#if debug > 0
    LOG2(ThisModule "className:%s\n", className.c_str());
#endif
    if (className.length() == 0) {
	LOG1(ThisModule "no classname found\n");
	object = NULL;
	return;
    }
#ifndef use_string_2
    if (className.find("#") == 0) {
#else
    if (className.index("#") == 0) {
#endif
#if debug > 0
	LOG2(ThisModule "comment%s", className.c_str());
#endif
#ifndef use_string_2
	getline(myUnderlyingStream, className);
#else
	getline(myUnderlyingStream, className);
#endif
//	LOG2(ThisModule "%s\n", className.c_str());
//	FLUSH;
	goto nextLine;
    }
#ifndef use_string_2
    int bi = className.find('<');
#else
    int bi = className.index('<');
#endif
    if (bi < 0) {
	LOG2(ThisModule "not a classname %s\n", className.c_str());
	object = NULL;
	return;
    }
#ifndef use_string_2
    int ei = className.find('>');
#else
    int ei = className.index('>');
#endif
    if (ei < 0) {
	LOG2(ThisModule "not a classname %s\n", className.c_str());
	object = NULL;
	return;
    }
#ifndef use_string_2
    className = className.substr(bi + 1, ei - 1);
#if debug > 1
    LOG3(ThisModule "className(string1):%s %d\n", className.c_str(),
	   className.find('/'));
#endif
    if (int (className.find('/')) >= 0) {
#else
    className = className.at(bi + 1, ei - 1);
#if debug > 1
    LOG2(ThisModule "className:%s\n", className.c_str());
#endif
    if (className.index('/') >= 0) {
#endif
	LOG2(ThisModule "outer closing classname %s\n",
	       className.c_str());
	object = NULL;
	return;
    }
#if debug >1
    LOG2(ThisModule "Read class name: %s\n", className.c_str());
    FLUSH;
#endif
/*
#ifndef use_string_2
    className = className.substr(1, className.length() - 2);
#else    
    className = className.at(1, className.length() - 2);
#endif    
*/

    // Create the object (of the relevant type)
    object = TypeManager::CreateInstanceOf(className.c_str());
    if (object) {
	// Okay then - we can make this object
	object->Read(*this);
	myUnderlyingStream >> className;
#if debug >1
	LOG2(ThisModule "Read closing class name:%s\n",
	       className.c_str());
	FLUSH;
#endif
    } else {
	LOG2(ThisModule "Unable to instantiate object of class %s\n",
	       className.c_str());
	FLUSH;
	abort();
    }
}

void Engine::Write(const string & str)
{
    myUnderlyingStream << "'" << str << "' ";
}

void Engine::Write(const double & str)
{
    char s[100];
    sprintf(s,"%0.15G", str);
#if debug >0    
    LOG3(ThisModule "%g  %0.12G\n", str,str);
    FLUSH;
#endif    
    myUnderlyingStream << s << " ";
}

void Engine::Read(string & str)
{
    string s;
    myUnderlyingStream >> str;
#if debug >0
    LOG3(ThisModule "%s %d\n", str.c_str(), str.length());
    FLUSH;
#endif
#ifndef use_string_2
    while (str.at(str.length() - 1) != '\'') {
#else
    while (str[str.length() - 1] != '\'') {
#endif
	myUnderlyingStream >> s;
#if debug >0
	LOG3(ThisModule "%s %d\n", s.c_str(), s.length());
	FLUSH;
#endif
	str = str + " " + s;
    }

#ifndef use_string_2
    str = str.substr(1, str.length() - 2);
#else
    str = str.at(1, str.length() - 2);
#endif
}

#define _RE(ar,ob)   ar.Read(ob); return ar
#define _OWE(ar,ob)   ar.Write(ob); return ar
#define _SWE(ar,ob)   ar.Write(ob); return ar
#define _WED(ar,ob)   ar.Write(ob); return ar
#define _WE(ar,ob)   ar.myUnderlyingStream<<ob<<" "; return ar
Engine & operator >> (Engine & ar, base * &ob) {
    _RE(ar, ob);
}

Engine & operator << (Engine & ar, base const *ob) {
    _OWE(ar, ob);
}

Engine & operator >> (Engine & ar, int8 & ob) {
    _RE(ar, ob);
}

Engine & operator << (Engine & ar, int8 ob) {
    _WE(ar, ob);
}

Engine & operator >> (Engine & ar, uint8 & ob) {
    _RE(ar, ob);
}

Engine & operator << (Engine & ar, uint8 ob) {
    _WE(ar, ob);
}

Engine & operator >> (Engine & ar, int16 & ob) {
    _RE(ar, ob);
}

Engine & operator << (Engine & ar, int16 ob) {
    _WE(ar, ob);
}

Engine & operator >> (Engine & ar, uint16 & ob) {
    _RE(ar, ob);
}

Engine & operator << (Engine & ar, uint16 ob) {
    _WE(ar, ob);
}

Engine & operator >> (Engine & ar, int32 & ob) {
    _RE(ar, ob);
}

Engine & operator << (Engine & ar, int32 ob) {
    _WE(ar, ob);
}

//Engine& operator <<( Engine& ar, coord ob) {
//    ar.myUnderlyingStream<<cellName(ob)<<" ";
//}
//Engine& operator <<( Engine& ar, int32 ob)  {_WE (ar,ob);}

Engine & operator >> (Engine & ar, uint32 & ob) {
    _RE(ar, ob);
}

Engine & operator << (Engine & ar, uint32 ob) {
    _WE(ar, ob);
}
#ifdef need_long_long
Engine & operator >> (Engine & ar, int64 & ob) {
    _RE(ar, ob);
}

Engine & operator << (Engine & ar, int64 ob) {
    _WE(ar, ob);
}

Engine & operator >> (Engine & ar, uint64 & ob) {
    _RE(ar, ob);
}

Engine & operator << (Engine & ar, uint64 ob) {
    _WE(ar, ob);
}
#endif
Engine & operator >> (Engine & ar, float &ob) {
    _RE(ar, ob);
}

Engine & operator << (Engine & ar, float ob) {
    _WE(ar, ob);
}

Engine & operator >> (Engine & ar, double &ob) {
    _RE(ar, ob);
}

Engine & operator << (Engine & ar, double ob) {
    _WED(ar, ob);
}

Engine & operator >> (Engine & ar, string & ob) {
    _RE(ar, ob);
}

Engine & operator << (Engine & ar, string ob) {
    _SWE(ar, ob);
}

Engine & operator >> (Engine & ar, bool & ob) {
    uint32 a;
    ar.Read(a);
    ob = a == 1;
    return ar;
}

Engine & operator << (Engine & ar, bool ob) {
    uint32 a = ob ? 1 : 0;
    ar.Write(a);
    return ar;
}

#undef _RE
#undef _WE
/*
    Changes:
    
    05/13/2002  Allow comments in object files. Is this already working ?
		Object Read & write functions return void now. 
		cout<<... replaced by printf(..
    10/13/2002 	replaced printf(... by logx(... where x is the number of parameters. Log output
		is thus printed to what is defined in log.h;		
    02/19/2003 	special write method for doubles allows to control precision.
    03/31/2003 	mode is not needed
*/
