/*
 Helper functions for messages in the spreadsheet viewer. Maybe replaced
 by something which allows to change language without recompilation.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001. 

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#include "cell.h"
#include "helpers.h"
#include "lib0.h"
#include "ptokens.h"

#include "vi_string.h"

extern "C" {
getLangString scLang = NULL;
}

string sc(int sc)
{
    if (scLang)
	return (scLang(sc));
    else
	return (digits(sc) + " Please provide a language module.");
}

string extDateTimeString(time_t t)
{
    string s = ctime(&t);
#ifndef use_string_2
    return (s.substr(0, s.length() - 1));
#else
    return (s.before((int) s.length() - 1));
#endif
}

string ioErrorText(int t)
{
    if (t<256) {
	if (t==0) return sc(scOfIoErrors);
	else return sc(scOfIoErrors+1);
    } else {
	string s="";
	int mask=256;
	int ec=2;
	for (int i=0;i<=7;i++) {
	    if (t & mask) {
		 s=s+sc(scOfIoErrors+ec)+" ";
	    }	 
	    ec++;
	    mask=mask<<1;
	} 
	return (s);
    }
}

string listTPN(pString & p, int decimals)
{
//    return ("fixme: listTPN");
    char token, tn;
    coord c;
    int j = 0, j5;
    string res = "";
    int i = p.length();
    double r;
    while (j < i) {
	token = p.at(j);
	j++;
	switch (token) {
	case ptADD:
	    res += " ADD";
	    break;
	case ptMUL:
	    res += " MUL";
	    break;
	case ptMINUS:
	    res += " -ToS";
	    break;
	case ptRECIP:
	    res += " 1/ToS";
	    break;
	case ptLESS:
	    res += " <?";
	    break;
	case ptEQUAL:
	    res += " =?";
	    
break;
	case ptGREATER:
	    res += " >?";
	    break;
#ifdef handleBorlandReal	    
	case ptDOSNumber:
	    r = p.getDosReal(j);
	    res += " PUSH " + formatDouble(r, 0, 3);
	    break;
#endif	    
	case ptDOUBLE:
	    r = p.getDouble(j);
	    res += " PUSH " + formatDouble(r, 0, 0);
	    break;
	case ptCELL:
	    c = p.getCoord(j);
	    tn = p.at(j);
	    if ((tn != ptSTATE) && (tn != ptTIME)) {
		res += " PUSH " + cellName(c);
	    }
	    break;
	case ptSTATE:
	    j5 = j - 5;		//This should be the position of the cell coord
	    c = p.getCoord(j5);
	    res += " PUSH STATE(" + cellName(c) + ")";
	    break;
	case ptSQRT:
	    res += " SQR";
	    break;
	case ptLN:
	    res += " LN";
	    break;
	case ptEXP:
	    res += " EXP";
	    break;
	case ptLOG10:
	    res += " LOG";
	    break;
	case ptSIN:
	    res += " SIN";
	    break;
	case ptCOS:
	    res += " COS";
	    break;
	case ptTAN:
	    res += " TAN";
	    break;
	case ptARCSIN:
	    res += " ASIN";
	    break;
	case ptARCCOS:
	    res += " ACOS";
	    break;
	case ptARCTAN:
	    res += " ATAN";
	    break;
	case ptABS:
	    res += " ABS";
	    break;
	case ptINT:
	    res += " INT";
	    break;
	case ptImpC:
	    j5 = p.getWord(j);
	    j++;
	    c = p.getCoord(j);
	    res += " PUSH " + digits(j5) + ":" + cellName(c);
	    break;
	case ptIndC:
	    c = p.getCoord(j);
	    res += " PUSH " + cellName(c) + "(ToS)";
	    break;
	case 'Z':
	    res += " <>?";
	    break;
	case 'Z' + 1:
	    res += " PUSH NOISE";
	    break;
	case 'Z' + 2:
	    res += " PUSH OLD";
	    break;
	case 'a':
	    res += " AND";
	    break;
	case 'b':
	    res += " OR";
	    break;
	case 'c':
	    res += " XOR";
	    break;
	
case 'd':
	    res += " PUSH DIFF";
	    break;
	case 'e':
	    res += " ERR";
	    break;
	case 'f':
	    res += " PUSH CPU-Last";
	    break;
	case 'g':
	    res += " SIGN";
	    break;
	case 'h':
	    res += " PUSH hours()";
	    break;
	case 'i':
	    res += " PUSH minutes()";
	    break;
	
case 'j':
	    res += " PUSH seconds()";
	    break;
	case 'k':
	    res += " PUSH year()";
	    break;
	case 'l':
	    res += " PUSH month()";
	    break;
	case 'm':
	    res += " PUSH day()";
	    break;
	case 'n':
	    res += " PUSH dow()";
	    break;
	case 'o':
	    res += " PUSH Bildnummer";
	    break;
	case ptSCAL:
	    res += " SCAL";
	    break;
	case ptTIME:
	    j5 = j - 5;		//This should be the position of the cell coord
	    c = p.getCoord(j5);
	    res += " PUSH TIME(" + cellName(c) + ")";
	    break;
	case 'r':
	    res += " PUSH DUR";
	    break;
	case ptFLAGS:
	    j5 = j - 5;		//This should be the position of the cell coord
	    c = p.getCoord(j5);
	    res += " PUSH FLAGS(" + cellName(c) + ")";
	    break;
	case ptMAX:
	    res += " MAX";
	    break;
	case ptMIN:
	    res += " MIN";
	    break;
	
case ptSECKT:
	    res += " SECKT";
	    break;
	case ptKTSEC:
	    res += " KTSEC";
	    break;
	
case ptBITS:
	    res += " BITS";
	    break;
	case ptMOD:
	    res += " MOD";
	    break;
	}
    }
    return res;
}

/*
    Changes:
    06/21/2002  * fixed some things for cell properties like STATE
    02/23/2003	changed include <tvision/...> to "tvision/..."
*/
