/*
 Some generally used functions.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.com) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#ifndef __lib0
#define __lib0
#include "mytypes.h"
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include "cell.h"

#include "vi_string.h"

#define debug 0
#define ThisModule "lib0     : "

#include "plclog.h"
//
// maske string uppercase:
//
string upperString(string s)
{
    for (unsigned int i = 0; i < s.length(); i++) {
	s[i] = toupper(s[i]);
    }
    return (s);
}

//
// remove any spaces:
//
string noSpace(string s)
{
    string s2 = "";
    for (unsigned int i = 0; i < s.length(); i++) {
	char c = s[i];
	if (c != ' ')
	    s2 += c;
    }
    return (s2);
}

//
// remove any dollar signs:
//
string noDollars(string s)
{
    string s2 = "";
    for (unsigned int i = 0; i < s.length(); i++) {
	char c = s[i];
	if (c != '$')
	    s2 += c;
    }
    return (s2);
}

//
// The code for column part of a cell coordinate:
//
string colCode(word r)
{
    string S = "";
    if (r == 0)
	return ("@");
    while (r > 0) {
	r--;
#ifndef use_string_2
	S.insert(S.begin(), char ('A' + (r % 26)));
#else
	S.prepend(char ('A' + (r % 26)));
#endif
	r /= 26;
    }
    return (S);
}

//
// The code for the row part of a cell coordinate:
//
string digits(word c)
{
    string S = "";
    if (c == 0)
	return ("0");
    while (c > 0) {
#ifndef use_string_2
	S.insert(S.begin(), char ('0' + (c % 10)));
#else
	S.prepend(char ('0' + (c % 10)));
#endif
	c /= 10;
    }
    return (S);
}

string ldigits(unsigned int c)
{
    string S = "";
    if (c == 0)
	return ("0");
    while (c > 0) {
#ifndef use_string_2
	S.insert(S.begin(), char ('0' + (c % 10)));
#else
	S.prepend(char ('0' + (c % 10)));
#endif
	c /= 10;
    }
    return (S);
}

//
// The code for a cell coordinate:
//
string cellName(coord k)
{
    return (colCode(k.col) + digits(k.row));
}

string cellName(cell * c)
{
    if (c)
	return cellName(c->k);
    else
	return "NULL";
}

//
// Make a number from the column part of a cell coordinate.
// s may be uppercase only:
//
int invColCode(string s)
{
    unsigned int i, j = 0;
    for (i = 0; i < s.length(); i++) {
	j = j * 26;
	if (s[i] >= 'A') {
	    j = j + (s[i] - 64);
	}
    }
    return (j);
}

//
// Make a number from a cell coordinate. It may contain dollar signs
// marking cells or rows which shall not be shifted. May contain spaces
// if user inputs any. May be mixed case.
//
coord getPos(string s)
{
    int i = 0, a;
    coord p;
    string s2;
    char c;
#if debug > 1
    LOG2(ThisModule "str: %s\n", s.c_str());
    FLUSH;
#endif    
    s = noSpace(noDollars(s));
#if debug > 1
    LOG2(ThisModule "str: %s\n", s.c_str());
    FLUSH;
#endif    
    s = upperString(s);
#if debug > 1
    LOG2(ThisModule "str: %s\n", s.c_str());
    FLUSH;
#endif
    c = s[i];
    while (c > '9') {
	c = s[i];
	i++;
    }
    i--;
    s2 = s.substr(i, s.length() - i);
    a = atol(s2.c_str());
    p.row = (word) a;
    s2 = s.substr(0, i);
    a = invColCode(s2);
    p.col = (word) a;
    return (p);
}

string formatDouble(double d, int len, int decimals)
{
    char s[264], f[20];
    sprintf(f, "%%%d.%df", len + decimals, decimals);
#if debug > 1
    LOG2(ThisModule "The format str is:%s\n", f);
    FLUSH;
#endif
    sprintf(s, f, d);
#if debug > 1
    LOG2(ThisModule "The result str is:%s\n", s);
    FLUSH;
#endif
    return (s);
}

int translateByTable(const char * s, siEntry * t, ui last) {
    for (ui i=0; i<last; i++) {
//	LOG3("name :%s value%d\n", t[i].s, t[i].value);
	if (strcmp(s, t[i].s)==0) {
	    return t[i].value;
	}    
    };
    return -1;
}    


#endif				/* _lib00_ */
/*
    Changes:
    05/25/2002    fixed some warnings about signed<->unsigned comparision
    05/25/2002    added ldigits()
        
*/
