/*
 list.cc: implements a basic list structure.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include "list.h"
#include "lib0.h"
#include <limits.h>
#include <stdio.h>
#include <string.h>
#include "plclog.h"

#define debug 0
#define maxCollectionSize INT_MAX / sizeof(void *)

//int list::indexOf(void *item)
int list::indexOf(base *item)
{
    for (int i = 0; i < count; i++)
	if (item == items[i])
	    return i;

//    error(1,0);
    return (0);
}


void list::atRemove(int index)
{
    if (index < 0 || index >= count)
	error(1, 0);

    count--;
    memmove(&items[index], &items[index + 1],
//	    (count - index) * sizeof(void *));
	    (count - index) * sizeof(base *));
}

//void list::free(void *item)
void list::free(base *item)
{
    atRemove(indexOf(item));
    LOG1("before delete item\n");
    delete item;
    LOG1("delete item done.\n");
}

int list::compare(void *key1, void *key2)
{
#if debug >3
    LOG1("list::compare\n");
#endif
    return (0);
}

//void list::atInsert(int index, void *item)
void list::atInsert(int index, base *item)
{
#if debug >3
    LOG1("list::atInsert\n");
#endif
    if (index < 0)
	error(1, 0);
    if (count == limit)
	setLimit(count + delta);

    memmove(&items[index + 1], &items[index],
	    (count - index) * sizeof(void *));
    count++;
    items[index] = item;
}

//int list::insert(void *item)
int list::insert(base *item)
{
#if debug >3
    LOG1("list::insert\n");
#endif
    int i;
    if (search(item, i) == 0)
	atInsert(i, item);
#if debug >3
    LOG1("list::insert done.\n");
    FLUSH;
#endif
    return i;
}

bool list::search(void *key, int &index)
{
    index = count;
    return (0);
}

void list::setLimit(int aLimit)
{
    if (aLimit < count)
	aLimit = count;
    if (aLimit > maxCollectionSize)
	aLimit = maxCollectionSize;
    if (aLimit != limit) {
//	void **aItems;
	base **aItems;
	if (aLimit == 0)
	    aItems = 0;
	else {
//	    aItems = new void *[aLimit];
	    aItems = new base *[aLimit];
	    if (count != 0 && aItems != 0 && items != 0)
		memcpy(aItems, items, count * sizeof(void *));
	}
	delete items;
	items = aItems;
	limit = aLimit;
    }
}

list::list():
count(0), limit(0), delta(10), items(0)
{
#if debug >3
    LOG1("list::list()\n");
#endif
    setLimit(10);
}

void list::forEach(todo t, void *arg) const
{
    for (int i = 0; i < count; i++) {
	if (items[i] != NULL)
	    t(items[i], arg);
    }
#if debug >1
    LOG1("list::forEach ends\n");
    FLUSH;
#endif
}

list::~list()
{
#if debug >3
    LOG1("list::~list()\n");
#endif
    for (int i = 0; i < count; i++) {
	delete (items[i]);
    }
}

/*
    Changes:
    
    05/13/2002  Sheet is now loaded using persistence mechanism.
		Object Read & write functions return void now. 
*/
