/*
 Load and evaluate the configuration file.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <iostream.h>
#include <fstream.h>

#ifdef withTVISION
#include "lib0.h"
#endif

#include "cellList.h"
#include <dlfcn.h>
#include "globals.h"
//#include "plcdrv.h"
#include "plcdrivers/PLCdriver.h"
#include <pthread.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include "helpers.h"

#ifdef withTVISION
#include "lang.h"
#else
#include "lang.h"
#endif

#include <stdio.h>
#include <signal.h>
#include "plclog.h"

#define debug 10
#define debugSignal 5

//#define ThisModule "loader   : "

#include "vi_string.h"

void loadfrom(int sh, string ext) {
    LOG4("trying to open: %s%d.%s\n",projectPath.c_str(), sh, ext.c_str());
    fstream f2((projectPath + digits(sh) + "."+ext).c_str(), ios::in);
    bool b = f2.ios::good();
    if (b) {
    LOG4("loading last settings from %s%d.%s\n",projectPath.c_str(), sh, ext.c_str());
	string setline, strk, strv;
	while (getline(f2, setline)) {
	    int p1 = (setline.Find("'"));
	    setline = setline.after(p1);
	    p1 = (setline.Find("'"));
	    strk = setline.before(p1);
	    setline = setline.after(p1 + 1);
	    p1 = (setline.Find(" "));
	    strv = setline.before(p1);
//	    LOG3( "set %s=%s\n", strk.c_str(), strv.c_str());
	    setCell(sh, getPos(strk), atof(strv.c_str()), 3);
        }
//#endif                        
    } else {
	LOG4( "no such file: %s%d.%s\n", projectPath.c_str(), sh, ext.c_str());
    }
    f2.close();
}    

PLCdriver * findDriver(int pid) {
    for (int i = 0; i < maxDrivers-2; i++) {
        if (registrations[i].mask != 0) {
	    if (registrations[i].pointerToInstance) {
	        PLCdriver *p =
		    ((PLCdriver *) registrations[i].pointerToInstance);
		if (p->pid==pid) {
		    return p;
	        }	
	    }
        }
    }    
    return NULL;
}	

/*
    Signal handler for SIGPIPE
*/
void sigpipeHandler(int a,siginfo_t * info,void*) {
//    int rfd=2;
    LOG2("sigpipeHandler called with: %d\n", a);
    if (info) {
	LOG2("Signal number: %d\n", info->si_signo);
        LOG2("An errno value: %d\n", info->si_errno);
	LOG2("Signal code: %d\n", info->si_code);
        LOG2("Sending process ID: %d\n", info->si_pid);
	LOG2("My process ID: %d\n", getpid());
	LOG2("Real user ID of sending process: %d\n", info->si_uid);
        LOG2("File descriptor: %d\n", info->si_fd);
    /*
                  uid_t    si_uid;    / Real user ID of sending process /
                  int      si_status; / Exit value or signal /
                  clock_t  si_utime;  / User time consumed /
                  clock_t  si_stime;  / System time consumed /
                  sigval_t si_value;  / Signal value /
                  int      si_int;    / POSIX.1b signal /
                  void *   si_ptr;    / POSIX.1b signal /
                  void *   si_addr;   / Memory location which caused fault /
                  int      si_band;   / Band event /
                  int      si_fd;     / File descriptor /
*/
    } else {		  
	LOG1("no info available\n");
    }	
/*    
    PLCdriver * drv=findDriver(info->si_pid);
    LOG2("found driver Module: %p\n", drv);
    if(drv) {
	if(drv->DI){
	    LOG1("Ok. Driver keeps di. I give the reconnect command.\n");
	    drv->DI->commands=2;
	    //driverCmdReload;
	} else {
	    LOG1("FIXIT: old driver doesnt keep it's DI.\n");
	}
    }
//    pthread_exit(NULL);
*/    
    FLUSH;
}

    

/*
    Signal handler for SIGHUP
*/
void sighupHandler(int a,siginfo_t * info,void*) {
//    int rfd=2;
    LOG2("sighupHandler called with: %d\n", a);
    if (info) {
	LOG2("Signal number: %d\n", info->si_signo);
        LOG2("An errno value: %d\n", info->si_errno);
	LOG2("Signal code: %d\n", info->si_code);
        LOG2("Sending process ID: %d\n", info->si_pid);
	LOG2("My process ID: %d\n", getpid());
	LOG2("Real user ID of sending process: %d\n", info->si_uid);
        LOG2("File descriptor: %d\n", info->si_fd);
    /*
                  uid_t    si_uid;    / Real user ID of sending process /
                  int      si_status; / Exit value or signal /
                  clock_t  si_utime;  / User time consumed /
                  clock_t  si_stime;  / System time consumed /
                  sigval_t si_value;  / Signal value /
                  int      si_int;    / POSIX.1b signal /
                  void *   si_ptr;    / POSIX.1b signal /
                  void *   si_addr;   / Memory location which caused fault /
                  int      si_band;   / Band event /
                  int      si_fd;     / File descriptor /
*/
    } else {		  
	LOG1("no info available\n");
    }	
    PLCdriver * drv=findDriver(getpid());
    LOG2("found driver Module: %p\n", drv);
    if(drv) {
	if(drv->DI){
	    LOG1("Ok. Driver keeps di. I give the reload command.\n");
	    drv->DI->commands=1;
	    //driverCmdReload;
	} else {
	    LOG1("FIXIT: old driver doesnt keep it's DI.\n");
	}
    }
//    pthread_exit(NULL);
    FLUSH;
}

/*
    Signal handler for SIGTERM
*/
void sigtermHandler(int a,siginfo_t * info,void*) {
//    int rfd=2;
    LOG2("sigtermHandler called with: %d\n", a);
    if (info) {
	LOG2("Signal number: %d\n", info->si_signo);
        LOG2("An errno value: %d\n", info->si_errno);
	LOG2("Signal code: %d\n", info->si_code);
        LOG2("Sending process ID: %d\n", info->si_pid);
	LOG2("My process ID: %d\n", getpid());
	LOG2("Real user ID of sending process: %d\n", info->si_uid);
        LOG2("File descriptor: %d\n", info->si_fd);
    } else {		  
	LOG1("no info available\n");
    }	
    
    for (int i = 0; i < maxDrivers-2; i++) {
        if (registrations[i].mask != 0) {
	    if (registrations[i].pointerToInstance) {
	        PLCdriver *p =
		    ((PLCdriver *) registrations[i].pointerToInstance);
		p->doRun=false;
	    }
        }
    }
/*    
    PLCdriver * drv=findDriver(getpid());
    LOG2("found driver Module: %p\n", drv);
    if(drv) {
	if(drv->DI){
	    LOG1("Ok. Driver keeps di. I give the reload command.\n");
	    drv->DI->commands=1;
	} else {
	    LOG1("FIXIT: old driver doesnt keep it's DI.\n");
	}
    }
    FLUSH;
*/    
}

typedef void* (*driverfunc) (void *);

void load2(const char * conffile)
{
    string s, tfiName;
    string driverPath, plcDriverPath, driverLibName, driverInfo;
    int i = 1, p;
    sheets[1] = NULL;
    struct sigaction newpipeact;
    fstream f(conffile, ios::in);
    time_t t;
    
    newpipeact.sa_sigaction=sigpipeHandler;
    newpipeact.sa_flags=SA_SIGINFO;
    int sa=sigaction(SIGPIPE,&newpipeact,NULL);
    LOG2("SIGPIPE handler installed. Result: %d\n", sa);
    
    newpipeact.sa_sigaction=sighupHandler;
    newpipeact.sa_flags=SA_SIGINFO;
    sa=sigaction(SIGHUP,&newpipeact,NULL);
    LOG2("SIGHUP handler installed. Result: %d\n", sa);
    
    newpipeact.sa_sigaction=sigtermHandler;
    newpipeact.sa_flags=SA_SIGINFO;
    sa=sigaction(SIGTERM,&newpipeact,NULL);
    LOG2("SIGTERM handler installed. Result: %d\n", sa);
#if debug > 3
    LOG1( "after open\n");
#endif
  nextline:
    while (getline(f, s)) {
#if debug > 3
	LOG3( "line %d : %s\n", i, s.c_str());
#endif
	i++;
	if (s.Find("#") == 0)
	    goto nextline;
	if (s.Find("project:") == 0) {
	    projectPath = noSpace(s.from(8));
#if debug > 0
	    LOG2( "found project directory: %s\n",
		   projectPath.c_str());
	    LOG1( "trying to load the spread sheets.\n");
#endif
#if debug > 0
	    time(&t);
	    LOG2( "%s", ctime(&t));
#endif
	    for (int sh = 1; sh <= maxSheet; sh++) {
		sheets[sh] = NULL;
#if debug > 3
		LOG3( "trying to load: %s%d.sht\n",
		       projectPath.c_str(), sh);
#endif
		fstream f((projectPath + digits(sh) + ".sht").c_str(),
			  ios::in);
		bool b = f.ios::good();
		if (b) {
#if debug > 1
		    LOG3( "loading %s%d.sht.\n", projectPath.c_str(), sh);
#endif
		    Engine *e = new Engine(f);
		    *e >> sheets[sh];
		    sheets[sh]->shNumber=sh;
		    delete e;
		}
		f.close();
		if (!b)
		    goto nextline;
		loadfrom(sh, "pre");
		loadfrom(sh, "per");
		loadfrom(sh, "set");
		time(&t);
#if debug > 1
		LOG2( "%s", ctime(&t));
#endif
//              if (!b)
//                  goto nextline;
	    }			// for
	    

	    goto nextline;
	}
	if (s.Find("driverpath:") == 0) {
	    driverPath = noSpace(s.from(11));
#if debug > 0
	    LOG2( "found driver directory: %s\n",
		   driverPath.c_str());
#endif
	    goto nextline;
	}
	if (s.Find("plcdriverpath:") == 0) {
	    plcDriverPath = noSpace(s.from(14));
#if debug > 0
	    LOG2( "found new style PLC driver directory: %s\n",
		   plcDriverPath.c_str());
#endif
	    goto nextline;
	}
	
	if (s.Find("plcdriver:") == 0) {
	    driverLibName = s.from(10);
	    do {
		p = driverLibName.Find(" ");
		if (p == 0)
		    driverLibName.erase(0, 1);
	    }
	    while (p == 0);
	    driverInfo = driverLibName.after(p);
	    driverLibName = driverLibName.before(p);
#if debug > 0
	    LOG3( "found plcdriver module: %s driver info: %s\n", driverLibName.c_str(),
		   driverInfo.c_str());
#endif
	    p = driverInfo.Find(".tfi");
	    if (p >= 0) {
		tfiName = driverInfo.before(p + 4);
		driverInfo.erase(0, p + 4);
#if debug > 0
		LOG3( "Driver Instruction List in: %s%s\n",
		       projectPath.c_str(), tfiName.c_str());
#endif
	    }
#if debug > 0
	    LOG2( "Preloading PLCdriver.so: I will dlopen: %sPLCdriver.so\n",
		   plcDriverPath.c_str());
#endif
	    void * prehandle =
		dlopen((plcDriverPath +"PLCdriver.so").c_str(), RTLD_NOW | RTLD_GLOBAL);
	    if (!prehandle)
		LOG3( "loading: %sPLCdriver dlopen():%s\n",
		       (plcDriverPath +"PLCdriver.so").c_str(),
		       dlerror());
//	    void *(*driver1) (void *);	       	    
//	    (void *) driver1 = dlsym(prehandle, "DriverThread");	       
#if debug > 0
	    LOG3( "I will dlopen plcdriver: %s%s\n",
		   plcDriverPath.c_str(), driverLibName.c_str());
#endif
	    void *handle =
		dlopen((plcDriverPath + driverLibName).c_str(), RTLD_LAZY);
	    if (!handle)
		LOG4( "loading: %s%s dlopen():%s\n",
		       plcDriverPath.c_str(),
		       driverLibName.c_str(), dlerror());
//	    void *(*driver) (void *);
//	    void * driver;
	    driverfunc driver;
	    pthread_t taskHandle;
//	    (void *) driver = dlsym(handle, "DriverThread__FPv");
//	    (void *) driver = dlsym(handle, "_Z12DriverThreadPv");
//	    (void *) driver = dlsym(handle, "DriverThread");
	    driver = (driverfunc)dlsym(handle, "DriverThread");
#if debug > 0
	    LOG2( "dlsym: %s\n", dlerror());
#endif
	    DriverInfo *DI = new DriverInfo;
	    DI->s1 = projectPath+tfiName;
	    DI->s = driverInfo;
	    DI->commands = 0;
#if debug > 0
	    int res =
#endif
		pthread_create(&taskHandle, NULL, driver, DI);
#if debug > 0
	    LOG2( "created plcdriver thread: %d\n", res);
	    FLUSH;
#endif
	    time(&t);
#if debug > 2
	    LOG2( "%s", ctime(&t));
#endif
	    goto nextline;
	}
	
	if (s.Find("driver:") == 0) {
	    driverLibName = s.from(7);
	    do {
		p = driverLibName.Find(" ");
		if (p == 0)
		    driverLibName.erase(0, 1);
	    }
	    while (p == 0);
	    driverInfo = driverLibName.after(p);
	    driverLibName = driverLibName.before(p);
#if debug > 0
	    LOG3( "found driver module: %s driver info: %s\n", driverLibName.c_str(),
		   driverInfo.c_str());
#endif
	    p = driverInfo.Find(".tfi");
	    if (p >= 0) {
		tfiName = driverInfo.before(p + 4);
		driverInfo.erase(0, p + 4);
#if debug > 0
		LOG3( "Driver Instruction List in: %s%s\n",
		       projectPath.c_str(), tfiName.c_str());
#endif
	    }
#if debug > 0
	    LOG3( "I will dlopen plcdriver: %s%s\n",
		   driverPath.c_str(), driverLibName.c_str());
#endif
	    void *handle =
		dlopen((driverPath + driverLibName).c_str(), RTLD_LAZY);
	    if (!handle)
		LOG4( "loading: %s%s dlopen():%s\n",
		       driverPath.c_str(),
		       driverLibName.c_str(), dlerror());
	    void *(*driver) (void *);
	    pthread_t taskHandle;
//	    (void *) driver = dlsym(handle, "DriverThread__FPv");
//	    (void *) driver = dlsym(handle, "_Z12DriverThreadPv");
//	    (void *) driver = dlsym(handle, "DriverThread");
	    driver = (driverfunc) dlsym(handle, "DriverThread");
#if debug > 0
	    LOG2( "dlsym: %s\n", dlerror());
#endif
	    DriverInfo *DI = new DriverInfo;
	    DI->s1 = projectPath+tfiName;
	    DI->s = driverInfo;
	    DI->commands = 0;
#if debug > 0
	    int res =
#endif
		pthread_create(&taskHandle, NULL, driver, DI);
#if debug > 0
	    LOG2( "created plcdriver thread: %d\n", res);
	    FLUSH;
#endif
	    time(&t);
#if debug > 2
	    LOG2( "%s", ctime(&t));
#endif
	    goto nextline;
	}
	if (s.Find("logger:") == 0) {
	    driverLibName = s.from(7);
	    do {
		p = driverLibName.Find(" ");
		if (p == 0)
		    driverLibName.erase(0, 1);
	    }
	    while (p == 0);
	    driverInfo = driverLibName.after(p);
	    driverLibName = driverLibName.before(p);
#if debug > 0
	    LOG2( "found Logger module: %s\n",
		   driverLibName.c_str());
	    LOG2( "found Logger info: %s\n",
		   driverInfo.c_str());
#endif
	    p = driverInfo.Find(".dri");
	    if (p >= 0) {
		tfiName = driverInfo.substr(0, p + 4);
		driverInfo.erase(0, p + 4);
#if debug > 0
		LOG3( "module Instruction List in: %s%s\n",
		       projectPath.c_str(), tfiName.c_str());
#endif
	    } else
		tfiName = driverInfo;	// Is this correct?
#if debug > 0
	    LOG3( "I will dlopen module: %s%s\n",
		   driverPath.c_str(), driverLibName.c_str());
#endif
	    void *handle =
		dlopen((driverPath + driverLibName).c_str(), RTLD_LAZY);
	    if (!handle)
		LOG2( "dlopen: %s\n", dlerror());
//	    void *(*moduleInitProc) (void *) = NULL;
	    driverfunc moduleInitProc = NULL;
//	    (void *) moduleInitProc = dlsym(handle, "setup__FPv");	//mangled gcc 2.95
//	    (void *) moduleInitProc = dlsym(handle, "_Z5setupPv");	//mangled gcc 3.0.4
//	    (void *) moduleInitProc = dlsym(handle, "setup");		//unmangled
	    moduleInitProc =(driverfunc) dlsym(handle, "setup");		//unmangled
	    if (!moduleInitProc)
		LOG2( "dlsym: %s\n", dlerror());
	    DriverInfo *di = new DriverInfo;
	    di->s1 = projectPath + tfiName;
	    di->s = driverInfo;
	    moduleInitProc(di);
	    time(&t);
#if debug > 2
	    LOG2( "%s", ctime(&t));
#endif
	    goto nextline;
	}

	if (s.Find("language:") == 0) {
	    driverLibName = s.from(9);
	    do {
		p = driverLibName.Find(" ");
		if (p == 0)
		    driverLibName.erase(0, 1);
	    }
	    while (p == 0);
#if debug > 0
	    LOG2( "found language module: %s\n",
		   driverLibName.c_str());
	    LOG3( "I will dlopen module: %s%s\n",
		   driverPath.c_str(), driverLibName.c_str());
#endif
	    void *handle =
		dlopen((driverPath + driverLibName).c_str(), RTLD_LAZY);
	    if (!handle)
		LOG2("dlopen: %s\n", dlerror());
//          void *(*moduleInitProc) (void *);
	    langInitProc langInit;
//	    (void *) (langInit) = dlsym(handle, "initLanguage__Fv"); 	//mangled gcc 2.95
//	    (void *) (langInit) = dlsym(handle, "_Z12initLanguagev");	//mangled gcc 3.0.4
//	    (void *) (langInit) = dlsym(handle, "initLanguage");	//unmangled		
	    langInit = (langInitProc) dlsym(handle, "initLanguage");	//unmangled		
	    if (!langInit)
		LOG2("dlsym: %s\n", dlerror());
	    else
		langInit();
	    goto nextline;
	}
	LOG2("*** Syntax error in line %d\n", i - 1);
    }
    LOG1("Configuration file ended.\n");
    f.close();
//    usleep(6000000);
    if (projectPath.empty()) {
	LOG1("***No project directory given. Change visual.cfg!\n");
	abort();
    }
}

list * loadlist(const char *name) {
    fstream f(name, ios::in);
    Engine *e = new Engine(f);
    list *il = new list();
    base *c;
    do {
	*e >> c;
	if (c != 0)
	    il->insert(c);
    } while (c != NULL);
    delete e;
    f.close();
    return il;
}

/*
    Changes:
    
    05/13/2002  cout<<... replaced by printf(..
    05/14/2002  load last settings into sheets.
    06/01/2002  fixed a bug which caused the loader not to load further
		sheets, when no settings file present.
    10/13/2002  replaced printf(... by LOGx(... where x is the number of parameters. LOG output
		is thus printed to what is defined in log.h;		
    12/13/2002  got rid of mangled names in calls to dlsym()
    02/27/2003  There is a problem when you want that a cell has a certain value after loading:
		You can insert this value manually. But next time you save the whole sheet,
		the actual value is stored. Hence you cannot be sure that cells which are set
		from a PLC driver have a certain value before the driver is available and running.
		To solve this problem, these following files are used to load presets after 
		loading a sheet:
		1. <sh>.pre.  This file is intended to preload external input cells with some 
		    value. There is no way to save such a file. Edit a copy with a text editor.
		2. <sh>.per.  This file is intended to reload values of self-referring formulas.
		    For example, you may could implement a run time counter using a formula
		    like =A1+1. This will count up in each recalculation. But when you save
		    the sheet, the actual score is saved and will reloaded on each program start.
		    The idea of .per-files is to save the last value of such formulas, reload them
		    on restart and continue from this score.
		    This is yet unimplemented. The problem is when to save them:
			after each calculation would cause a lot of unnecessary i/o.
			before program termination needs a very save mechanism to guarentee that
			the operation is performed in case of abnormal termination.
		3. <sh>.per.  This file is stores the last settings from input elements.
    07/21/2003  Added SIGPIPE handler. I found no possibility to make signal handlers for
		per module thread, as libpthread clone(2)s threads with signals shared.
    07/22/2003  Have a single loader (this) for VISUAl and VISUALD. loader2 an loader3 just 
		#define or #undef withTVISION and #include it
    09/20/2003  moved plcdrv.h to drivers directory.
    09/28/2003  load only new drivers.
    09/29/2003  Load old and new drivers. Preload PLCdriver.so if any new plcdriver is loaded.
		dlopen() will know when it has already been loaded.
    11/10/2004  Added loadlist() to be used by PLCdrivers and modules, so it is in a central
		place and their code size is reduced.
*/
