/*
 Implementation of a class to hold the TPN character sequences.
 Behaves much like class string, but is not afraid of storing 0 bytes
 in the middle of sequence, which is needed to store the byte patterns
 of numbers and coords.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include "pString.h"
#include "iostream"
#include "plclog.h"
#define debug 0
//#define handleBorlandReal
int numnum=0;
int numtot=0;
pString::pString() {
	len = 0;
	buf = NULL;
	cap = 0;
	number=++numnum;
	numtot++;
//	LOG3("pString() number:%d total:%d\n",number,numtot);
    }

pString::~pString() {
    numtot--;
//    LOG5("~pString() number:%d total:%d buf:%p len:%d\n",number,numtot,buf,len);
//    FLUSH;
    if(len)free(buf);
} 
    

pString & pString::operator + (char s) {
    append(s, 1);
    return (*this);
}

pString & pString::operator += (char s) {
    append(s, 1);
    return (*this);
}

pString & pString::operator + (coord s) {
    append(s);
    return (*this);
}

pString & pString::operator += (coord s) {
    append(s);
    return (*this);
}

pString & pString::operator + (double d) {
    append(d);
    return (*this);
}

pString & pString::operator += (double d) {
    append(d);
    return (*this);
}

pString & pString::operator + (word s) {
    append(s);
    return (*this);
}

pString & pString::operator += (word s) {
    append(s);
    return (*this);
}



void
 pString::append(char *c, int l)
{
#if debug > 0
    cout << "pString.cc: this is append. len:" << len << " l:" << l <<
	endl;
    cout.flush();
#endif
    if (len + l > cap) {
	cap = ((len + l) / 16 + 1) * 16;
	void *b = malloc(cap);
	memcpy(b, buf, len);
	free(buf);
	buf = (char *) b;
    }
    memcpy(buf + len, c, l);
    len += l;
}

#ifdef handleBorlandReal
double _fromDOSreal(unsigned char *tdr)
{
    unsigned char ex, vz;	// var ex,vz:byte;
    float res;
    char *S = (char *) &res;	// S:SL;
    ex = tdr[0];		// Ex:=FL(R).B1;
    if (ex > 2)
	ex -= 2;
    else
	ex = 0;			// if ex>2 then dec(ex,2) else ex:=0;
    vz = tdr[5] && 0x80;	// vz:=FL(R).B6 and $80;
    S[3] = (ex >> 1) | vz;	// S.B4:=ex shr 1 or VZ;
    S[0] = tdr[3];		// S.B1:=FL(R).B4;
    S[1] = tdr[4];		//       S.B2:=FL(R).B5;
    S[2] = tdr[5] & 0x7f | ((ex & 1) << 7);	//       S.B3:=FL(R).B6 and $7f or (ex and 1) shl 7;
//       FromDOSReal:=S.A;
//       FromDOSReal:=S.A;
    return (res);
}

double pString::getDosReal(int &l)
{
    double d = _fromDOSreal(((unsigned char *) &buf[l]));
    l += 6;
    return (d);
}
#endif

double pString::getDouble(int &l)
{
    double (d);
    d = *((double *) &buf[l]);
    l += sizeof(double);
    return (d);
}

coord pString::getCoord(int &l)
{
    coord c;
    memcpy(&c, &buf[l], 4);
    l += 4;
    return (c);
}

word pString::getWord(int &l)
{
    word c;
    memcpy(&c, &buf[l], sizeof(word));
    l += sizeof(word);
    return (c);
}



