/*
 Definition of a class to hold the TPN character sequences.
 Behaves much like class string, but is not afraid of storing 0 bytes
 in the middle of sequence, which is neede to store the byte patterns
 of numbers and coords.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#ifndef __pString
#define __pString
#include "mytypes.h"
#include <stdlib.h>
#include "plclog.h"

class pString {
  public:
    pString();
    pString& asd();
    ~pString();
    void append(char *c, int l);
    void append(pString & s) {
	append(s.buf, s.len);
    }
    void append(double d) {
	append((char *) &d, sizeof(double));
    }
    void append(coord c) {
	append((char *) &c, sizeof(coord));
    }
    void append(word w) {
	append((char *) &w, sizeof(word));
    }
    void append(char c, int l) {
	append(&c, 1);
    }
//    pString operator +(double d); //{p.append(d);return(p);}
    inline char at(int i) {
	return (buf[i]);
    }
    inline int length() {
	return (len);
    }
    inline int capacity() {
	return (cap);
    }
    double getDosReal(int &l);
    double getDouble(int &l);
    coord getCoord(int &l);
    word getWord(int &l);
    pString & operator + (char s);
    pString & operator += (char s);
    pString & operator + (coord s);
    pString & operator += (coord s);
    pString & operator + (double d);
    pString & operator += (double d);
    pString & operator + (word w);
    pString & operator += (word w);
  private:
    int len;
    int cap;
    char *buf;
    int number;
};

#endif				/* _pString_ */
