/*
 Implementation of a recursive parser which converts infix to 
 tokenized postfix notation. 
 Maybe replaced by a YACC/BISON parser later.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <stdlib.h>
#include <ctype.h>
#include "pString.h"
#include "parser.h"
#include "lib0.h"
#include "ptokens.h"
#include <stdio.h>
#include "plclog.h"

#include "vi_string.h"

#define PARENTHESIS
#define ThisModule "parser   : "
#define debug 0

string prePars(string s)
{
    string s2 = "";
    s = noDollars(noSpace(s));
    for (unsigned int i = 0; i < s.length(); i++) {
	char c = s[i];
	if (c == '-')
	    s2 += "+m";
	else if (c == '/')
	    s2 += "*d";
	else
	    s2 += toupper(c);
    }
    return (s2);
}

extern void rparse(string s, string & all, pString & p, bool indexed);

int sop(string s, string op, char c, string & all, pString & p)
{
    bool indexed = false;
    int pos = s.Find(op);
    if (pos >= 0) {
#if debug >5
	LOG2("sop: %d\n", pos);
#endif

	rparse(s.from(int (pos + op.length())), all, p, indexed);
	rparse(s.before(pos), all, p, indexed);
	all = all + op + " ";
	p += c;
	return (1);
    }
    return (0);
}

int sof(string s, string op, char c, string & all, pString & p)
{
    bool indexed = false;
    int pos = s.Find(op);
    if (pos >= 0) {
#if debug >5
	LOG2("sof: %d\n", pos);
#endif
	rparse(s.from( (int) (pos + op.length())), all, p, indexed);
	all = all + " " + op;
	p += c;
	return (1);
    }

    return (0);
}


void rparse(string s, string & all, pString & p, bool indexed)
{
#if debug >0
    LOG2("rparse line: %s\n", s.c_str());
    FLUSH;
#endif
    if (s.length() == 0)
	return;
    int pos;
    int brLev = 0;
    int i;
    char c;
#ifdef PARENTHESIS
//
// A(n ugly?) trick. Here we hide operators in brackets from RPARSE.
//    
    for (i = 0; (unsigned int) i < s.length(); i++) {
	char c = s[i];
	switch (c) {
	case '+':
	    if (brLev > 0)
		s[i] = 'a';
	    break;		// handle later
	case '*':
	    if (brLev > 0)
		s[i] = 'b';
	    break;
	case '>':
	    if (brLev > 0)
		s[i] = 'e';
	    break;
	case '<':
	    if (brLev > 0)
		s[i] = 'f';
	    break;
	case '=':
	    if (brLev > 0)
		s[i] = 'g';
	    break;
	case ',':
	    if (brLev > 0)
		s[i] = 'h';
	    break;
	case 'a':
	    if (brLev == 0)
		s[i] = '+';
	    break;
	case 'b':
	    if (brLev == 0)
		s[i] = '*';
	    break;
	case 'e':
	    if (brLev == 0)
		s[i] = '>';
	    break;
	case 'f':
	    if (brLev == 0)
		s[i] = '<';
	    break;
	case 'g':
	    if (brLev == 0)
		s[i] = '=';
	    break;
	case 'h':
	    if (brLev == 0)
		s[i] = ',';
	    break;
	case ')':
	    brLev--;
	    break;
	case '(':
	    brLev++;
	    break;
	}
    }
    
#if debug >0
	LOG2("after replacing operators in brackets: %s\n",
	       s.c_str());
    FLUSH;
#endif				/*  */
    pos = s.Find('(');
#if debug >0
    LOG2("position of first '(': %d\n", pos);
    FLUSH;
#endif
    if (pos >= 0) {
	brLev = 1;		// brLev is 1, 1st open '('
	i = pos;		// search for ')' from here
	do {
	    i++;		// next char
	    c = s[i];
	    switch (c) {
	    case ')':
		brLev--;
		break;
	    case '(':
		brLev++;
		break;
	    }
	}
	while ((brLev != 0) && ((unsigned int) i < s.length()));	// closing ')' matches 1st open
	
// break into substrings and evaluate separately, beginnig with the inner
// expression:
#if debug >0
        LOG1("calling rparse inner.\n");
	FLUSH;
#endif
//#ifndef use_string_2
	rparse(s.substr(pos + 1, i - pos - 1), all, p, indexed);
//#else
//	rparse(s.at(pos + 1, i - pos - 1), all, p, indexed);
//#endif
	
// evaluate left subexpression:
	    if (pos > 0) {
#if debug >0
	    LOG2("calling rparse left of %d\n", pos);
	    FLUSH;
#endif
	    char c = s.c_str()[pos - 1];
	    bool isIndexed = (('0' <= c) && (c <= '9'));
	    if (isIndexed) {
		LOG2(ThisModule"indexed: %s\n", s.substr(0,pos).c_str());
	    }
	    rparse(s.substr(0,pos), all, p, isIndexed);
	}
#if debug >0
	else {
	    LOG2("nothing left of %d\n", pos);
	    FLUSH;
	}
#endif
#if debug >0
// evaluate right subexpression:

	LOG3("calling rparse right from %d %s\n", i,
	       s.c_str());
	FLUSH;
#endif
	rparse(s.after(i), all, p, indexed);
	return;		// all substrings done, leave rparse
    }
    
#endif
#if debug >0
	LOG2("rparse after processing brackets: %s\n",
	       s.c_str());
    FLUSH;
#endif
    if (sop(s, "<>", ptNOTEQUAL, all, p))
	return;
    if (sop(s, "=", ptEQUAL, all, p))
	return;
    if (sop(s, "<", ptLESS, all, p))
	return;
    if (sop(s, ">", ptGREATER, all, p))
	return;

    // colon separated arguments:
    pos = s.Find(',');
#if debug >0
    LOG2("colon at: %d\n", pos);
    FLUSH;
#endif
    if (pos >= 0) {
	rparse(s.before(pos), all, p, indexed);
	rparse(s.after(pos), all, p, indexed);
	return;
    }
    if (sop(s, "+", ptADD, all, p))
	return;
    if (sop(s, "*", ptMUL, all, p))
	return;

    if (sop(s, "MOD", ptMOD, all, p))
	return;
    if (sop(s, "AND", ptAND, all, p))
	return;
    if (sop(s, "XOR", ptXOR, all, p))
	return;
/*	
 * search for 'ERROR' before searching for 'OR', because 'ERROR' contains 'OR'
 * somebody might change all this to a bison parser.
 */
    if (sof(s, "ERROR", ptERR, all, p))
	return;
    if (sop(s, "OR", ptOR, all, p))
	return;

    if (sof(s, "SQR", ptSQRT, all, p))
	return;
    if (sof(s, "SQRT", ptSQRT, all, p))
	return;
    if (sof(s, "LN", ptLN, all, p))
	return;
    if (sof(s, "EXP", ptEXP, all, p))
	return;
    if (sof(s, "LOG", ptLOG10, all, p))
	return;
    if (sof(s, "SIN", ptSIN, all, p))
	return;
    if (sof(s, "COS", ptCOS, all, p))
	return;
    if (sof(s, "TAN", ptTAN, all, p))
	return;
    if (sof(s, "ASIN", ptARCSIN, all, p))
	return;
    if (sof(s, "ACOS", ptARCCOS, all, p))
	return;
    if (sof(s, "ATAN", ptARCTAN, all, p))
	return;
    if (sof(s, "ABS", ptABS, all, p))
	return;
    if (sof(s, "DIFF", ptDIFF, all, p))
	return;

    if (sof(s, "CPU", ptCPULAST, all, p))
	return;
    if (sof(s, "SIGN", ptSIGN, all, p))
	return;
    if (sof(s, "STUNDEN", ptHOURS, all, p))
	return;
    if (sof(s, "STATE", ptSTATE, all, p))
	return;
    if (sof(s, "HOURS", ptHOURS, all, p))
	return;
    if (sof(s, "JAHR", ptYEAR, all, p))
	return;
    if (sof(s, "YEAR", ptYEAR, all, p))
	return;
    if (sof(s, "MONAT", ptMONTH, all, p))
	return;
    if (sof(s, "MONTH", ptMONTH, all, p))
	return;
    if (sof(s, "DAY", ptDAY, all, p))
	return;
    if (sof(s, "TAG", ptDAY, all, p))
	return;
    if (sof(s, "DOW", ptDOW, all, p))
	return;
    if (sof(s, "WOTAG", ptDOW, all, p))
	return;
    if (sof(s, "TIME", ptTIME, all, p))
	return;
    if (sof(s, "PREV", ptPREV, all, p))
	return;
    if (sof(s, "NOISE", ptRAND, all, p))
	return;
    if (sof(s, "INT", ptINT, all, p))
	return;
    if (sof(s, "SECONDS", ptSECONDS, all, p))
	return;
    if (sof(s, "USECS", ptUSECS, all, p))
	return;	
    if (sof(s, "SEKUNDEN", ptSECONDS, all, p))
	return;
    if (sof(s, "MINUTES", ptMINUTES, all, p))
	return;
    if (sof(s, "MINUTEN", ptMINUTES, all, p))
	return;
    if (sof(s, "MAX", ptMAX, all, p))
	return;
    if (sof(s, "MIN", ptMIN, all, p))
	return;


    if (sof(s, "d", ptRECIP, all, p))
	return;
    if (sof(s, "m", ptMINUS, all, p))
	return;
//  
// here we should have left cell coordinates and numbers only:
//
    pos = s.Find(':');		// has it a sheet prefix ?
    if (pos >= 0) {
	p += ptImpC;
	p += (word) (atol(s.before(pos).c_str()));
	s = s.after(pos);
    }
/*    
#ifndef use_string_2
    if ((char) s.at(0) > '9') {
#else
    if ((char) s.elem(0) > '9') {
#endif
*/
	
    if (s[0] > '9') {
#if debug >4
	    LOG2("cell: %s\n", s.c_str());
#endif
	all = all + "PUSHC " + s + " ";
	if (indexed)
	    p += ptIndC;
	else
	    p += ptCELL;
	p += getPos(s);
	return;
    } else {
#if debug >4
	LOG2("number: %s\n", s.c_str());
#endif
	all = all + "PUSHN " + s + " ";
	p += ptDOUBLE;
	p += atof(s.c_str());
    }
}

/*
pString& parse(string s)
{
    pString& p= *(new pString());
    s = prePars(s);
    string a;
    rparse(s, a, p, false);
#if debug >0
    LOG2("parse: %s\n", a.c_str());
#endif
/ *    
    p+=']';
    p+=6.789;
    p+=']';
    p+=6.789;
    p+='A';
* /
    return (p);
}
*/
pString& parse(string s,pString& p)
{
//    pString& p= *(new pString());
    s = prePars(s);
    string a;
    rparse(s, a, p, false);
#if debug >0
    LOG2("parse: %s\n", a.c_str());
#endif
/*    
    p+=']';
    p+=6.789;
    p+=']';
    p+=6.789;
    p+='A';
*/
    return (p);
}

/*
    Changes:
    05/06/2002  TIME(), PREV() functions added.
    06/20/2002 	* got rid of warning about signed/unsigned
		  If string index is SIGNED, why the hack is length() UNSIGNED?
		  Ok, when they are returned SIGNED, -1 can represent "not found"
		  or some condition. But this way length of *OPERATIVE* strings
		  can never exceed unsigned positive range. Or?
    10/13/2002 replaced printf(... by LOGx(... where x is the number of parameters. Log output
		is thus printed to what is defined in log.h;		  
    07/16/2002  added USeconds system var
    11/10/2004  new logging, removed ThisModule
*/
