/*
 Implementation of a driver for GE Fanuc PLCs via Ethernet.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2005.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include "mytypes.h"
#include "PLCdriver.h"
#include "cellList.h"
#include "cell.h"
#include <termios.h>
#include "setport.h"
#include "lib0.h"

#include "vi_string.h"

#define debug 6

#include <stdio.h>

extern "C" {
#define LINUX
#include <openSRTP.h>
}

#include "plclog.h"
#include "GE_Core.h"

class GEFanucDriver:public GEFcoreDriver {
  public:
    GEFanucDriver() {};
    void execute() {
	while(doRun){
	    if(di_error) {
		LOG1("execute: di error\n");
	        if (rfd>0){	
		    LOG1("closing file descriptors\n");
		    if ((wfd!=rfd) && (wfd>0)) close(wfd);
		    close(rfd);
		}	
		LOG2("execute: write fd: %d trying to reconnect\n", wfd);
		    
		rfd=setPort(portParams, &wfd);
		if(wfd>0) {
		    di_error=false;
		    openCount=0;
		} 
	    }	
	    if(!di_error) {    
		if (openSRTP()) {
		    protOK=true;
		    DI->commands=0;
		    while (protOK && doRun) {
			execList();
			execCount++;
			if(!protOK) {
			    DI->commands|=1;
			    close (wfd);
			    close (fd.rfd);
			    di_error=true;
			    wfd=-1;
			    rfd=-1;
			    usleep(999999);
			}    
		    }
		    LOG1("execute: protOK is false.\n");
		    FLUSH;
		} else {
		    usleep(200000);
		}            
		} else {
		    usleep(2000000);
		}            
	}    
	shutdown();
    };

};

#include "thread_makro.cc"
#if debug>0
DTDebug(GEFanucDriver)
#else
DT(GEFanucDriver)
#endif
