/*
 Definition of a driver for GE Fanuc PLCs via Ethernet.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2005.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#ifndef __GE_core
#define __GE_core

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include "mytypes.h"
#include "PLCdriver.h"
#include "cellList.h"
#include "cell.h"
#include <termios.h>
#include "setport.h"
#include "lib0.h"

#include "vi_string.h"

#define debug 6

#include <stdio.h>

extern "C" {
#define LINUX
#include <openSRTP.h>
}

#include "plclog.h"

char * srtpAreaName(int area);

class GEF_TI:public TFI {
  public:
    int dataArea;	// 0=Data, 1=Flags, 2=Outputs, 3=inputs
    DECLARE_PERSISTENCE(GEF_TI)
    void GEF_TI::Read(Engine & e);
};

class GEFcoreDriver:public PLCdriver {
  public:
    bool di_error;
    srtpOS_IOhandle fd;
    srtpConnection * dc;
    
    int openCount;
    GEFcoreDriver();
    bool openSRTP();
    virtual void intSetup(DriverInfo * i);
    virtual void invalidateConnection();
    virtual int protocolRead(TFI * p);
    virtual int protocolWrite(TFI * p, int PLCadr, uc * data,int length);
    virtual uc * getProtocolData();
    virtual char * getProtocolErrorMessage(int code);
    bool protOK;
    int retryCount;
//    virtual void execute();
    virtual const char * getPLCAreaName(TFI * p);
    virtual int getPLCBitsPerItem(TFI * p);
    virtual int getPLCFirstBitNumber();
};

#endif
