//#include "bin_drv.h"
#include "PLCdriver.h"
//class AllenBradley:public binDriver {
class AllenBradley:public PLCdriver {
  public:
    word DriverNr;
    word DriverState;
    int TNS;
    int lastLen, readLen;
    uc lastACK;
    uc lastMessage[512]; //saves last message for retransmit 
    uc rcvdMessage[512]; //saves last message for retransmit 
    AllenBradley() {}

    void send_ACK() {
	lastACK=ACK;
	sendLA();
    }
    
    void sendNAK() {
	lastACK=NAK;
	sendLA();
    }
    
    void sendLA() {
	char ack[2] = { DLE, ACK };
	ack[1]=lastACK;
	write(wfd, ack, 2);
    }
    
    int readChars(uc *b, int tmo) {
	fd_set FDS;
	bool charsAvail;
	int res;
	timeval t;
	t.tv_sec = 0;
	t.tv_usec = tmo;
	FD_ZERO(&FDS);
	FD_SET(rfd, &FDS);
	charsAvail = (select(rfd + 1, &FDS, NULL, NULL, &t) > 0);
	t.tv_sec = 0;
	t.tv_usec = tmo;
#if debug >2
	LOG1("readChars (here we are) \n");
#endif
	if (!charsAvail) {
	    LOG1("timeout !\n");
	    return (0);
	} else {
#if debug >2
	    LOG1("readChars reading up to 1024 chars.\n");
#endif
	    res = read(rfd, b, 1024);
//	    * real_size=res;
#if debug >2
	    int i;
#include "dumpchars3.cc"
	    LOG2("readChars %d chars read.\n", res);
	    FLUSH;
#endif
	    return (res);
	} 
    }


    int readAnswer(uc * b1) {
	int res = 0, newc;
	do {    
	    newc = readChars(b1+res, tmo_normal);
	    res += newc;
	    if (newc == 0) {
    		return -1;
	    } else {
//do we have the final DLE ETX? 
		if ((b1[res - 4] == DLE) && (b1[res - 3] == ETX)) {
#if debug >2
    		    LOG1("readAnswer() found DLE ETX.\n");
#endif
		    return res;
		} 
	    }
	} while (1);	
    }
    
    unsigned short ab_send(AB_block * ab, int size) {
	unsigned short sum = 0, retTNS;
	int j;
	unsigned char *b1 = (unsigned char *) ab;
	retTNS = ab->loTNS + 256 * ab->hiTNS;

	sum = calc_CRC((unsigned char *) ab, size);
	int j2 = 2;
	lastMessage[0] = DLE;
	lastMessage[1] = STX;
	for (j = 0; j < size; j++) {
	    lastMessage[j2] = b1[j];
	    if (b1[j] == DLE) {
		j2++;
		lastMessage[j2] = DLE;
	    }
	    j2++;
	}
	lastMessage[j2] = DLE;
	j2++;
	lastMessage[j2] = ETX;
	j2++;
	lastMessage[j2] = sum % 256;
	j2++;
	lastMessage[j2] = sum / 256;
	j2++;
	lastLen=j2;
#if debug >3	
	LOG4("I calc CRC: %x = %d,%d.\n", sum, sum % 256, sum / 256);
#endif	
	reTransmit();
	return retTNS;
    }
    
    void reTransmit() {
#if debug >5    
	LOG1("(re)transmit \n");
#endif	
	int j2=lastLen;
#ifdef sim_broken_transport
	double pr=random();
	pr/=RAND_MAX;
	
        if (pr>=0.95) {
	    LOG2("faking broken transport: %0.2f >= 0.95 \n",pr);
	    j2-=2;
	}    
#endif		    
#if debug >3
	uc * b=lastMessage;
	int i,res;
	
	res = j2;
#include "dumpchars.cc"
#endif
	write(wfd, b, j2);
    }

    int readAnswer3(uc * b1,uc ** b2, int * errorState) {
	int newc;
	readLen=0;
	int lrl=0;
	int rpos=0;
	int state=0;
	unsigned short sum;
	do {    
	    newc = readChars(b1+readLen, tmo_normal);
	    readLen += newc;
	    if (newc == 0) {
		*errorState |= 512;
		sendNAK();
    		return -1;
	    } else {
//do we have the final DLE ETX? 
		while (lrl<readLen) {
		    if (b1[lrl] == DLE) {
			if (state) {
			    rcvdMessage[rpos]=DLE;
			    rpos++;
			    state=0;
			} else {
			    state=1;
			}       
		    } else {
			if (state) {
			    switch (b1[lrl]) {
				case STX: rpos=2; break;
				case ACK: break;
				case NAK: reTransmit(); break;
				case ENQ: sendLA(); break;
				case ETX: 
				    rcvdMessage[rpos]=DLE;
				    rpos++;
				    LOG3("readAnswer3() found DLE ETX. readLen: %d lrl %d\n",
					readLen,lrl);
				    if (readLen-lrl<3) {	
					newc = readChars(b1+readLen, tmo_normal);
					readLen += newc;
					if (newc == 0) {
					    *errorState |= 512;
					    sendNAK();
    					    return -1;
					}
				    }		
				    rcvdMessage[rpos]=b1[lrl];
				    rpos++;lrl++;
				    rcvdMessage[rpos]=b1[lrl];
				    rpos++;lrl++;
				    rcvdMessage[rpos]=b1[lrl];
				    rpos++;lrl++;
				    goto ready; break;
			    }
			    state=0;
			} else {
			    rcvdMessage[rpos]=b1[lrl];
			    rpos++;
			}    
		    }
		    lrl++;
		}
//		*errorState |= 8;
//		lastACK=NAK; sendLA();
//		return (-1);
/*		
ready:		    
	        LOG1("readAnswer3() found DLE ETX.\n");
	        uc * bs = rcvdMessage;
	        readLen=rpos;
	        unsigned short crc = (bs[readLen - 2] + 256 * bs[readLen - 1]);
		    
#if debug>4
	        LOG4("recvd CRC %x %x %x.\n", crc,bs[readLen - 2],bs[readLen - 1]);
#endif
	        sum = calc_CRC(&bs[2], readLen - 6);
	        if (sum != crc) {
		    LOG3("CRC error Rcvd:%x Calc%x.\n", crc, sum);
		    *errorState |= 1;
		}
		*b2=bs;
		return readLen;
*/		
	    }
	} while (1);	
ready:		    
//	        LOG1("readAnswer3() found DLE ETX.\n");
	        uc * bs = rcvdMessage;
	        readLen=rpos;
	        unsigned short crc = (bs[readLen - 2] + 256 * bs[readLen - 1]);
		    
#if debug>4
	        LOG4("recvd CRC %x %x %x.\n", crc,bs[readLen - 2],bs[readLen - 1]);
#endif
	        sum = calc_CRC(&bs[2], readLen - 6);
	        if (sum != crc) {
		    LOG3("CRC error Rcvd:%x Calc%x.\n", crc, sum);
		    *errorState |= 1;
		}
		*b2=bs;
		send_ACK();
		return readLen;	
    }

};

