/*
 Implementation of a PLC driver for the BECKHOFF BC9000/BK9000 controller. 

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include "mytypes.h"
#include "PLCdriver.h"
#include "cellList.h"
#include "cell.h"
#include "lib0.h"
#include <stdio.h>
#include "plclog.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <errno.h>
#include "nethelp.h"

#include "byteswap.h"

#include "vi_string.h"

#include "modbusD.h"

class beckhoffDriver:public modbusDriver {
    void watchdog_reset();
    virtual int checkAnswer(int i, int count, int func);
};

void beckhoffDriver::watchdog_reset()
{
    int i;
    LOG1("watchdogReset\n");
    msg.unit_id = 1;
    msg.function = 6;
    msg.ddd[0] = 0x11;
    msg.ddd[1] = 0x21;
    msg.ddd[2] = 0xbe;
    msg.ddd[3] = 0xcf;
    sendmodbus(12);
    i=readChars(261);
    checkAnswer(i, 0, 6);

    msg.unit_id = 1;
    msg.function = 6;
    msg.ddd[0] = 0x11;
    msg.ddd[1] = 0x21;
    msg.ddd[2] = 0xaf;
    msg.ddd[3] = 0xfe;
    sendmodbus(12);
    i=readChars(261);
    checkAnswer(i, 0, 6);
}

int beckhoffDriver::checkAnswer(int i, int num_regs, int func) {
    int errC=0;
    if (i < 9) {
	if (i == 0) {
	    LOG1("unexpected close of connection at remote end\n");
	    errC|=512;
	} else {
	    LOG2("response was too short - %d chars\n", i);
	    errC|=256;
	}
    } else if (0 != (ibuf[7] & 0x80)) {
	LOG2("MODBUS exception response - type %d\n", ibuf[8]);
	errC|=ibuf[8];
	if (ibuf[8] == 4) {
	    if (num_regs) watchdog_reset();
	}
    } else if (ibuf[7] != func) {
	LOG3("wrong response %d for function %d\n", func, ibuf[7]);
	tmo_read = 10 + (3 * tmo_read) / 2;
	errC|=1024;
	return errC;
    } else if (i != (9 + num_regs)) {
	LOG3("incorrect response size is %d expected%d\n",i, (9 + num_regs));
	errC|=2048;
    } else {
    if (driverDebugLevel>11) {
	for (i = 0; i < num_regs; i++) {
	    int w = (ibuf[9 + i]);	
	    // + (ibuf[10+i+i]& 0x00FF);
	    LOG3("byte %d = %x\n", i, w);
	} 
    }
	if (!tmo_read) tmo_read = 1;
//	return errC;
    }
    if (driverDebugLevel>2)
	LOG2("about to return error code %d \n", errC);
    return errC;
}

#include "thread_makro.cc"
#if debug>0
DTDebug(beckhoffDriver)
#else
DT(beckhoffDriver)
#endif

/*
    Changes:
    08/21/2005	written
*/
