/*
 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2004.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

#include "config.h"

//extern "C" {
//#include <mysql/mysql.h>
//}


#include "mytypes.h"
#include "PLCdriver.h"
#include "cellList.h"
#include "cell.h"
#include <termios.h>
//#include "setport.h"
#include "lib0.h"

#include "vi_string.h"

#define debug 6

#include <stdio.h>

extern "C" {
#include "ADS.h"
}

#include "plclog.h"

//#define sim_broken_transport 

class ADSTI:public TFI {
  public:
    int dataArea;	// 0=Data, 1=Flags, 2=Outputs, 3=inputs
    int ackAddr;
    int repeats;
    string table;
//    int rfd,wfd;
    DECLARE_PERSISTENCE(ADSTI)
    void ADSTI::Read(Engine & e);
};

IMPLEMENT_PERSISTENCE(ADSTI, "ADS")

#define igrINPUTS 0xf020
#define igrPINPUTS 0xf060
#define igrOUTPUTS 0xf030
#define igrMARKERS 0x4020
#define igrRWDB 0x4040

char * areaName(uc n) {
    switch (n) {
	case igrRWDB:	return "Register";
	case igrMARKERS:	return "Markers";
	case igrINPUTS:		return "Inputs";
	case igrPINPUTS:		return "PeriInputs";
	case igrOUTPUTS:	return "Outputs";
	default:return "unknown area!";
    }	
}

siEntry areat[] = {
     {"D",igrRWDB},
     {"F",igrMARKERS},
     {"M",igrMARKERS},
     {"A",igrOUTPUTS},
     {"O",igrOUTPUTS},
     {"Q",igrOUTPUTS},
     {"E",igrINPUTS},
     {"I",igrINPUTS},
     };    

void ADSTI::Read(Engine & e)
{
    TFI::Read(e);
    e>>ackAddr>>repeats>>table;
    show();
    dataArea=translateByTable(area.c_str(),areat,sizeof(areat)/sizeof(siEntry));
    if (dataArea==-1)
        LOG1("Warning: data area code must be in M,D,O or Q,I (english) or M,D,A,E (german)");
#if debug >3
    LOG3("Data area      :%d (%s)\n", dataArea, areaName(dataArea));
//    LOG2("Data Block     :%d\n", areaNumber);
    LOG2("first word     :%d\n", startAddr);
    LOG2("Sheet          :%d\n", sheet);
    LOG2("Bytes          :%d\n", byteCount);
#endif
}

class ADSDriver:public PLCdriver {
  public:
    int res;
    word DriverNr;
    ADSTI *pa;
    word DriverState;
    int openCount;
    int connectionError;
    ADSInterface * di;
    ADSConnection * dc;
    AMSNetID me;
    AMSNetID other;
    int localAMSport, peerAMSport;
    ADSDriver() {};
//    MYSQL connection;
//    MYSQL_RES *result;
    
    virtual void intSetup(DriverInfo * i);
    virtual int protocolRead(TFI * p);
    virtual int protocolWrite(TFI * p, int PLCadr, uc * data,int length);
    virtual uc * getProtocolData(){
	return (uc*) dc->dataPointer;
    }
    bool protOK;
    int retryCount;
    void execute() {
	while(doRun){
	    if(connectionError) {
	        LOG1("execute: di error\n");
	        if (rfd>0){	
		    if ((wfd!=rfd) && (wfd>0)) close(wfd);
		    close(rfd);
		}	
		LOG2("execute: write fd: %d trying to connect\n", wfd);
//		res=ADSConnect(DI->s.c_str(), &rfd, &wfd);
		rfd=setPort(portParams, &wfd);
		di->fd.rfd=rfd;
		di->fd.wfd=wfd;
	        if(di->fd.wfd>0) {
		    di->error=0;
		    openCount=0;
		}	
		if(wfd>0) {
		    connectionError=0;
		    openCount=0;
		}	
		LOG2("execute: write fd: %d after connect\n", wfd);
	    }    
	    if (connectionError==0) {
	        protOK=true;
	        DI->commands=0;
	        while (protOK && doRun) {
		    LOG1("execute: before execList()\n");
		    res=execList();
		    if (res!=0) protOK=false;
		    LOG1("execute: after execList()\n");
		    execCount++;
		    if(!protOK) {
			LOG1("execute: error detected.\n");
		        DI->commands|=1;
		        close (wfd);
		        connectionError=1;
		        wfd=-1;
		        rfd=-1;
		        usleep(999999);
		    }    
		}
		LOG1("execute: protOK is false.\n");
		FLUSH;
	    } else {
	        usleep(200000);
	    }            
	}    
    };
    
    virtual const char * getPLCAreaName(TFI * p) {
	if (p->area=="D") return "dataArea=igrRWDB?";
	else if (p->area=="M") return "%M";
	else if (p->area=="A") return "%Q";
	else if (p->area=="O") return "%Q";
	else if (p->area=="Q") return "%Q";
	else if (p->area=="E") return "%I";
	else if (p->area=="I") return "%I";
	else if (p->area=="P") return "dataArea=igrPINPUTS?";
    }    
};

AMSNetID me={172,16,17,8,1,1};
AMSNetID other={172,16,17,7,1,1};

void ADSDriver::intSetup(DriverInfo * ddi)
{
    _ADSOSserialType fds;
    localAMSport=300;
    peerAMSport=800;
    PLCdriver::intSetup(ddi);
    ADSDebug=libraryDebugLevel;
    fds.rfd = setPort(portParams, &fds.wfd);
    rfd=fds.rfd;
    wfd=fds.wfd;    
    LOG2("intSetup write fd: %d\n", wfd);
    retryCount=0;
    openCount=0;
    DI=ddi;
    if (wfd!=failed_fd) {
	di=ADSNewInterface(fds, me, localAMSport,"IF1");
	dc=ADSNewConnection(di, other, peerAMSport);
    }
    TypeManager::Add("ADS", CreateNewADSTI);
    loadList(DI);
    bitMask = registerDriver("Beckhoff ADS", this);
    LOG3("intSetup done fd: %d errorMask %x\n", wfd, bitMask);
/*
    
    if (!mysql_init(&connection)) {
	LOG1(ThisModule "Error initializing client. mysql_init() failed.\n");
    } else {
	LOG1(ThisModule "Client initialized.\n");
    }
    FLUSH;
    if (!mysql_real_connect(&connection, host.c_str(),	// host
			    user.c_str(),	// user
			    passwd.c_str(),	// password
			    database.c_str(),	//"testdata", // database
			    0,	// port
			    NULL,	// unix socket
			    0	// client flags         
	)) {
	LOG2(ThisModule "Connection error.%s\n",
	       mysql_error(&connection));
    } else {
	LOG1(ThisModule "Connection established.\n");
    }
*/    
    FLUSH;
};

int ADSDriver::protocolRead(TFI * p) {
    LOG1("Hello from protocolRead()\n");
    int i,len,j;
    
    us* dp;
    ADSTI *pa=(ADSTI *)p;
    
    ADSTI p2=*pa;
    
    len=p->byteCount;
    if(pa->repeats>0) len=len*pa->repeats;
     
    int res=ADSreadBytes(dc, 
	pa->dataArea, /*p->areaNumber,*/ p->startAddr, len, NULL);
#ifdef sim_broken_transport
    
    double pr=random();
    pr/=RAND_MAX;
	
    if (pr>=0.95) {
        LOG2("faking broken transport: %0.2f >= 0.95 \n",pr);
	res=-1;
    }    
#endif	

#if debug >5
    LOG2("protocolRead: ADSreadBytes returned:%d\n", res); 
#endif	    	
    if(res==0) {
	dp=(us*)dc->dataPointer;
	string sx;
	i=0;
//	char buf[1024];
	
	
	
	while(i<pa->repeats) {
	    LOG2("store %d to DB\n", i); 
	    for (j=0;j<pa->byteCount;j++) {
		storeByFormat((uc*)dc->dataPointer, &p2, 0, 1);
	    }
	    
	    i++;
//	    ((uc*)dc->dataPointer)+=p->byteCount;
	    dc->dataPointer+=p->byteCount;
	    p2.startCell.row++;
	    p2.startAddr+=p->byteCount;
/*	    
	    sprintf(buf,"INSERT INTO %s VALUES(0,%d)",pa->table.c_str(),val);
	    int qres = mysql_real_query(&(connection), sx.c_str(), sx.length());
	    
	    if (qres) {
		LOG3(ThisModule "Query returned %d %s\n", qres, mysql_error(&(connection)));
		FLUSH;
	    } else {
//		m->result = mysql_use_result(&(connection));
	    }
	    
	    }
*/	    
	}
	if(pa->repeats>0){
	    LOG1("reset full\n"); 
	    int data=0;
	    int res=ADSwriteBytes(dc, pa->dataArea, pa->ackAddr, 1, &data);
	    return -1111;  // just a kludge to let PLCdriver treat the result specially
	}        
	
	
	return 0; 
    }
    else{
      protOK=false;
      return -1;
    }  
}

int ADSDriver::protocolWrite(TFI * p, int PLCadr, uc*data, int length) {
    LOG3("protocolWrite(dat:%p, len:%d)\n",data,length);
    ADSTI *pa=(ADSTI *)p;
    int res=ADSwriteBytes(dc, pa->dataArea, /*p->areaNumber,*/ PLCadr, length, data);
    return res;
}

#include "thread_makro.cc"
#if debug>0
DTDebug(ADSDriver)
#else
DT(ADSDriver)
#endif
/*
    Changes:
    03/11/2005 	introduced doRun flag, so the driver stops on request.
    03/12/2005 	made ADS AMSNetIDs configureable.
*/
