/*
 Part of multithreaded network servers. The servers MAIN thread is started by
 the main program. It starts in turn the ACCEPTER thread, which accepts a
 connection and passes the file descriptor to the MAIN thread. The server MAIN
 thread starts a PORT SERVER thread to handle the connection. Thus, the 
 PORT SERVER thread is NOT a child of ACCEPTER, but a brother and SIGCHILD goes
 to the server MAIN. If it would go to ACCEPTER, which is blocked in the 
 accept() system call, we would get "process interrupted in system call" and 
 that's it. Maybe its different with pthread library..?

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

/*
    This waits in select for a file descriptor from accepter and starts a new child server
    with this file descriptor.
*/
extern "C" void * DriverThread(void * arg);
void *DriverThread(void *arg)
{
    int filedes[2], res, newfd;
    string s = ((DriverInfo *) arg)->s;
    LOG2(ThisModule "Main serv: %s\n", ((DriverInfo *) arg)->s.c_str());
#ifndef use_string_2
    string s2 = s.substr(s.find('=') + 1);
#else
    string s2 = s.after(s.index('='));
#endif
    LOG2(ThisModule "Main serv: %s\n", s2.c_str());

    fd_set FDS;
    pipe(filedes);
    pthread_attr_t attr;
    pthread_t ac, ps;
    accepter_info ai;
    ai.port = atol(s2.c_str());
    LOG2(ThisModule "Main serv: %d\n", ai.port);
    LOG2(ThisModule "Main serv: Accepter pipe fd: %d\n", ai.fd);
    ai.fd = filedes[1];
//    ai.port=182;
    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr,PTHREAD_CREATE_DETACHED);
    res=pthread_create(&ac, &attr, accepter, &ai /*&filedes[1] */ );
    do {
	FD_ZERO(&FDS);
	FD_SET(filedes[0], &FDS);

	LOG2(ThisModule "Main serv: about to select on %d\n",
	       filedes[0]);
	FLUSH;
	if (select(filedes[0] + 1, &FDS, NULL, &FDS, NULL) > 0) {
	    LOG1(ThisModule "Main serv: about to read\n");
	    res = read(filedes[0], &newfd, sizeof(newfd));
	    LOG3(ThisModule
		   "Main serv: about to create new port server for fd:%d read res:%d\n", newfd,res);
	    ps=0;		   
	    pthread_attr_init(&attr);
	    pthread_attr_setdetachstate(&attr,PTHREAD_CREATE_DETACHED);
	    res=pthread_create(&ps, &attr, portServer, &newfd);
	    LOG3(ThisModule
		   "Main serv: create res:%d %ld\n", res, ps);
	    if(res) {
		LOG2(ThisModule
		   "Main serv: create error:%s\n", strerror(res));		   
		close(newfd);
		usleep(100000);
	    }	   
	}
    }
    while (1);
}
