/*
 Implementation of a PLC driver for the BECK/FESTO EASY PLCs. 

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2004.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/


#include "easyIP.h"
/*
#define PORT_EASYIP                     995
#define MAX_EASYIP_DATASIZE             256
#define EASYIP_ERROR_OPERAND           0x01
#define EASYIP_ERROR_OFFSET            0x02
#define EASYIP_ERROR_DATASIZE          0x04

#define TimeoutDefault                    2

#define easyTypeMEM	1
#define easyTypeINPUT	2
#define easyTypeOUTPUT	3
#define easyTypeREGISTER	4
#define easyTypeTIMER	5

/ *
    We need exact positioning of component fields to reflect the PLC?s packet structure.
* /
#pragma pack (1)

typedef struct
{
  unsigned char   flags;	// 0x80: response packet, 0x40 do not respond	
  unsigned char   error;	// 0=ok, 1,2,4 see above
  unsigned short  counter;	// set by client,copied by server
  unsigned short  short_index;	// part of counter?
  unsigned char   spare1;	
  unsigned char   senddata_type;	// easyTypeXXXX
  unsigned short  senddata_size;	// number of words
  unsigned short  senddata_offset;	// start address in server
  unsigned char   spare2;		
  unsigned char   reqdata_type;
  unsigned short  reqdata_size;
  unsigned short  reqdata_offset_server;
  unsigned short  reqdata_offset_client;
  unsigned char   data[256];
} easyIPpacket;


typedef struct
{
  unsigned short  active;
  unsigned short  timeout;
  unsigned short  counter;
  unsigned short *ptr_status;
} struct_easyIPentry;
*/

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
//#include <termios.h>

#include <stdio.h>
#include <errno.h>

#include "plclog.h"
#define ThisModule "EASYIP   : "

#include <fcntl.h>
/* 
    include this to use sockets
*/    
#include <sys/socket.h>	
#include <sys/un.h>	
#include <netinet/in.h> 
#include <arpa/inet.h>	


int beckEasyDebug=0;

int beckConnect(const char * peer, int * rfd, int * wfd)
{
    struct sockaddr_in addr;
    socklen_t addrlen;
    int res;
//    peer = noSpace(peer);
//    tmo_read = 16291;	// ????
    while (*peer==' ') peer++;
    if(beckEasyDebug>0) {
	LOG2("enter OpenSocket for peer (%s)\n", peer);
	FLUSH;
    }
    addr.sin_family = AF_INET;
    addr.sin_port = (((PORT_EASYIP) & 0xff) << 8) | (((PORT_EASYIP) & 0xff00) >> 8);
    inet_aton(peer, &addr.sin_addr);
    *rfd = *wfd= socket(AF_INET, SOCK_DGRAM, 0);
    res=fcntl(*rfd, F_SETFL, O_NONBLOCK);
    if(beckEasyDebug>0) {
        LOG3("Set O_NONBLOCK %d error: %s\n", res, strerror(errno));
    }
    if(beckEasyDebug>0) {
	LOG2("OpenSocket: socket is %d\n", wfd);
    }
    addrlen = sizeof(addr);
    errno=0;
    if (connect(*rfd, (struct sockaddr *) & addr, addrlen)) {
	if(beckEasyDebug>0) {
	    LOG2(" Socket error: %s ", strerror(errno));
	}
	close(*rfd);
	*rfd = *wfd= -1;
    } else {
	if(beckEasyDebug>0) {
	    LOG2(" Connected to host: %s \n", peer);
	}
	errno=0;
	res=fcntl(*rfd, F_SETFL, O_NONBLOCK);
	if(beckEasyDebug>0) {
	    LOG3("Set O_NONBLOCK %d error: %s\n", res, strerror(errno));
	}
    }	
    FLUSH;
}

int counter;	// no good idea !! conflicts when multiple instances.
//int readChars(int rfd,int size,uc * buf);

int beckReadBytes(int rfd,int wfd, int dataArea, int startAddr, int wordCount, uc * buffer) {
    easyIPpacket p, p2;
    int res;
    
    memset(&p, 0, sizeof(p));

    p.reqdata_type=dataArea;
    p.reqdata_size=wordCount;
    p.reqdata_offset_server=startAddr;
//    p.reqdata_offset_client= // what to do with that??
    counter++;
    p.counter=counter;
    if(beckEasyDebug>0) {
        LOG1("before write\n");
    }
    write(wfd, &p, sizeof(p)-256);
    if(beckEasyDebug>0) {
        LOG1("before read\n");
    }
/*
    This could be a simple read, if the O_NONBLOCK would work,
    but when I kill the server, read blocks.
*/    
    res=readChars(rfd, (uc *)(&p2), sizeof(p2));
//    res=read(rfd, (uc *)(&p2), sizeof(p2));
    if (res<=0) return -1;		// no response
    if(p2.flags & 0x80 ==0) return -2;	// not a response
    if(p2.counter !=counter) {
	if(beckEasyDebug>0) {
	    LOG3("My Counter %d response count:%d\n", counter, p2.counter);
	}
	return -3;	// not the response to this request
    } 
    if(p2.reqdata_size !=wordCount) return -4; // short packet
    memcpy(buffer, &p2.data, 2*wordCount);
    return 0;
//    read(rfd,&p2, sizeof(p2));
}

int beckWriteBytes(int rfd,int wfd, int dataArea, int startAddr, int wordCount, uc * data) {
    easyIPpacket p, p2;
    int res;
    
    memset(&p, 0, sizeof(p));

    p.senddata_type=dataArea;
    p.senddata_size=wordCount;	
    p.senddata_offset=startAddr;
    memcpy(&p.data, data, 2*wordCount);
    
    counter++;
    p.counter=counter;
    if(beckEasyDebug>0) {
        LOG1("before write\n");
    }
    write(wfd, &p, sizeof(p)-sizeof(p.data)+2*wordCount);
    
    if(beckEasyDebug>0) {
        LOG1("before read\n");
    }
/*
    This could be a simple read, if the O_NONBLOCK would work,
    but when I kill the server, read blocks.
*/    
    res=readChars(rfd, (uc *)(&p2), sizeof(p2));
    if (res<=0) return -1;		// no response
    if(p2.flags & 0x80 ==0) return -2;	// not a response
    if(p2.counter !=counter) {
	if(beckEasyDebug>0) {
	    LOG3("My Counter %d response count:%d\n", counter, p2.counter);
	}
	return -3;	// not the response to this request
    } 
    return 0;
}

//extern  INTVECT OldTmrInt;
//INTVECT OldTmrInt;

#define tmo_read 500000

int readChars(int rfd,uc * buf,int size)
{
    fd_set FDS;
    int charsAvail;
    unsigned char *b;
    int i;
    int res;
    struct timeval t;
    t.tv_sec = 0;
    t.tv_usec = tmo_read;
    usleep(tmo_read);
    FD_ZERO(&FDS);
    FD_SET(rfd, &FDS);
    charsAvail = select(rfd + 1, &FDS, NULL, NULL, &t);
    LOG2("readChars %d\n", charsAvail);
    if (charsAvail<=0) {
	LOG2(" timeout %d \n", charsAvail);
    }
    if (charsAvail>0) {
	LOG2("readChars reading up to %d chars.\n", size);
	res = read(rfd, buf, size);
	LOG2("readChars %d chars read.\n", res);
	FLUSH;
	return (res);
    } else
	return (0);
}
