/*
 Implementation of a PLC driver for the BECK/FESTO EASY PLCs. 

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2004.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#include "mytypes.h"


#define WORD  unsigned int
#define BYTE  unsigned char
#define DWORD unsigned long

#define USER_MEM_SIZE     2048

//************ global variables  *********************
//int EASYIP_sd;
//BYTE far *GbpMyMem;



//****  Plattform switch ***************
//#define MWT
#define DOS


//#define DEBUG     // DEBUG Switch

//typedef void (interrupt *INTVECT)();



#define PORT_EASYIP                     995
#define MAX_EASYIP_DATASIZE             256
#define EASYIP_ERROR_OPERAND           0x01
#define EASYIP_ERROR_OFFSET            0x02
#define EASYIP_ERROR_DATASIZE          0x04

#define TimeoutDefault                    2

#define easyTypeMEM	1
#define easyTypeINPUT	2
#define easyTypeOUTPUT	3
#define easyTypeREGISTER	4
#define easyTypeTIMER	5

/*
    We need exact positioning of component fields to reflect the PLC?s packet structure.
*/
#pragma pack (1)

typedef struct
{
  unsigned char   flags;	// 0x80: response packet, 0x40 do not respond	
  unsigned char   error;	// 0=ok, 1,2,4 see above
  unsigned short  counter;	// set by client,copied by server
  unsigned short  short_index;	// part of counter?
  unsigned char   spare1;	
  unsigned char   senddata_type;	// easyTypeXXXX
  unsigned short  senddata_size;	// number of words
  unsigned short  senddata_offset;	// start address in server
  unsigned char   spare2;		
  unsigned char   reqdata_type;
  unsigned short  reqdata_size;
  unsigned short  reqdata_offset_server;
  unsigned short  reqdata_offset_client;
  unsigned char   data[256];
} easyIPpacket;


typedef struct
{
  unsigned short  active;
  unsigned short  timeout;
  unsigned short  counter;
  unsigned short *ptr_status;
} struct_easyIPentry;

int beckConnect(const char * peer, int * rfd, int * wfd);
extern int beckEasyDebug;
extern int beckReadBytes(int rfd,int wfd, int dataArea, int startAddr, int wordCount, uc * buffer);
extern int beckWriteBytes(int rfd,int wfd, int dataArea, int startAddr, int wordCount, uc * data);


