#include <unistd.h>
#include <sys/time.h>
#include "modbus.h"
#include "plclog.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#define DONT_USE_GETHOSTBYNAME

char * modbusFunctionName(int function) {
    switch (function) {
    case mbfuncLOOP_BACK_TEST: return "loopback test";
    case mbfuncREAD_OUTPUT_REGISTERS: return "read output registers";
    case mbfuncREAD_INPUT_STATUS: return "read input status";
    case mbfuncREAD_OUTPUT_STATUS: return "read output status";
    case mbfuncREAD_INPUT_REGISTERS: return "read input registers";
    case mbfuncFORCE_SINGLE_COIL: return "force single coil";
    case mbfuncPRESET_SINGLE_REGISTER: return "preset single register";
    case mbfuncFORCE_MULTIPLE_COILS: return "force multiple coils";
    case mbfuncPRESET_MULTIPLE_REGISTERS: return "preset multiple registers";
    default: return "not implemented";
    }
};

int modbusSend(int len, int * transaction_id, int tmo_read, struct modbusMessage * msg, int wfd) {
    int mblen;
    fd_set FDS;
    int canSend;
    struct timeval t;
    uc *b;
    mblen = len - 6;
    t.tv_sec = 0;
    t.tv_usec = tmo_read;
#if debug > 1
    LOG1("sendmodbus enter.\n");
    FLUSH;
#endif
    msg->hi_ti = *transaction_id / 256;
    msg->lo_ti = *transaction_id % 256;
    msg->hi_pi = 0x0;
    msg->lo_pi = 0x0;
    msg->hi_len = mblen / 256;
    msg->lo_len = mblen % 256;
    msg->unit_id = 1;
    b = (uc *) msg;
#if debug>1        
    int i;
#endif    
    int res=len;
#if debug>1    
#include "dumpchars2.cc"    
#endif
    if (wfd<=0){
	LOG2("sendmodbus: wfd is %d!\n",wfd);
	return;
    }	
    FD_ZERO(&FDS);
    FD_SET(wfd, &FDS);
    canSend = select(wfd + 1, NULL, &FDS, NULL, &t);
    if (canSend>0) {
#if debug>1    
	LOG1("sendmodbus: about to call write\n");
#endif	
	res = write(wfd, b, len);
	*transaction_id++;
#if debug>1
	LOG2("sendmodbus: %d chars send.\n", res);
#endif	
	return res;
    } else {
	return -1;
//	LOG1("sendmodbus: cannot write. Must reconnect\n");
//	openSocket();
    }	
}

int openSocket(const int port, const char * peer) {
    int fd,res,opt;
    struct sockaddr_in addr;
    socklen_t addrlen;
#ifndef DONT_USE_GETHOSTBYNAME    
    struct hostent *he;
#endif    
    LOG1("enter OpenSocket");
    FLUSH;
    addr.sin_family = AF_INET;
    addr.sin_port =htons(port);
#ifndef DONT_USE_GETHOSTBYNAME
    he = gethostbyname(peer);
    memcpy(&addr.sin_addr, he->h_addr_list[0], sizeof(addr.sin_addr));
#else
    inet_aton(peer, &addr.sin_addr);
#endif

    fd = socket(AF_INET, SOCK_STREAM, 0);
    LOG2("OpenSocket: socket is %d\n", fd);
    LOG3("setsockopt %s %d\n", strerror(errno),res);
    
    addrlen = sizeof(addr);
    if (connect(fd, (struct sockaddr *) & addr, addrlen)) {
	LOG2("Socket error: %s \n", strerror(errno));
	close(fd);
	fd = 0;
    } else {
        LOG2("Connected to host: %s \n", peer);
	errno=0;
	opt=1;
	res=setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, &opt, 4);
        LOG3("setsockopt %s %d\n", strerror(errno),res);	
    }	
    FLUSH;
    return fd;
}
