/*
 Implementation of the network server. The servers MAIN thread is started by
 the main program. It starts in turn the ACCEPTER thread, which accepts a
 connection and passes the file descriptor to the MAIN thread. The server MAIN
 thread starts a PORT SERVER thread to handle the connection. Thus, the 
 PORT SERVER thread is NOT a child of ACCEPTER, but a brother and SIGCHILD goes
 to the server MAIN. If it would go to ACCEPTER, which is blocked in the 
 accept() system call, we would get "process interrupted in system call" and 
 that's it. Maybe its different with pthread library..?

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <pthread.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include "plcdrv.h"
#include "nethelp.h"
#include "mytypes.h"
#include "lib0.h"
#include "cellList.h"
//#include "shsaver.h"



#include "vi_string.h"

#include "plclog.h"

#define ThisModule "modbus_sl: "

#define debug 10

#include "accepter.cc"

#define	_32bit_big_endian
#define uc unsigned char 
/*
    Transmit a packet
*/
void respond(int fd,uc * b, int len) {
    b[4]=len / 256;	// Length field 
    b[5]=len % 256;
    len+=6;
#if debug >3
    int i,res = len;
#include "dumpchars.cc"
#endif
    write(fd, b, len);
}

/*
    Send an ack packet
*/
void ackResponse(int fd, uc * packet) {
    packet[8]=packet[1];
    packet[9]=packet[2];
    packet[10]=0;
    packet[11]=1;
    respond(fd,packet,6);
}    


void parseLine(int fd, uc * msg, int mask)
{
    union {
	float a;
	uc b[4];
    } f;
    union {
	short a;
	uc b[2];
    } w;
    float aa;
    coord c;
    bool ok;
    int wc=256*msg[10]+msg[11];
    int ad=256*msg[8]+msg[9];
    int sheet=msg[6];  // the unit
    c.col=1+msg[8];
    c.row=1+msg[9];
    c.col=1+ad/100;
    c.row=1+ad%100;
#if debug >3
	    uc * b=msg;
	    int i,res;    
#endif	    
    switch(msg[7]){
	case 3:  //registers
	case 4:  //and input registers are the same for VISUAL
	    aa=numContents(sheet,c,ok);
	    res=0;
	    if (ok) {
		f.a=aa;//+1e-19;
		if (wc==2) {
		    msg[8]=4;
		    msg[9]=f.b[3];
		    msg[10]=f.b[2];
		    msg[11]=f.b[1];
		    msg[12]=f.b[0];
		    msg[5]=7;
		    res = 13;
		} else {
		    if(wc==1) {
			w.a=(int)f.a;
			msg[8]=2;
			msg[9]=w.b[1];
			msg[10]=w.b[0];
			msg[5]=5;
			res = 11;
		    } else {
			msg[7]|=0x80;
			msg[8]=2; 		//Illegal Data Address
			LOG3(ThisModule "Illegal Data size for:  %d:%s \n",sheet,cellN(c));
			respond(fd,msg,3);
			FLUSH;	
		    }
		}
		LOG5(ThisModule "port server: read multiple(%d) registers from %d:%s =%g.\n", wc,sheet,cellN(c),f.a);
		FLUSH;
#if debug >3
#include "dumpchars.cc"
#endif	    
		write(fd,msg,res);
	    } else {
		msg[7]|=0x80;
		msg[8]=2; 		//Illegal Data Address
		LOG3(ThisModule "Illegal Address:  %d:%s \n",sheet,cellN(c));
		respond(fd,msg,3);
		FLUSH;
	    }
	    break;
	case 0x10: 
	    if (wc==2) {
#ifdef	_32bit_big_endian
		f.b[0]=msg[10+6];
		f.b[1]=msg[10+5];
		f.b[2]=msg[10+4];
		f.b[3]=msg[10+3];
#endif		   
		LOG3(ThisModule "write output words %g to %s\n",f.a,cellN(c));
		setCell(sheet, c, f.a, soExternal);		
	    }
	    if (wc==1) {
#ifdef	_32bit_big_endian
		w.b[0]=msg[10+4];
		w.b[1]=msg[10+3];
#endif		   
		LOG3(ThisModule "write output words %d to %s\n",w.a,cellN(c));
		setCell(sheet, c, w.a, soExternal);		
	    }
	    LOG3(ThisModule "write output words %g to %s\n",f.a,cellN(c));
	    ackResponse(fd, msg);
	    break;    
	default:
	    msg[7]|=0x80;
	    msg[8]=1; 		//Illegal function code
	    respond(fd,msg,3);
	    LOG2(ThisModule "unimplemented function %d \n", msg[7]);
	    FLUSH;
    }
}

void *portServer(void *arg)
{
    int fd,rr;
    PLCdrv *dummy = new PLCdrv;
    int changeMask =0;
//     registerDriver("server", dummy);
    dummy->errorMask = changeMask;
//    dummy->PID=getpid();
    uc buffer[bSize];
    string line;
    fd = *(int *) arg;
    LOG2(ThisModule "portServer: My fd is:%d\n", fd);
    FLUSH;
    int waitCount = 0;
    while (waitCount < 300) {
	while ( (rr=read(fd, &buffer[0], 6)) ) {
	    if (rr==6) {
		rr+=read(fd, &buffer[6], buffer[5]);
	    }	
	    LOG2(ThisModule "port server: request:%d\n", rr);
	    if (rr>6) {
#if debug >3
		uc * b=buffer;
		int i,res;
		res = rr;
#include "dumpchars.cc"
#endif	    
		FLUSH;
		parseLine(fd, buffer, changeMask);
		usleep(10000);
	    }
	}
	waitCount++;
    }
    LOG1(ThisModule "portServer: I closed my fd.\n");
    FLUSH;
    unRegisterDriver(changeMask);
//    destroy(dummy);
    return NULL;
}

#include "driverThread.cc"
/*
    Changes:
    
    02/12/2002  server uses port given in visual.cfg
    05/13/2002  Cosmetical changes to align debug output.
    06/27/2002  added missing returns in savesteeing,savesheet
    
*/
