/*

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>

#include "vi_string.h"

#define bSize 1024
#define debug 0

bool readline(int &fd, char *buffer, string & line, int &wpos, int &rpos)
{
    int i;
    char c;
    fd_set FDS;
    timeval tv;

    line = "";
  bread:
    if (fd < 0)
	return (false);
#if debug >2
    LOG2("nethelp  : w: %d r: %d\n", wpos, rpos);
    FLUSH;
#endif
    while (rpos != wpos) {	// do we still have buffered chars?
	c = *(buffer + rpos);	// read from buffer
	if ((c != 0x0a) && (c != 0x0d)) {	// end of line ?
	    line += c;		// no, append to line
	}
	rpos++;			// next
	if (rpos >= bSize)
	    rpos = 0;		// end of buffer ?, use it from start if so
	if (wpos >= bSize)
	    wpos = 0;
	if (c == 0x0a)
	    return (true);
    }
//
// no more buffered chars, read from file descriptor:
//
#if debug >1
    LOG2("nethelp  : read from fd to buffer\n");
    FLUSH;
#endif
    tv.tv_sec = 3;
    tv.tv_usec = 3;
    FD_ZERO(&FDS);
    FD_SET(fd, &FDS);
    if (select(fd + 1, &FDS, NULL, NULL, &tv) > 0) {
//
// This will fill the ringbuffer up to last position at maximum. If there are
// still more chars, first the buffers contents is processed and then wpos
// will be set back in the above "read from buffer" code.
//    
	i = read(fd, buffer + wpos, bSize - wpos);
	if (i > 0) {
	    wpos += i;
	    goto bread;
	}
    }
#if debug >1
    LOG2("nethelp  : read timeout! I closed the socket!\n");
    FLUSH;
#endif
    close(fd);
    fd = -1;
    return (false);
}
