/*
 Definitions of a skeleton PLC driver and the instructions it can process.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#ifndef __plcdrv
#define __plcdrv

#include <sys/time.h>
#include "mytypes.h"
#include "list.h"
#include "persist.h"
#include "vi_string.h"
//#include "driverInfo.h"

struct driverError {
    time_t time;
    int level1;
    int level2;
};

/*
    Definition of a transport instruction for all PLC drivers.
    One instruction describes a block of data in PLC which is copied
    to or from a column of the spreadsheet beginning at coordinate
    startCell. Transport instructions are held in a list, which is 
    executed round and round by the driver. Conditional execution by be 
    achieved using controlCell.
*/

class transferInstruction:public base {
//  DECLARE_PERSISTENCE(transferInstruction) public:
public:
    int sheet;			// a sheet used for this instruction
    int startMW;		// first address in PLC
    int endMW;			// last address in PLC
    int direction;		// 0=read from PLC, 1=write to PLC
    int DBNumber;		// PLC data block number or data area, 
    // interpretation depends on driver 
    coord startCell;		// first cell, corresponds to startMW
    // consecutive rows are used until endMW is reached. 
    coord ctrlCell;		// instruction is only executed if this cell exists
    // and has value != 0
    string comment;		// a comment                    
     transferInstruction();
//    virtual string listIt();
//    virtual bool Write(Engine & e) const;
    virtual void Read(Engine & e);
};

#include "driverinfo.h"
/*
//
// what the loader passes to a PLC driver:
//
struct DriverInfo {
    pthread_t th;
    string s1;
    string s;
};
*/
//
// PLC driver prototype:
//
class PLCdrv:public base {
  public:
    DriverInfo * DI;
    int rfd;			// a file Descriptor to read from
    int wfd;			// a file Descriptor to write to
    int errorMask;		// bit mask to mark errors as originating from this driver
    int changedMask;		// bit mask to extract drivers "new value flag"
    // set by calculation thread
    bool execError;		// error during execution
    pthread_t handle;
    string params;		// some parameters to setup serial line
    driverError lastError;
//private:    
    int _PID;
    bool isRunning;
public:    
    int execCount;
     PLCdrv();
    list *il;			//list of transfer instructions
    virtual void load(DriverInfo *);
    virtual void execList();
    virtual void execTI(transferInstruction * ti);
//    inline int getPID(){return PID;}
    void clrPushButton(int sheet, coord & cell);
    virtual void intSetup(DriverInfo * i);
    void hexDump(char * text,uc * buffer, int len);
};

#endif
/*
    Changes:
    
    05/13/2002  Object Read & write functions return void now. 
    07/25/2002  Mechanism to reload on command (norammly given by loader on SIGHUP).
*/
