/*
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2005.

 LibOpenSRTP is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 LibOpenSRTP is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with Libnosrtp; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

/*
    prevent redefinition in case of multiple includes:
*/
#ifndef __nosrtp
#define __nosrtp

/*
    Some data types:
*/
#define uc unsigned char
#define us unsigned short
#define u32 unsigned int

/*
    A data type containig whatever the underlying OS or system uses to identify the I/O connections.
    The 2 filedescriptors for UNIX/LINUX are there to strictly separate read and write operations. 
    Normally, they have identical values, burt they may be useful:
    a) to "pipe" data through half duplex pipes.
    b) if some OS or environment, e.g. on a microcontroller needs to deal with separate 
       structures for read and write buffers, function pointers etc.
*/
#ifdef LINUX
#define DECL2
#define EXPORTSPEC
typedef struct {
    int rfd;
    int wfd;
} srtpOS_IOhandle;
#include <stdlib.h>
#else    
#ifdef CYGWIN
typedef struct {
    int rfd;
    int wfd;
} srtpOS_IOhandle;
#include <stdlib.h>
#else    
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
typedef struct {
    HANDLE rfd;
    HANDLE wfd;
} srtpOS_IOhandle;
#else
#ifdef BCCWIN
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
//#define DECL2 WINAPI
#define DECL2
#ifdef DOEXPORT
#define EXPORTSPEC __declspec (dllexport)
#else
#define EXPORTSPEC __declspec (dllimport)
#endif
typedef struct {
    HANDLE rfd;
    HANDLE wfd;
} srtpOS_IOhandle;
#else
#error Fill in what you need for your OS or API.
#endif
#endif
#endif
#endif

/*
    GE Fanuc "segment selector" values. They are defined with 2 different name each, so I hope
    everybody will find a name he is used to or can easily remember.
*/
#define srtpAreaInputBit 0x46
#define srtpAreaInputByte 0x10
#define srtpAreaIBit 0x46
#define srtpAreaIByte 0x10

#define srtpAreaOutputBit 0x48
#define srtpAreaOutputByte 0x12
#define srtpAreaQBit 0x48	// This is not the QBIT for Quantum Computers !!	
#define srtpAreaQByte 0x12

#define srtpAreaTimerBit 0x4a
#define srtpAreaTimerByte 0x14
#define srtpAreaTBit 0x4a
#define srtpAreaTByte 0x14

#define srtpAreaFlagBit 0x4c	// %M as bits
#define srtpAreaFlagByte 0x16	// %M as bytes
#define srtpAreaMBit 0x4c	// %M as bits
#define srtpAreaMByte 0x16	// %M as bytes

#define srtpAreaSABit 0x4e
#define srtpAreaSAByte 0x18
#define srtpAreaSBBit 0x50
#define srtpAreaSBByte 0x1a
#define srtpAreaSCBit 0x52
#define srtpAreaSCByte 0x1c
#define srtpAreaSBit 0x54
#define srtpAreaSByte 0x1e

#define srtpAreaGlobalBit 0x56	// %G
#define srtpAreaGlobalByte 0x38	
#define srtpAreaGBit 0x56	// %G
#define srtpAreaGByte 0x38	


#define srtpAreaAnalogInput 0x0a
#define srtpAreaAnalogOutput 0x0c
#define srtpAreaRegisters 0x08
#define srtpAreaSubblockLocal 0x0
#define srtpAreaProgramBlockData 0x4

#define srtpAreaAI 0x0a
#define srtpAreaAQ 0x0c
#define srtpAreaR 0x08
#define srtpAreaL 0x0
#define srtpAreaP 0x4
/*
    Some error codes. These are defined by this library and not by the CPU.    
*/
#define srtpResOK 0
#define srtpErrorNoResponse -1
#define srtpErrorUnknownPacketType -2
#define srtpErrorNAKPacketType -3
#define srtpErrorWrongSequence -4

#define srtpMaxPacketLen 2000	// dont think its really longer than a single ethernet packet 

EXPORTSPEC char * DECL2 srtpStrerror(int code);

typedef struct _srtpConnection  srtpConnection;
/*
    Connection "object". Quite simple here. The separate file descriptors can be used:
    
*/
struct _srtpConnection {
    srtpOS_IOhandle fd;
    us sequence;
    uc * resultPointer;	/* used to retrieve single values from the result byte array */
    uc * _resultPointer;
    uc msgIn[srtpMaxPacketLen];
    int AnswLen;
};

/*
    create a new connection structure:
*/
EXPORTSPEC srtpConnection * DECL2 srtpNewConnection(srtpOS_IOhandle ioh);

/*
    send a packet to PLC:
*/
EXPORTSPEC int DECL2 srtpWrite(srtpConnection * sc, uc * msg, int len);

/*
    read a packet from PLC:
*/
EXPORTSPEC int DECL2 srtpRead(int fd, uc *b, int max);
/*
    Read count items from PLC memory segment "area".
    Return an error code.
    If a buffer pointer is provided, data will be copied into this buffer.
*/
EXPORTSPEC int DECL2 srtpReadBytes(srtpConnection * sc, int area, int start, int count, void * buffer);

/*
    Write count items from buffer into PLC memory segment "area".
    Return an error code.
*/
EXPORTSPEC int DECL2 srtpWriteBytes(srtpConnection * sc, int area, int start, int count, void * buffer);

/*
    packet hexdump:
*/
EXPORTSPEC void DECL2 srtpDump(char * name, uc*b, int len);

/*
    get bit length of an item in segment segSel:
*/
EXPORTSPEC int DECL2 srtpBitsPerItem(int segSel);

/*
    dump fields from an SRTP packet:
*/
EXPORTSPEC int srtpDecode(uc * b,int ki);

EXPORTSPEC int srtpConnectPLC(srtpConnection * sc);

//extern EXPORTSPEC int srtpDebug;

/*
    set and read debug level:
*/
EXPORTSPEC void DECL2 srtpSetDebug(int nDebug);
EXPORTSPEC int DECL2 srtpGetDebug(void);


#define srtpDebugOpen 0x1
#define srtpDebugPacket 0x2
#define srtpDebugPrintErrors 0x4
#define srtpDebugRaw 0x8
#define srtpDebugConnect 0x10
#define srtpDebugAll srtpDebugOpen|srtpDebugPacket|srtpDebugPrintErrors|srtpDebugRaw|srtpDebugConnect

/**
    Conversion routines. While such routines are not really necessary on Intel systems that use
    little endian equal to GE Fanuc PLCs, they come in handy on big endian systems and users are
    encouraged to use them in programs for portability.
    As the terms WORD, INT, INTEGER etc have different meanings
    for users of different programming languages and compilers, I choose to provide a new 
    set of conversion routines named according to the bit length of the value used. The 'U'
    or 'S' stands for unsigned or signed.
**/
EXPORTSPEC float DECL2 toPLCfloat(float ff);
EXPORTSPEC short DECL2 bswap_16(short ff);
EXPORTSPEC int DECL2 bswap_32(int ff);

/*
    Get a value from the position b points to. B is typically a pointer to a buffer that has
    been filled with srtpReadBytes:
*/
EXPORTSPEC int DECL2 srtpGetS8from(uc *b);
EXPORTSPEC int DECL2 srtpGetU8from(uc *b);
EXPORTSPEC int DECL2 srtpGetS16from(uc *b);
EXPORTSPEC int DECL2 srtpGetU16from(uc *b);
EXPORTSPEC int DECL2 srtpGetS32from(uc *b);
EXPORTSPEC unsigned int DECL2 srtpGetU32from(uc *b);
/*
    Get a value from the current position in the last result read on the connection dc.
    This will increment an internal pointer, so the next value is read from the position
    following this value.
*/
EXPORTSPEC int DECL2 srtpGetS8(srtpConnection * dc);
EXPORTSPEC int DECL2 srtpGetU8(srtpConnection * dc);
EXPORTSPEC int DECL2 srtpGetS16(srtpConnection * dc);
EXPORTSPEC int DECL2 srtpGetU16(srtpConnection * dc);
EXPORTSPEC int DECL2 srtpGetS32(srtpConnection * dc);
EXPORTSPEC unsigned int DECL2 srtpGetU32(srtpConnection * dc);
EXPORTSPEC float DECL2 srtpGetFloat(srtpConnection * dc);
/*
    Get a value from a given position in the last result read on the connection dc.
*/
EXPORTSPEC int DECL2 srtpGetS8at(srtpConnection * dc, int pos);
EXPORTSPEC int DECL2 srtpGetU8at(srtpConnection * dc, int pos);
EXPORTSPEC int DECL2 srtpGetS16at(srtpConnection * dc, int pos);
EXPORTSPEC int DECL2 srtpGetU16at(srtpConnection * dc, int pos);
EXPORTSPEC int DECL2 srtpGetS32at(srtpConnection * dc, int pos);
EXPORTSPEC unsigned int DECL2 srtpGetU32at(srtpConnection * dc, int pos);
/*
    put one byte into buffer b:
*/
EXPORTSPEC uc * DECL2 srtpPut8(uc *b,int v);
EXPORTSPEC uc * DECL2 srtpPut16(uc *b,int v);
EXPORTSPEC uc * DECL2 srtpPut32(uc *b,int v);
EXPORTSPEC uc * DECL2 srtpPutFloat(uc *b,float v);
EXPORTSPEC void DECL2 srtpPut8at(uc *b, int pos, int v);
EXPORTSPEC void DECL2 srtpPut16at(uc *b, int pos, int v);
EXPORTSPEC void DECL2 srtpPut32at(uc *b, int pos, int v);
EXPORTSPEC void DECL2 srtpPutFloatat(uc *b,int pos, float v);


#endif
