#include "cell.h"
#include "cellList.h"
#include <stdio.h>
#include "plclog.h"
#include "lib0.h"

#include "vi_string.h"

#include "refCell.h"

#define debug 4
#define ThisModule "refCell  : "

RefCell::RefCell()
{
    sheet=1;
#if debug > 0
    LOG1(ThisModule "this is RefCell::RefCell()\n");
    FLUSH;
#endif
    
}

void RefCell::setupFromString()
{
#if debug > 0
    LOG2(ThisModule "text %s\n",text.c_str());
    FLUSH;
#endif    
    int p=text.Find(',');
    cIndex=getPos(text.before(p)); 
#if debug > 0
    LOG2(ThisModule "column index: %s\n",cellN(cIndex));
    FLUSH;
#endif        
    rIndex=getPos(text.after(p)); 
#if debug > 0
    LOG2(ThisModule "row index: %s\n",cellN(rIndex));
    FLUSH;
#endif            
}

void RefCell::Read(Engine & e)
{
    tCell::Read(e);
#if debug > 0
    LOG2(ThisModule "text %s\n",text.c_str());
    FLUSH;
#endif    
    setupFromString();
}

IMPLEMENT_PERSISTENCE(RefCell, "RCELL")

coord RefCell::getTargetCoord() {
    coord c;
    cell * rind=sheets[sheet]->find(rIndex);
    if (rind==NULL) {
	c.row=1;
	c.col=1;
	return c;
    }	

    cell *  cind=sheets[sheet]->find(cIndex);
    if (cind==NULL) {
	c.row=1;
	c.col=1;
	return c;
    }	
#if debug > 0
    LOG3(ThisModule "indices: %s %s\n",cellN(cIndex),cellN(rIndex));
    FLUSH;
    LOG3(ThisModule "index values: %g %g\n",cind->getVal(), rind->getVal());
    FLUSH;
#endif
    c.col=(int)cind->getVal();
    c.row=(int)rind->getVal();
    return c;    
}

string RefCell::typeString() {
    return ::typeString(sheet, getTargetCoord() );
}

double RefCell::getVal() {
	cell * realCell=sheets[sheet]->find(getTargetCoord());
	if (realCell) 
	    return realCell->getVal(); 
	else 
	    return 0;
}

cell * RefCell::getRealCell() {
    return sheets[sheet]->find(getTargetCoord());
}

void RefCell::setVal(double d) {
    cell * realCell=getRealCell();
    if (realCell) realCell->setVal(d); 
}

string RefCell::getViewerText(int decimals, bool preferText)  {
    cell * realCell=getRealCell();
    if (realCell)
	return realCell->getViewerText(decimals, preferText);
    else 
	return stringContents(0,12);
}  

int RefCell::getError(){
    cell * realCell=getRealCell();
    if (realCell)
	return realCell->getError();
    else 
	return 0;
}

void RefCell::setError(int e){
    cell * realCell=getRealCell();
    if (realCell)
	return realCell->setError(e);
}

double RefCell::getPreVal(){
    cell * realCell=getRealCell();
    if (realCell)
	return realCell->getPreVal();
    else 
	return 0;
}

void RefCell::setPreVal(double d){
    cell * realCell=getRealCell();
    if (realCell)
	return realCell->setPreVal(d);
}

int RefCell::getSourceType() {
    cell * realCell=getRealCell();
    if (realCell)
	return realCell->getSourceType(); 
    else 
	return 0;
}

void RefCell::setSourceType(int s) {
    cell * realCell=getRealCell();
    if (realCell)
	realCell->setSourceType(s); 
}  

unsigned int RefCell::getCflags() {
    cell * realCell=getRealCell();
    if (realCell)
	return realCell->getCflags();
    else 
	return (0);
}  

void RefCell::setCflags(unsigned int c) {
    cell * realCell=getRealCell();
    if (realCell)
	return realCell->setFlags(c);
}  

unsigned int RefCell::getFlags() {
    cell * realCell=getRealCell();
    if (realCell)
	return realCell->getFlags();
    else 
	return 0;
}  

void RefCell::setFlags(unsigned int c) {
    cell * realCell=getRealCell();
    if (realCell)
	realCell->setFlags(c);
}  

unsigned int RefCell::getState() {
    cell * realCell=getRealCell();
    if (realCell)
	return realCell->getState();
    else 
	return 0;
}  

void RefCell::setState(unsigned int c) {
    cell * realCell=getRealCell();
    if (realCell)
	realCell->setState(c);
}  
