/*
 Definitions for a PLC driver for the SIEMENS S5 series.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#ifndef _s5_
#define _s5_
//
// some timeout values, maybe they could be less:
//
#define tmo_answ 138		// timeouts in milliseconds
#define tmo_read 138
#define tmo_read2 138
#define tmo_test 138
#define tmo_test2 138
//
// constants marking certain steps in executing driver functions with 
// the AS511 protocol:
//
#define SF_T1006               27
#define SF_T02                 1
#define SF_T161003             2
#define EF_T1003               3
#define EF_T02                 4
#define EF_T121003             5
#define GBI_SF                 6
#define GBI_T100602            7
#define GBI_ntrdch_Test02      8
#define GBI_ntrdch_Test121003  9
#define GBI_rdch_EF            10
#define read_SF               11
#define read_T100602          12
#define read_T00000           13
#define read_readCh           14
#define read_EndF             15
#define SVAR_SF                16
#define SVAR_T100602           17
#define SVAR_T121003           18
#define GSP_SF_1               19
#define GSP_T100602_1          20
#define GSP_ReadCh             21
#define GSP_EF_1               22
#define GSP_SF_2               23
#define GSP_T100602            24
#define GSP_TestCh             25
#define GSP_EndF_2             26
#define DBReadFault            28
#define DBNotPresent           29

#include "mytypes.h"

#pragma pack (1)
//
// These blocks represent PLC defined data structures.
// The PLC has no or a different idea of alignment...
//
struct sysPars {
    
/*
 contains informations about the CPU module provided by function 0x18.
*/
    unsigned char b1, b2, b3, b4, b5;	// meaning not known
    word Inputs;		// start address of PAE (image of inputs)
    word Outputs;		// start address of PAA (image of outputs)
    word Merker;		// start address of 'Merker' (flag words)
    word Zeiten;		// start address of timers
    word Zaehler;		// start address of counters
    word BSMem;			// start address of system memory
    word W1;			// meaning not known
    word UsrMemEnd;		// end address of application memory
    word W2_10[9];		// array[2..10] of  word // meaning not known
    unsigned char b6;
    				// meaning not known
     word DevInpBuf;		// meaning not known
    word BaustKLen;		// length of block header
    word W11;			// meaning not known
    unsigned char waste[12];	// meaning not known
};

/* 
    Information about a program/data block as provided by function 0x1A: 
*/ 
    struct blockInfo {
    unsigned char fix0;		// allways 0 ?
    word address;		// address in PLC memory
    unsigned char a, b;		// allways 'pp'
    unsigned char typ;		// block type
    unsigned char num;		// number of block
    unsigned char fix80;	// allwas 0x80
    int fix0s;			// allways 0
    unsigned char len;		// length of block in words
};

//
// this is in field "typ" for these types of blocks:
//
#define bt_DB 0x01
#define bt_FB 0x08
#define bt_OB 0x10
//
// operand types for S5:
//
#define opdAB 0			// output byte
#define opdEB 1			// input byte
#define opdMB 2			// flag byte
#define opdAW 3			// output word
#define opdEW 4			// input word
#define opdMW 5			// flag word
#define opdDW 6			// word in data block
#endif				/* _s5_ */
