/*
 A programm that will receive from a serial port and forward this to a
 TCP connection and vice versa.
 There are two possible applications for this:

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "vi_string.h"

#include "plclog.h"

#define ThisModule "serClient: "
#define uc unsigned char

#include <sys/time.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>


#include "setport.h"

//#define DONT_USE_GETHOSTBYNAME

#define bSize 256

#include "openSocket.cc"

char * cnettty="net->tty";
char * cttynet="tty->net";

int main(int argc, char **argv)
{
    bool dolog=false;
    int netFd, ttyRfd, ttyWfd,i;
    FILE * logFile;
    char * nettty, * ttynet;
    nettty = cnettty;
    ttynet = cttynet;

    if (argc<3) {
	printf("Usage: serialClient host:port 'tty baud parity bitlen stopbits' [logfile]\n");
	printf("Example: serialClient 192.168.17.110:11777 '/dey/ttyS0 38400 o 8 2' mylog\n");
	return -1;
    }
    if (argc>=3) {
	logFile = fopen(argv[3],"w+");
	if (logFile==NULL) {
	    printf("Could not open log file!\n");
	    return -1;
	}
	dolog=true;
	printf("Logging to %s\n",argv[3]);
	
	if (argc>=4) {
	    nettty=argv[4];
	    if (argc>=5) {
		ttynet=argv[5];
	    }
	}
    }	
    uc buffer[bSize];
    string peer = argv[1];
    string portN = peer.after(peer.find(':'));
    peer = peer.before(peer.find(':'));
    string serial = argv[2];
    int port = atol(portN.c_str());
    LOG3(ThisModule "host: %s port %d\n", peer.c_str(),port);
    ttyRfd=setPort(serial,&ttyWfd);
    if (ttyRfd<=0) {
	printf("Could not open serial port!\n");
	return -1;
    }
    netFd=openSocket(port, peer.c_str());
    if (netFd<=0) {
	printf("Could not connect to host!\n");
	return -1;
    }
    LOG2(ThisModule "netFd: %d\n", netFd);
    
    
    int maxFD=netFd;
    if (maxFD<ttyRfd) maxFD = ttyRfd;

    fd_set FDS;
    FD_ZERO(&FDS);
    FD_SET(netFd, &FDS);
    FD_SET(ttyRfd, &FDS);
    
    int rr, waitCount=0;
    while (waitCount < 1000) {
	FD_ZERO(&FDS);
	FD_SET(netFd, &FDS);
	FD_SET(ttyRfd, &FDS);
	timeval t;
	t.tv_sec = 0;
	t.tv_usec = 500000;
	int sr=select(maxFD + 1, &FDS, NULL, NULL, &t);
	if (sr > 0) {
//	    LOG2(ThisModule "after select %d\n",sr);
	    if FD_ISSET(ttyRfd, &FDS) {
		rr=read(ttyRfd, buffer, 256);
		if (rr>0) {
		    write(netFd, buffer, rr);
		    waitCount = 0;
    		    LOG2(ThisModule "portServer: tty read:%d\n", rr);
		    if (dolog) {
			fprintf(logFile,"%s",ttynet);
			for (i = 0; i < rr; i++) {
			    fprintf(logFile, "%x,", buffer[i]);
			}
			fprintf(logFile,"\n");
		    }	
		}
	    }
	    

	    if FD_ISSET(netFd, &FDS) {
		rr=read(netFd, buffer, 256);
		if (rr>0) {
		    write(ttyWfd, buffer, rr);
		    waitCount = 0;
		    LOG2(ThisModule "portServer: tty write:%d\n", rr);
		    if (dolog) {
			fprintf(logFile,"%s",nettty);
			for (i = 0; i < rr; i++) {
			    fprintf(logFile, "%x,", buffer[i]);
//			    if ((i % 16) == 15)
			}
			fprintf(logFile,"\n");
		    }
		}
	    }
	    waitCount++;
	} else {
	    waitCount++;
	    fflush(logFile);
	}    
    }
    LOG1(ThisModule "portServer: I closed my fd.\n");
    FLUSH;
    return 0;
}


/*
    Changes:
    
    05/12/2002	created
    12/22/2002	added sys/time.h
    07/12/2003	moved openSocket to it's own file, because it can be reused in other TCP clients
*/
