/*
 A programm that will open a serial port and couple it to a TCP connection.
 There are two possible applications for this:

 1. Temporarily suspend the connection to the PLC driver, then connect
    with a client that forwards serial port to TCP and use the TCP
    connection for remote programming/maintenance of PLC.
 2. Run VISUAL on a remote machine.
  
 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "plcdrv.h"

#include "vi_string.h"

#include <pthread.h>


#include "plclog.h"

#define ThisModule "serProxy : "
#define uc unsigned char

#include <sys/time.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>


#include "drivers/accepter.cc"
#include "setport.h"

#define bSize 256


struct portInfo {
    int fd;
    int ttyRfd;
    int ttyWfd;
};

char * cnettty="net->tty:";
char * cttynet="tty->net:";
FILE * logFile;
char * nettty, * ttynet;
bool dolog;

void *portServer(void *arg)
{
    int rr, i;
    uc buffer[bSize];
    portInfo * pi=(portInfo *) arg;
    LOG4(ThisModule "portServer: My fd is:%d ttyRfd %d ttyWfd %d\n", 
	pi->fd, pi->ttyRfd, pi->ttyWfd);
    FLUSH;
    int waitCount = 0;
    int maxFD=pi->fd;
    if (maxFD<pi->ttyRfd) maxFD = pi->ttyRfd;
    fd_set FDS;
    FD_ZERO(&FDS);
    FD_SET(pi->fd, &FDS);
    FD_SET(pi->ttyRfd, &FDS);
    timeval t;
    t.tv_sec = 0;
    t.tv_usec = 20000;
    while (waitCount < 10000) {
    
	FD_ZERO(&FDS);
	FD_SET(pi->fd, &FDS);
	FD_SET(pi->ttyRfd, &FDS);
	timeval t;
	t.tv_sec = 0;
	t.tv_usec = 500000;
	
	int sr=select(maxFD + 1, &FDS, NULL, NULL, &t);
	if (sr > 0) {
//	    LOG2(ThisModule "after select %d\n",sr);
	    if FD_ISSET(pi->ttyRfd, &FDS) {
		rr=read(pi->ttyRfd, buffer, 256);
		if (rr>0) {
		    write(pi->fd, buffer, rr);
		    waitCount = 0;
		    if (dolog) {
			fprintf(logFile,"%s",ttynet);
			for (i = 0; i < rr; i++) {
			    fprintf(logFile, "%x,", buffer[i]);
			}
			fprintf(logFile,"\n");
			fflush(logFile);
		    }
		}    
//		LOG2(ThisModule "portServer: tty read:%d\n", rr);
	    }
	    if FD_ISSET(pi->fd, &FDS) {
		rr=read(pi->fd, buffer, 256);
		if (rr>0) {
		    write(pi->ttyWfd, buffer, rr);
		    waitCount = 0;
		    if (dolog) {
			fprintf(logFile,"%s",nettty);
			for (i = 0; i < rr; i++) {
			    fprintf(logFile, "%x,", buffer[i]);
//			    if ((i % 16) == 15)
			}
			fprintf(logFile,"\n");
			fflush(logFile);
		    }
		}    
//		LOG2(ThisModule "portServer: tty write:%d\n", rr);
	    }
	    waitCount++;
	} else {
	    waitCount++;
	}    
    }
    LOG1(ThisModule "portServer: I closed my fd.\n");
    FLUSH;
    return NULL;
}

    
/*
    main() waits in select() for a file descriptor from accepter and starts a new child server
    with this file descriptor.
*/

int main(int argc, char **argv)
{
    nettty = cnettty;
    ttynet = cttynet;

    if (argc<2) {
	printf("Usage: serialProxy port 'tty baud parity bitlen stopbits' [logfile] [prefix1] [prefix2]\n");
	printf("prefix1 is put before each message send from ethernet to local port.\n");
	printf("prefix2 is put before each message send from local port to ethernet.\n");
	printf("Example: serialProxy 11777 '/dey/ttyS0 38400 o 8 2'\n");
	return -1;
    }
    if (argc>3) {
	logFile = fopen(argv[3],"w+");
	if (logFile==NULL) {
	    printf("Could not open log file!\n");
	    return -1;
	}
	dolog=true;
	printf("Logging to %s\n",argv[3]);
	if (argc>4) {
	    nettty=argv[4];
	    if (argc>5) {
		ttynet=argv[5];
	    }
	}
    }	

    int filedes[2], res, newfd;
    string s = argv[1];
    LOG2(ThisModule "Main serv: %s\n", s.c_str());
    string s2 = s.after(s.Find('='));
    LOG2(ThisModule "Main serv: %s\n", s2.c_str());
    
    portInfo pi;
    string s3 = argv[2];
    pi.ttyRfd=setPort(s3,&pi.ttyWfd);

    fd_set FDS;
    pipe(filedes);
    pthread_attr_t attr;
    pthread_t ac, ps;
    accepter_info ai;
    ai.port = atol(s2.c_str());
    LOG2(ThisModule "Main serv: %d\n", ai.port);
    LOG2(ThisModule "Main serv: Accepter pipe fd: %d\n", ai.fd);
    ai.fd = filedes[1];
//    ai.port=182;
    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr,PTHREAD_CREATE_DETACHED);
    res=pthread_create(&ac, &attr, accepter, &ai /*&filedes[1] */ );
    do {
	FD_ZERO(&FDS);
	FD_SET(filedes[0], &FDS);

	LOG2(ThisModule "Main serv: about to select on %d\n",
	       filedes[0]);
	FLUSH;
	if (select(filedes[0] + 1, &FDS, NULL, &FDS, NULL) > 0) {
	    LOG1(ThisModule "Main serv: about to read\n");
	    res = read(filedes[0], &pi.fd, sizeof(pi.fd));
	    LOG3(ThisModule
		   "Main serv: about to create new port server for fd:%d read res:%d\n", pi.fd,res);
	    ps=0;		   
	    pthread_attr_init(&attr);
	    pthread_attr_setdetachstate(&attr,PTHREAD_CREATE_DETACHED);
	    res=pthread_create(&ps, &attr, portServer, &pi);
	    LOG3(ThisModule
		   "Main serv: create res:%d %ld\n", res, ps);
	    if(res) {
		LOG2(ThisModule
		   "Main serv: create error:%s\n", strerror(res));		   
		close(newfd);
		usleep(100000);
	    }	   
	}
    }
    while (1);
    return 0;
}


/*
    Changes:
    
    05/12/2002 created
    23/12/2002 added sys/time.h
    04/14/2004 give a hint about usage
*/
