/*
 Saves a spreadsheet. Might be modified to save in the right directories.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#include "cellList.h"
#include <iostream.h>
#include <fstream.h>
#include "list.h"
#include "persist.h"
#include "lib0.h"
#include "globals.h"
#include "plclog.h"
#include "stdio.h"

#define ThisModule "shsaver  : "
#define debug 10

void saveSheet(int shn)
{
#if debug >0
    LOG1(ThisModule"enter save\n");
    FLUSH;
#endif
    if ((sheets[shn]) != 0) {
	string s = projectPath + (digits(shn) + ".sht");
#if debug >0
	LOG2(ThisModule"save %s\n",s.c_str());
	FLUSH;
#endif
	fstream f(s.c_str(), ios::out);

	Engine *e = new Engine(f);
//      sheets[shn]->forEach(putit22, e);
	cellList *cl = sheets[shn];
	*e << cl;
	delete e;
	f.close();
    }
    sheets[shn]->changed = false;
}

void saveSettings(int shn)
{
    nCell *c;

    if ((sheets[shn]) != 0) {
	c = (nCell *) ((sheets[shn])->numList);
	string s = projectPath + (digits(shn) + ".set");
	fstream f2(s.c_str(), ios::out);
	Engine *e = new Engine(f2);
	while (c != NULL) {
	    if (c->getSourceType() == soSetByHMI) {
		*e << c;
	    }
	    c = c->nextNum;
	}
	delete e;
	f2.close();
    }
}

void putMarked(void *a, void *x)
{
    if (((cell *) a)->getFlags() & flMarked) {
	*((Engine *) x) << ((base *) a);
    }
}

void saveMarked(int shn)
{
#if debug >0
    LOG1(ThisModule"enter saveMarked\n");
    FLUSH;
#endif
    if ((sheets[shn]) != 0) {
	string s = projectPath + (digits(shn) + ".mkd");
#if debug >0
	LOG2(ThisModule"save %s\n",s.c_str());
	FLUSH;
#endif
	fstream f(s.c_str(), ios::out);

	Engine *e = new Engine(f);
	sheets[shn]->forEach(putMarked, e);
//      cellList *cl = sheets[shn];
//      *e<<cl;
	delete e;
	f.close();
    }
}

/*
    Changes:
    
    05/13/2002  Sheet is now loaded using persistence mechanism.
    04/18/2003  Use logging from log.h
    09/20/2003  removed #include plcdrv.h, it's not used
*/
