/*
 Implementation of a spreadsheet viewer/editor based on TURBOVISION 
 library. TURBOVISION is still copyrighted by Borland, but you get it
 for free with SuSE Linux.

 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#define Uses_TView
#define Uses_TWindow
#define Uses_TDeskTop
#define Uses_TDialog
#define Uses_TCheckBoxes
#define Uses_TInputLine
#define Uses_TSItem
#define Uses_TButton
#define Uses_TLabel
#define Uses_Program
#define Uses_TKeys
#define Uses_TEvent
#define Uses_EventCodes
#define Uses_TApplication
#define Uses_MsgBox
#include <tvision/tv.h>
#include "shview.h"
#include "cell.h"
#include "lib0.h"
#include "cellList.h"
#include "helpers.h"
#include "shcommands.h"
#include "colorattr.h"
#include <stdio.h>
#include "tvtoplus.h"
#include "ediCellList.h"
#include "hcontext.h"
#include "reflist.h"
#include "shsaver.h"
#include "globals.h"
#include "lang.h"

#include "vi_string.h"

#include "plclog.h"

#define opShTPN 1		// show TPN representation of formulae
#define opShForm 2		// show formulae instead of results
#define opShFlags 4 		// show flags
#define opShState 8 		// show state Info
#define opShHex   0x10		// display numbers as hex 
#define opShSource 0x20		// shom detailed value source information 
#define opShNoShift 0x40	// do NOT shift coords when copying cells
#define opShCflags 0x80 	// show cflags

inline bool confirm(int n)
{
    return (messageBox
	    (sc(n).c_str(),
	     mfConfirmation + mfYesButton + mfNoButton) == cmYes  );
}

inline void errMb(int n)
{
    messageBox(sc(n).c_str(), mfError + mfOKButton);
}

void cellInputBox(int n, coord * c) {
    char s2[256];
    string(s) = cellName(*c);
    strncpy(s2, s.c_str(), 256);
    if (inputBox(" ", " ", s2, 255) == cmOK) {
	*c = getPos(string(s2));
    }
}

CellDetailView::CellDetailView(const TRect & r):TView(r) {
}

void writeFlags(int flags, TDrawBuffer & buf) {
    buf.moveChar(39, '-', yellowOnBlack, 32);
    int mask = 0x00000001;
    for (int i = 0; i < 32; i++) {
	if (flags & mask)
	    buf.moveChar(39 + i, '*', yellowOnBlack, 1);
	 mask <<= 1;
    }
}

void writeState(int flags, TDrawBuffer & buf) {
    //const char 
    const char *states = "X**D0123456789ABCDEFGHIJKLMNOPQR";
    buf.moveChar(39, '-', yellowOnBlack, 32);
    int mask = 0x00000001;
    for (int i = 0; i < 32; i++) {
	if (flags & mask)
	    buf.moveChar(39 + i, states[i], yellowOnBlack, 1);
	 mask <<= 1;
    }
}

#define contPos 8

void CellDetailView::draw() {
    cell *z;
    coord k;
    string sw, sv;
    char sw1[64];
    TDrawBuffer2 buf;
    ShWin * own = ((ShWin *) owner);
    buf.moveChar(0, ' ', getColor(1), size.x);
    k = own->ccPos();
    z = sheets[own->shNumber]->find(k);
    if (z != NULL) {
	buf.moveString(0, sheets[own->shNumber]->symbolName(k, symOnly).substr(0,130), 0); //TDrawBuffer is 132 wide
//	buf.moveString(40, sheets[own->shNumber]->getSource(k), 0);
	if (own->shOptions & opShFlags) {
	    writeFlags(z->getFlags(), buf);
	}
	if (own->shOptions & opShCflags) {
	    writeFlags(z->getCflags(), buf);
	}
	if (own->shOptions & opShState) {
	    writeState(z->getState(), buf);
	}
	writeBuf(0, 2, size.x, 1, buf);
	buf.moveChar(0, ' ', getColor(1), size.x);
    }
    buf.moveString(0, cellName(k), 0);
    if (z == NULL) {
	buf.moveString(contPos, sc(scShLeer), 0);
	writeBuf(0, 0, size.x, 1, buf);
	buf.moveChar(0, ' ', getColor(1), size.x);
	
/*	
	If ShOptions and opShSource.........
*/
	writeBuf(0, 1, size.x, 1, buf);
	writeBuf(0, 2, size.x, 1, buf);
    } else {
	if (z->typeID() == ttCell) {
	    buf.moveString(contPos, sc(scShText), 0);
	    buf.moveString(contPos+sc(scShText).length()+1, ((tCell *) (z))->text, 0);
	    writeBuf(0, 0, size.x, 1, buf);
	    buf.moveChar(0, ' ', getColor(1), size.x);
	    writeBuf(0, 1, size.x, 1, buf);
	} else {
	    if (z->typeID() == tnAvrCell) {
		sprintf(sw1, "%g", z->getVal());
		buf.moveString(contPos, sc(scShZahl), 0);
		buf.moveStr(contPos+sc(scShZahl).length()+1, sw1, 0);
		writeBuf(0, 0, size.x, 1, buf);
		buf.moveChar(0, ' ', getColor(1), size.x);
		sprintf(sw1, "%8.3f", z->getPreVal());
/*		
		buf.moveStr(0, sc(scShAvr)+cellName(source)+' '+sw1+'/'+Ziffern(count)
	        +'  '+Ziffern(AvrTime)+sc(scShsec)+Ziffern(AvrClock mod AvrTime),0);
*/
		writeBuf(0, 1, size.x, 1, buf);
		buf.moveChar(0, ' ', getColor(1), size.x);
	    } else {
		if ((z->typeID() == tnCell) || (z->typeID() == tnTimeCell)) {
		    sprintf(sw1, "%g", z->getVal());
		    buf.moveString(contPos, sc(scShZahl), 0);
		    buf.moveString(contPos+sc(scShZahl).length()+1, sw1, 0);
		    
			//buf.moveString(37,formatDouble(z->value,0,own->Decimals),0);
		    writeBuf(0, 0, size.x, 1, buf);
		    buf.moveChar(0, ' ', getColor(1), size.x);
		    buf.moveString(0, sc(scOfSourcet +z->getSourceType()), 0);
		    buf.moveString(64, sheets[own->shNumber]->getSource(k), 0);
		    if (z->getSourceType() == soIoState) {
			buf.moveString(45, ioErrorText(int (z->getVal())), 0);
		    }
		    if (z->typeID() == tnTimeCell) {
			buf.moveString(20, (sc(scShLastChange)
					    +
					    extDateTimeString(((nTimeCell
								*) z)->
							      time)), 0);
		    }
		    /*
		       if ((own->shOptions & opShSource)!= 0) {
		       buf.moveStr(contPos,((ediCellList*)(sheets[own->shNumber)->
		       SourceName(c),0);
		       }
		     */
		    writeBuf(0, 1, size.x, 1, buf);
		    buf.moveChar(0, ' ', getColor(1), size.x);
		} else {
		    if (z->isFormula()) {
			buf.moveString(contPos, sc(scShForm), 0);
			buf.moveString(contPos+sc(scShForm).length()+1, ((fCell *) z)->text, 0);
			writeBuf(0, 0, size.x, 1, buf);
			buf.moveChar(0, ' ', getColor(1), size.x);
			if (z->getError() != ceNone) {
			    buf.moveString(0, sc(scOfMath +
					    z->getError()),
					    0);
			}
			/*      
			   if ((own->shOptions & opShSource) != 0).....
			*/
			if (z->typeID() == tfTimeCell) {
			    buf.moveString(20, sc(scShLastChange)+
				extDateTimeString(((fTimeCell *)
				z)->time),
				0);
			}
			writeBuf(0, 1, size.x, 1, buf);
			buf.moveChar(0, ' ', getColor(1), size.x);
			if ((own->shOptions & opShTPN) != 0) {
			    buf.moveString(0,
					    listTPN(((fCell *) z)->TPN,
						    own->Decimals), 0);
			    writeBuf(0, 1, size.x, 1, buf);
			}
		    }
		}
		}
	    }
	//writeBuf(0,2,size.x,1,buf);
    }
    
}

int ShWin::curCol() {
    return (CurX + Xoff);
}

int ShWin::curRow() {
    return (CurY + Yoff);
}

coord ShWin::ccPos() {
    coord c;
    c.col = curCol();
    c.row = curRow();
    return (c);
}

ShWin::ShWin():TWindow(TProgram::deskTop->getExtent(), sc(scShTitle).c_str(),
	wnNoNumber), TWindowInit(&ShWin::initFrame)
{
    TRect R;
    CellDetailView *c;
    ShView * v;
//    DispX2 = 10;		// size of spreadsheet display window in columns
//    DispY2 = 16;
    			// size of spreadsheet display window in rows
    Decimals = 2;
    			// default numer of decimals to display
    ColWidth = 7;
    			// default column width
    CurX = 1;		// cursor row and column
    CurY = 1;
    Xoff = 0;
    Yoff = 0;
    shNumber = 1;		// default sheet to display
    shOptions = 0;		// controls display of additional details
    AreaValid = false;		// Area is not valid in a newly created window
    helpCtx = hcShViewer;
    palette = wpGrayWindow;
    options = options & (~ofBuffered);	// we must redraw the sheet when something changes
    R = getExtent();
    R.grow(-1, -1);
    R.a.y = R.b.y - 3;
    c = new CellDetailView  (R);
    c->growMode = gfGrowLoY | gfGrowHiX | gfGrowHiY;
    insert(c);
//    R = TRect(1, 1, 79, 19);
    R = getExtent();
    R.grow(-1, -1);
    R.b.y = R.b.y - 3;
    DispX2 = (R.b.x / ColWidth)-1;	// size of spreadsheet display window in columns
    DispY2 = (R.b.y )-3;		// size of spreadsheet display window in columns
    				// this is not ok when there are more then 25 display
				// lines!!
    if (sheets[1]==NULL) {		
	sheets[1] = new cellList();
    } 
    v = new ShView(R);
    v->growMode = gfGrowHiY | gfGrowHiX;
    insert(v);
}


void ShWin::moveAbs(int x, int y) {
    moveSheet(x - curCol(), y - curRow());
}

void ShWin::curMove(TEvent & event) {
    int mine;
    TPoint MP;
    mine = 1;
    if (event.what == evMouseDown) {
	MP = makeLocal(event.mouse.where);
	if (MP.y > 2) {
	    CurX = (MP.x - 1) / ColWidth;
	    CurY = MP.y - 2;
	    message(owner, evBroadcast, cmshUpdtSheet, NULL);
	}
    }
    switch (event.keyDown.keyCode) {
    case kbHome:
	moveAbs(1, 1);
	break;
    case kbUp:
	moveSheet(0, -1);
	break;
    case kbDown:
	moveSheet(0, 1);
	break;
    case kbRight:
	moveSheet(1, 0);
	break;
    case kbLeft:
	moveSheet(-1, 0);
	break;
    case kbCtrlRight:
	moveSheet(DispX2, 0);
	break;
    case kbCtrlLeft:
	moveSheet(-DispX2, 0);
	break;
    case kbPgUp:
	moveSheet(0, -DispY2);
	break;
    case kbPgDn:
	moveSheet(0, DispY2);
	break;
    case kbCtrlPgUp:
	moveSheet(0, -10 * DispY2);
	break;
    case kbCtrlPgDn:
	moveSheet(0, 10 * DispY2);
	break;
    default:
	mine = 0;
    }				// switch
    if (mine != 0) {
	event.what = evNothing;
    }
}

void ShWin::moveSheet(int x, int y) {
    CurX += x;
    CurY += y;
    if (CurX > DispX2) {
	Xoff = curCol() - DispX2;
	CurX = DispX2;
    }
    if (CurX < 1) {
	Xoff = curCol() - 1;
	CurX = 1;
    }
    if (CurY > DispY2) {
	Yoff = curRow() - DispY2;
	CurY = DispY2;
    }
    if (CurY < 1) {
	Yoff = curRow() - 1;
	CurY = 1;
    }
    if (Yoff < 0) {
	Yoff = 0;
    }
    if (Xoff < 0) {
	Xoff = 0;
    }
    if (Yoff > 10000) {
	Yoff = 10000;
    }
    if (Xoff > 10000) {
	Xoff = 10000;
    }
    message(owner /*deskTop */ , evBroadcast, cmshUpdtSheet, NULL);
}


void ShWin::editCell() {
    coord c;
    string s, tempLabel;
    int tempFlags, tempCFlags;
    char s2[256];
    bool ok;
    c = ccPos();
    //    cout<<"about to call stringContents("<<shNumber<<cellName(c)<<10<<12<<")"<<endl;
	s = stringContents(shNumber, c, 10, 12);
    
//    cout<<"stringContents returned:"<<s<<endl;
//
// if a cell has a label, we must preserve and reinsert it, because it
// will be deleted in a deleteCell .
//
    tempLabel = sheets[shNumber]->symbolName(c, symOnly);
    tempFlags = sheets[shNumber]->getFlags(c, ok);
    tempCFlags = sheets[shNumber]->getCFlags(c, ok);
    strncpy(s2, s.c_str(), 256);
    if (inputBox(" ", " ", s2, 255) == cmOK) {
	s = s2;
	((ediCellList *) sheets[shNumber])->deleteCell(c);
	if (!s.empty()) {
	    ((ediCellList *) sheets[shNumber])->createCell(c, s, soDirect);
	}
//
// restore label and cFlags:
//    
//      if (!tempLabel.empty()) {
	((ediCellList *) sheets[shNumber])->insertLabelAndCFlags(c, tempLabel, tempCFlags);
//      }
    }
}

void ShWin::editLabel() {
    coord c;
    string s;
    char s2[256];
    c = ccPos();
    if (sheets[shNumber]->find(c)) {	// cannot attach labels to empty cells
	s = sheets[shNumber]->symbolName(c, symOnly);
	strncpy(s2, s.c_str(), 256);
	if (inputBox(" ", " ", s2, 255) == cmOK) {
	    s = s2;
	    ((ediCellList *) sheets[shNumber])->insertLabel(c, s);
	}
    }
}

struct candt {
    int type;
    coord current;
    coord best;
};

void isOftype(void *p, void *p1) {
    cell *c = (cell *) p;
    candt *d = (candt *) p1;
    if ((c->k.col >= d->current.col) &&	// the cell is right from or in cursor column
	(c->k.col <= d->best.col) && 	// the cell is nearer to cursor position than former best best
	(c->k.row >= d->current.row) &&	// the cell is down from or in cursor row
	(c->k.row <= d->best.row) &&	// the cell is nearer to cursor position than former best best
	(c->k.l != d->current.l) && 	// the cell is NOT at cursor position
	((d->type == 0) ||	// either no type required..
	 (c->typeID() == d->type))	// ..or cell has required type
	)
	{
	    d->best = c->k;
	}
}

void ShWin::findCellOfType(int type) {
    candt ct;
    ct.current.col = curCol();
    ct.current.row = curRow();
    ct.best.col = 11000;
    ct.best.row = 11000;
    ct.type = type;
    sheets[shNumber]->forEach(isOftype, &ct);
    moveAbs(ct.best.col, ct.best.row);
}

void optionsDialog(ShWin * S) {
#pragma pack(1)
    struct {
	word opt;
	char x[5];
	char xx[5];
    } data;

    TDialog *D = LangOptionsDialog();
    data.opt = S->shOptions;
    sprintf(&data.x[0], "%d", S->ColWidth);
    sprintf(&data.xx[0], "%d", S->Decimals);
    TProgram::application->executeDialog(D, &data);
    S->shOptions = data.opt;
    S->ColWidth = atol(data.x);
    S->Decimals = atol(data.xx);
}



void ShWin::handleEvent(TEvent & event) {
    coord c;
    int x;
//    cout<<"this is ShWin::handleEvent()"<<endl;
    if ((event.what & evMessage) != 0) {
	if (event.message.command == cmshUpdtSheet) {
	    drawView();
	}
	if (event.message.command == cmshExistsView) {
	    clearEvent(event);
	}
    }
    curMove(event);
    if (event.keyDown.charScan.charCode == 0x0d) {
	if (markArea == 1) {
	    AreaS = ccPos();
	    markArea++;
	} else {
	    if (markArea == 2) {
		AreaE = ccPos();
		markArea = 0;
		AreaValid = true;
		if (AreaS.row > AreaE.row) {
		    x = AreaE.row;
		    AreaE.row = AreaS.row;
		    AreaS.row = x;
		}
		if (AreaS.col > AreaE.col) {
		    x = AreaE.col;
		    AreaE.col = AreaS.col;
		    AreaS.col = x;
		}
	    }
	}
    }
    switch (event.message.command) {
    case cmshSave:
	saveSheet(shNumber);
	break;
    case cmshEditCell:
	editCell();
	break;
    case cmshEditLabel:
	editLabel();
	break;
    case cmshGoto:
	c = ccPos();
	cellInputBox(scShGoto, &c);
	moveAbs(c.col, c.row);
	break;
    case cmshNext:
	x = shNumber + 1;
	if (x > maxSheet)
	    x = maxSheet;
	if (sheets[x] == NULL) {
	    if (confirm(scShAskCreate))
		sheets[x] = new cellList();
	    else
		x--;
	}
	shNumber = x;
	break;

    case cmshPrev:
	shNumber--;
	if (shNumber < 1)
	    shNumber = 1;
	break;

    case cmshOptions:
	optionsDialog(this);
	/*
	   if (sheets[shNumber]^.Sources)
	   delete sheets[shNumber]->Sources;
	   sheets[shNumber]^.Sources=NULL;
	 */
	break;

    case cmshDelete:
	c = ccPos();
	((ediCellList *) sheets[shNumber])->deleteCell(c);
	break;
    case cmshFindAny:
	findCellOfType(0);
	break;
    case cmshFindNum:
	findCellOfType(tnCell);
	break;
    case cmshFindFor:
	findCellOfType(tfCell);
	break;
    case cmshFindText:
	findCellOfType(ttCell);
	break;
/*
    case cmshFindHMICell:
	if (sheets[glHMIsheet] != NULL) {
	    shNumber = glHMIsheet;
	    moveAbs(glHMImarked.col, glHMImarked.row);
	} else {
	    messageBox((sc(scShNoSheet) + digits(glHMIsheet)).c_str(),
		       mfError + mfOKButton);
	}
	break;
*/	
    case cmshArea:
	markArea = 1;		//mark1st;  
	break;

    case cmshMark:
	c = ccPos();
	((ediCellList *) sheets[shNumber])->mark(c, markToggle);
	message(owner, evBroadcast, cmshListChanged, NULL);
	break;

    case cmshMarkArea:
	if (!AreaValid)
	    errMb(scShNoArea);
	else
	     {
	    ((ediCellList *) sheets[shNumber])->
		withArea(srMark, AreaS, AreaE, "CS5", AreaValid);
	    message(owner, evBroadcast, cmshListChanged, NULL);
	    }
	break;
    case cmshDelArea:
	if (!AreaValid)
	    errMb(scShNoArea);
	else {
	    if (confirm(scShDelArea)) {
		((ediCellList *) sheets[shNumber])->
		    withArea(srDelete, AreaS, AreaE, "CS5", AreaValid);
	    }
	}
	break;
    case cmshCopyArea:
	if (!AreaValid)
	    errMb(scShNoArea);
	else
	    ((ediCellList *) sheets[shNumber])->
		copyCells(AreaS, AreaE, ccPos(), (shOptions & opShNoShift)==0 );
	break;
    case cmshMarkRef:
	c = ccPos();
	list * l = new list();
	reflist((sheets[shNumber])->find(c), l);
	for (x = 0; x < l->count; x++) {
	    ((ediCellList *) sheets[shNumber])->
		mark(*((coord *) l->items[x]), true);
	}
	message(owner, evBroadcast, cmshListChanged, NULL);
	break;
    }
    TWindow::handleEvent(event);
}

ShView::ShView(const TRect & r):TView(r)
{
}

void ShView::draw()
{				// displays sheet contents

    TDrawBuffer2 buf;
    coord dc;			// coordinate to displays
    cell *c;
    bool marked;
    int cursorColor = whiteOnGreen, color2;
    ShWin * own = ((ShWin *) owner);	// promised: owner is allways a ShWin
    if (own->markArea == 1) {
	cursorColor = whiteOnRed;
    }
    if (own->markArea == 2) {
	cursorColor = yellowOnBlack;
    }

    buf.moveChar(0, ' ', whiteOnBlue, size.x);
    if (own->shOptions & opShForm) {
	buf.moveString(0, sc(scShFOR), 0) ;
    } else {
	buf.moveString(0, sc(scShRES), 0);
    }
    if (sheets[own->shNumber]->changed) {
	buf.moveString(11, sc(scShSave), 0);
    }
//  buf.moveChar (0, ' ', whiteOnBlue, size.x);

    if (own->AreaValid) {
	buf.moveString(30,	/*sc(scShArea) */
			cellName(own->AreaS) + "..." +
			cellName(own->AreaE), 0);
    }
    buf.moveString(69, formatDouble(1e-4 * glCPULoad, 0, 2) + " %CPU", 0);
    writeBuf(0, 0, size.x, 1, buf);

    buf.moveChar(0, ' ', whiteOnRed, size.x);
    buf.moveString(0, sc(scShPageNumber) + digits(own->shNumber), 0);
    int zz = 5 + own->ColWidth / 2;
    for (int ic = 1; ic <= own->DispX2; ic++) {
	buf.moveString(zz + (ic - 1) * own->ColWidth,
		       colCode(ic + own->Xoff) + "    ", 0);
    }
    writeBuf(0, 1, size.x, 1, buf);
    buf.moveChar(0, ' ', whiteOnRed, own->ColWidth);
    for (int ir = 1; ir <= own->DispY2; ir++) {
	dc.row = ir + own->Yoff;
	buf.moveString(0, digits(dc.row), 0);
	buf.moveChar(5, ' ', whiteOnBlue, size.x);
	for (int ic = 1; ic <= own->DispX2; ic++) {

	    dc.col = ic + own->Xoff;
	    c = sheets[own->shNumber]->find(dc);
	    if (c != NULL) {
		if (c->getFlags() & flMarked) {
		    color2 = yellowOnBlue;
		    marked = true;
		    for (int i = 0; i < own->ColWidth; i++) {
			buf.putAttribute(5 + (ic - 1) * own->ColWidth + i,
					  yellowOnBlue);
		    }
		}
	    }

	    if ((ic == own->CurX) && (ir == own->CurY)) {
		for (int i = 0; i < own->ColWidth; i++) {
		    buf.putAttribute(5 + (ic - 1) * own->ColWidth + i,
				      cursorColor);
		}
	    }

	    if (c != NULL) {
		buf.moveString(
		    5 + (ic - 1) * own->ColWidth,
		    c->getViewerText(own->Decimals,((((ShWin *) own)->shOptions & opShForm) != 0)),
		    0);
	    }

	}
	writeBuf(0, 1 + ir, size.x, 1, buf);
    }
    
}

Boolean ShWin::valid(ushort command)
{
    Boolean v;
    v = TWindow::valid(command);
    if ((command == cmClose) && (sheets[shNumber]->changed)) {
	if (confirm(scShClose))
	    v = True ;
	else
	    v = False;
    }
    return (v);
}

ShWin::~ShWin()
{
    LOG1("~ShWin\n");
    FLUSH;    
//  ~TWindow();
	if (message(owner, evBroadcast, cmshExistsView, NULL) == NULL) {
	//deskTop->disableCommands( sheetCmndSet );
    }
    LOG1("~ShWin ends.\n");
    FLUSH;    
}

/*
    Changes:
    
    06/25/2002  Set XOff,YOff to 0 in ShWin() constructor. This
		makes the new window come up with cursor in A1.
    10/15/2002  Now as it runs in xterm, adapt size to screen size.
    04/14/2003  Create a first sheet on wndow open, if there is not yet one.
    07/05/2003  Now a data source location can be shown, if information provided by PLC driver
    08/04/2003  Fixed? buffer overflow from long labels.
*/
