/*
 The VISUAL main program with TURBOVISION based viewer.
 
 Part of VISUAL, a human machine interface and data acquisition program
 
 (C) Thomas Hergenhahn (thomas.hergenhahn@web.de) 2001.

 VISUAL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 VISUAL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Visual; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
*/
#define Uses_TView
#define Uses_TRect
#define Uses_TStatusLine
#define Uses_TStatusDef
#define Uses_TStatusItem
#define Uses_TKeys
#define Uses_MsgBox
#ifdef use_tv_help
#define Uses_fpstream
#endif
#define Uses_TEvent
#define Uses_TDeskTop
#define Uses_TWindow
#define Uses_TDialog
#define Uses_TStaticText
#define Uses_TButton
#define Uses_TWindow
#define Uses_TMenuBar
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuItem
#define Uses_TApplication
#define Uses_TDialog
#define Uses_TEventQueue

#include "tvision/tv.h"
#include "visual.h"


#include "help.h"

#ifdef use_tv_help
#include "tvision/help.h"
#endif

#include <stdio.h>
#include <unistd.h>

//#include "vi_string.h"

#include <pthread.h>

#include "shview.h"
#include "calcthread.h"
#include "loader2.h"
#include "hcontext.h"
#include "shcommands.h"
#include "viewcommands.h"
#include "viewers.h"
#include "lang.h"
#include "plclog.h"

#include <sys/types.h>
#include <signal.h>

#include <plcdrivers/PLCdriver.h>

#ifdef use_tv_help
#define HELP_FILENAME "DEMOHELP.H32"
#endif

#define debug 5

//#define ThisModule "visual   : "

int main(int argc, char **argv)
{
    time_t t;
    int res;
    LOG2("%d arguments.\n", argc);
    if (argc>1) load2(argv[1]); else load2("visual.cfg");
    time(&t);
    LOG2("%s", ctime(&t));
    LOG1("before creating calculation thread.\n");
    FLUSH;
    pthread_t calculatorHandle;

    res = pthread_create(&calculatorHandle, NULL, calcThread, NULL);
    time(&t);
    LOG3("created calculation thread: %d %s", res, ctime(&t));
    FLUSH;

    VisualApp *thisApp = new VisualApp(argc, argv);

    thisApp->run();
    LOG2("after run %s", ctime(&t));
    
    for (int i = 0; i < maxDrivers-2; i++) {
        if (registrations[i].mask != 0) {
	    if (registrations[i].pointerToInstance) {
	        PLCdriver *p =
		    ((PLCdriver *) registrations[i].pointerToInstance);
		p->doRun=false;
	    }
        }
    }
    
    bool aDriverIsStillRunning;
    int repeatCount=0;
    do{
	aDriverIsStillRunning=false;
	usleep(20000);
	for (int i = 0; i < maxDrivers-2; i++) {
    	    if (registrations[i].mask != 0) {
		if (registrations[i].pointerToInstance) {
	    	    PLCdriver *p =
			((PLCdriver *) registrations[i].pointerToInstance);
		    if (p->isRunning) {
			aDriverIsStillRunning=true;
		    }
		}
    	    }
	}    
	repeatCount++;
    } while ((aDriverIsStillRunning) && (repeatCount<150));
    LOG2("after stopping PLC drivers %s", ctime(&t));
    TObject::destroy(thisApp);
    LOG2("after destroy %s", ctime(&t));
/*
    This has never been necessary before, but now program hangs here. 10/16/02
*/
    kill(-getpid(),9);
    return 0;
}


//
// Constructor for the application. 
//

VisualApp::VisualApp(int argc, char **argv):
TProgInit(&VisualApp::initStatusLine,
	  &VisualApp::initMenuBar, &VisualApp::initDeskTop)
{
//    TView *w;
    char fileSpec[128];
    int len;
/*
  TRect r = getExtent ();	// Create the clock view.
  r.a.x = r.b.x - 9;
  r.b.y = r.a.y + 1;
  clock = new TClockView (r);
  insert (clock);
*/

    while (--argc > 0)		// Display files specified
    {				//  on command line.
	strcpy(fileSpec, *++argv);
	len = strlen(fileSpec);
    }
    openSheet();
}


//
//  VisualApp::getEvent()
//  Event loop to check for context help request
//

void
 VisualApp::getEvent(TEvent & event)
{
#ifdef use_tv_help    
    TWindow *w;
    THelpFile *hFile;
    fpstream *helpStrm;
    static Boolean helpInUse = False;
#endif    

    TApplication::getEvent(event);
    switch (event.what) {
    case evCommand:
#ifdef use_tv_help    
	if ((event.message.command == cmHelp) && (helpInUse == False)) {
	    helpInUse = True;
	    helpStrm = new fpstream(HELP_FILENAME, ios::in | ios::bin);
	    hFile = new THelpFile(*helpStrm);
	    if (!helpStrm) {
		messageBox("Could not open help file",
			   mfError | mfOKButton);
		delete hFile;
	    } else {
		w = new THelpWindow(hFile, getHelpCtx());
		if (validView(w) != 0) {
		    execView(w);
		    destroy(w);
		}
		clearEvent(event);
	    }
	    helpInUse = False;
	}
#endif	
	break;
    case evMouseDown:
	if (event.mouse.buttons != 1)
	    event.what = evNothing;
	break;
    }

}

//
// Create statusline.
//

TStatusLine *VisualApp::initStatusLine(TRect r)
{
    r.a.y = r.b.y - 1;
    if (LangStatus)
	return (LangStatus(r));
    return (new TStatusLine(r,
			    *new TStatusDef(0, 50) +
			    *new TStatusItem("~F1~ Help", kbF1, cmHelp) +
			    *new TStatusItem("~Alt-X~ Exit", kbAltX,
					     cmQuit) + *new TStatusItem(0,
									kbAltF3,
									cmClose)
			    + *new TStatusItem(0, kbF10,
					       cmMenu) +
			    *new TStatusItem(0, kbF5,
					     cmZoom) + *new TStatusItem(0,
									kbCtrlF5,
									cmResize)
			    + *new TStatusDef(hcShViewer,
					      hcShViewer) +
			    *new TStatusItem("~F1~ Help", kbF1,
					     cmHelp) +
			    *new TStatusItem("~F2~ Save", kbF2,
					     cmshSave) +
			    *new TStatusItem("~Alt-E~ Edit", kbAltE,
					     cmshEditCell) +
			    *new TStatusItem("~Alt-L~ Label", kbAltL,
					     cmshEditLabel) +
			    *new TStatusItem("~Alt-G~ Goto", kbAltG,
					     cmshGoto) +
			    *new TStatusItem("~Alt-C~ Delete", kbAltC,
					     cmshDelete) +
			    *new TStatusItem("~Alt-B~ Area", kbAltB,
					     cmshArea) +
			    *new TStatusItem(0, kbAltF3,
					     cmClose) + *new TStatusItem(0,
									 kbF10,
									 cmMenu)));
}

//
// Tile function
//

void VisualApp::tile()
{
    if(deskTop)
	deskTop->tile(deskTop->getExtent());
}

//
// Spreadhead Viewer open function
//

void VisualApp::openSheet()
{
    TView *w = validView(new ShWin());
    if (w != 0)
	if(deskTop)
	    deskTop->insert(w);
#if debug >0
    LOG1("insert ShWin done\n");
#endif
}


//
// "Out of Memory" function ( called by validView() )
//

void VisualApp::outOfMemory()
{
    messageBox("Not enough memory available to complete operation.",
	       mfError | mfOKButton);
}

//
// isTileable() function ( checks a view on desktop is tileable or not )
//

static Boolean isTileable(TView * p, void *)
{
    if ((p->options & ofTileable) != 0)
	return True;
    else
	return False;
}

//
// idle() function ( updates heap and clock views for this program. )
//

void VisualApp::idle()
{
    TProgram::idle();
//  clock->update ();
    usleep(20000);
    if(deskTop)
	message(deskTop, evBroadcast, cmshUpdtSheet, NULL);
    if(deskTop)	
    if (deskTop->firstThat(isTileable, 0) != 0) {
	enableCommand(cmTile);
	enableCommand(cmCascade);
    } else {
	disableCommand(cmTile);
	disableCommand(cmCascade);
    }
}

//
// closeView() function
//
/*
static void closeView(TView * p, void *p1)
{
    message(p, evCommand, cmClose, p1);
}
*/

//
// Menubar initialization.
//

TMenuBar *VisualApp::initMenuBar(TRect r)
{
    if (LangMenu)
	return (LangMenu(r));
    TSubMenu & sub1 =
	*new TSubMenu("~\360~", 0, hcSystem) +
	newLine() +
	*new TMenuItem("~A~bout...", cmAboutCmd, kbNoKey, hcSAbout);

    TSubMenu & sub2 = *new TSubMenu("~F~ile", 0, hcFile) +
//        *new TMenuItem( "~O~pen...", cmOpenCmd, kbF3, hcFOpen, "F3" ) +
	newLine() + *new TMenuItem("E~x~it", cmQuit, kbAltX, hcFExit,
				   "Alt-X");


    TSubMenu & sub3 =
	*new TSubMenu("~W~indows", 0, hcWindows) +
	*new TMenuItem("~R~esize/move", cmResize, kbCtrlF5, hcWSizeMove,
		       "Ctrl-F5") + *new TMenuItem("~Z~oom", cmZoom, kbF5,
						   hcWZoom,
						   "F5") +
	*new TMenuItem("~N~ext", cmNext, kbF6, hcWNext,
		       "F6") + *new TMenuItem("~C~lose", cmClose, kbAltF3,
					      hcWClose,
					      "Alt-F3") +
	*new TMenuItem("~T~ile", cmTile, kbNoKey,
		       hcWTile) + *new TMenuItem("C~a~scade", cmCascade,
						 kbNoKey, hcWCascade);


    TSubMenu & sub4 =
	*new TSubMenu("~S~preadsheet", 0, hcOptions) +
	*new TMenuItem("~O~pen", cmshOpen, kbF3, hcNoContext) +
	*new TMenuItem("~S~ave", cmshSave, kbF2, hcNoContext) +
	*new TMenuItem("~N~ext", cmshNext, '+', hcNoContext) +
	*new TMenuItem("~P~rev", cmshPrev, '-', hcNoContext) +
	*new TMenuItem("Op~t~ions", cmshOptions, kbNoKey, hcNoContext) +
	*new TMenuItem("~F~ind", 0,
		       new TMenu(*new
				 TMenuItem("~N~ext Cell", cmshFindAny,
					   kbNoKey, hcNoContext, NULL,
					   new TMenuItem("~N~ext Formula",
							 cmshFindFor,
							 kbNoKey,
							 hcNoContext, NULL,
							 new
							 TMenuItem
							 ("~N~ext Text",
							  cmshFindFor,
							  kbNoKey,
							  hcNoContext,
							  NULL,
							  new
							  TMenuItem
							  ("~N~ext Number with time",
							   cmshFindTnum,
							   kbNoKey,
							   hcNoContext,
							   NULL,
							   new
							   TMenuItem
							   ("~N~ext Formula with time",
							    cmshFindTfor,
							    kbNoKey,
							    hcNoContext,
							    NULL,
							    new
							    TMenuItem
							    ("~C~oupled cell for HMI element",
							     cmshFindHMICell,
							     kbNoKey,
							     hcNoContext)
							   )
							  )
							 )
					   )
				 )
		       )
		       , hcNoContext) +
//    )+
	*new TMenuItem("~M~ark", 0,
		       new TMenu(*new
				 TMenuItem("~C~ell", cmshMark, kbAltM,
					   hcNoContext, NULL,
					   new TMenuItem("Re~f~erences",
							 cmshMarkRef,
							 kbNoKey,
							 hcNoContext, NULL,
							 new
							 TMenuItem
							 ("~R~ange",
							  cmshMarkArea,
							  kbNoKey,
							  hcNoContext)
					   )
				 )
		       )
		       , hcNoContext) +
	*new TMenuItem("~A~rea", 0,
		       new TMenu(*new
				 TMenuItem("~D~elete", cmshDelArea,
					   kbNoKey, hcNoContext, NULL,
					   new TMenuItem("~C~opy",
							 cmshCopyArea,
							 kbAltK,
							 hcNoContext, NULL,
							 new
							 TMenuItem
							 ("~M~ark",
							  cmshMarkArea,
							  kbNoKey,
							  hcNoContext)
					   )
				 )
		       )
		       , hcNoContext);


    r.b.y = r.a.y + 1;
    TSubMenu & sub5 =
	*new TSubMenu("~V~iew", 0, hcWindows) +
	*new TMenuItem("~D~rivers", cmViewDrivers, kbNoKey, hcNoContext,
		       "") + *new TMenuItem("~R~ecorders", cmViewRecorders,
					    kbNoKey, hcNoContext,
					    "") + *new TMenuItem("~T~ile",
								 cmTile,
								 kbNoKey,
								 hcWTile) +
	*new TMenuItem("C~a~scade", cmCascade, kbNoKey, hcNoContext);

    return (new TMenuBar(r, sub1 + sub2 + sub3 + sub4 + sub5));
}

void VisualApp::handleEvent(TEvent & event)
{
    TApplication::handleEvent(event);

    if (event.what == evCommand) {
	switch (event.message.command) {
	case cmAboutCmd:	//  About Dialog Box
	    aboutDlgBox();
	    break;

	case cmshOpen:		//  Open a spreadsheet
	    openSheet();
	    break;

	case cmTile:		//  Tile current file windows
	    tile();
	    break;

	case cmCascade:	//  Cascade current file windows
	    cascade();
	    break;
	default:
	    if ((event.message.command >= cmViewCommand) &&
		(event.message.command <= cmViewCommand + 200)) {
		ViewersHandleEvent(event);
	    } else
		return;		//  Unknown command
	}
	clearEvent(event);
    }
}



//
// About Box function()
//

void VisualApp::aboutDlgBox()
{
    TDialog *aboutBox = new TDialog(TRect(0, 0, 49, 17), "About");
    aboutBox->
	insert(new TStaticText(TRect(4, 2, 44, 11), sc(scAbout).c_str()));
    aboutBox->
	insert(new TStaticText(TRect(4, 11, 44, 13), sc(scAbout+1).c_str()));	
    aboutBox->
	insert(new TButton(TRect(19, 14, 26, 16), " OK", cmOK, bfDefault));
    aboutBox->options |= ofCentered;
    executeDialog(aboutBox);
}

//
// Cascade function
//

void VisualApp::cascade()
{
    if(deskTop)
	deskTop->cascade(deskTop->getExtent());
}

/*
    Changes:
    
    02/12/2002	text for about box now provided by language module
    10/13/2002	replaced printf(... by logx(... where x is the number of parameters. Log output
		is thus printed to what is defined in log.h;
    10/16/2002	threw out help. it never worked and it is no good idea to create TURBOVISION
	        helpsreens first and HTML pages later.
    02/23/2003	changed include <tvision/...> to "tvision/..."
    09/21/2003  Open a spreadsheet window on startup.
    10/04/2003  optionally load other configuration files. Use config file name as the first 
		argument: visual myconfig.
    01/24/2003  optionally load other configuration files. Use config file name as the first 
		argument: visual myconfig.		
    03/11/2005  added PLCdriver shutdown code.
    08/21/2005  modified PLCdriver shutdown code not to wait endlessly.
*/
